CREATE GENERATOR GEN_ADDIT_FIELDS_ID;

CREATE TABLE ADDIT_FIELDS (
    ID            INTEGER NOT NULL,
    OWNER_FIELDS  INTEGER NOT NULL,
    FIELD_NAME    VARCHAR(128) NOT NULL
);

CREATE GENERATOR GEN_ADDIT_FIELDS_ORG_ID;

CREATE TABLE ADDIT_FIELDS_ORGANIZATION (
    ID               INTEGER NOT NULL,
    ORGANIZATION_ID  INTEGER NOT NULL,
    ADDIT_FIELDS_ID  INTEGER NOT NULL,
    FIELD_VALUE      VARCHAR(256)
);

ALTER TABLE ADDIT_FIELDS ADD CONSTRAINT PK_ADDIT_FIELDS PRIMARY KEY (ID);
ALTER TABLE ADDIT_FIELDS ADD CONSTRAINT UNQ1_ADDIT_FIELDS UNIQUE (OWNER_FIELDS, FIELD_NAME);

ALTER TABLE ADDIT_FIELDS_ORGANIZATION ADD CONSTRAINT PK_ADDIT_FIELDS_ORGANIZATION PRIMARY KEY (ID);
ALTER TABLE ADDIT_FIELDS_ORGANIZATION ADD CONSTRAINT UNQ1_ADDIT_FIELDS_ORGANIZATION UNIQUE (ORGANIZATION_ID, ADDIT_FIELDS_ID);
ALTER TABLE ADDIT_FIELDS_ORGANIZATION ADD CONSTRAINT FK_ADDIT_FIELDS_ORGANIZATION_1 FOREIGN KEY (ORGANIZATION_ID) REFERENCES ORGANIZATION (ID) ON DELETE CASCADE;
ALTER TABLE ADDIT_FIELDS_ORGANIZATION ADD CONSTRAINT FK_ADDIT_FIELDS_ORGANIZATION_2 FOREIGN KEY (ADDIT_FIELDS_ID) REFERENCES ADDIT_FIELDS (ID) ON DELETE CASCADE;


CREATE OR ALTER TRIGGER ADDIT_FIELDS_ORGANIZATION_BI FOR ADDIT_FIELDS_ORGANIZATION
ACTIVE BEFORE INSERT POSITION 0
as
begin

  if (new.id is null)
  then
    new.id = gen_id(GEN_ADDIT_FIELDS_ORG_ID, 1);

end;

CREATE OR ALTER TRIGGER ADDIT_FIELDS_AI FOR ADDIT_FIELDS
ACTIVE AFTER INSERT POSITION 0
AS
    DECLARE VARIABLE ORG_ID INTEGER;
BEGIN
  IF (NEW.OWNER_FIELDS = 0)
  THEN
    BEGIN
        FOR
            SELECT
                ORGANIZATION.ID
            FROM
                ORGANIZATION
            INTO
                :ORG_ID
            DO
                BEGIN
                    INSERT INTO ADDIT_FIELDS_ORGANIZATION
                    (
                        ORGANIZATION_ID,
                        ADDIT_FIELDS_ID
                    )
                    VALUES
                    (
                        :ORG_ID,
                        NEW.ID
                    );
                END
    END
END;


CREATE OR ALTER TRIGGER ADDIT_FIELDS_BI FOR ADDIT_FIELDS
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.id IS NULL)
  THEN
    NEW.id = GEN_ID(gen_addit_fields_id, 1);
END;

CREATE OR ALTER TRIGGER ORGANIZATION_AI FOR ORGANIZATION
ACTIVE AFTER INSERT POSITION 0
AS
    DECLARE VARIABLE F_ID INTEGER;
BEGIN

    FOR
        SELECT
            ADDIT_FIELDS.ID
        FROM
            ADDIT_FIELDS
        WHERE
            ADDIT_FIELDS.OWNER_FIELDS = 0
        INTO
            :F_ID

     DO
        BEGIN
            INSERT INTO ADDIT_FIELDS_ORGANIZATION
            (
                ORGANIZATION_ID,
                ADDIT_FIELDS_ID
            )
            VALUES
            (
                NEW.ID,
                :F_ID
            );
        END
            
END;