ALTER TABLE USERS ADD DIRECTORY VARCHAR(256) CHARACTER SET WIN1251 COLLATE WIN1251;

ALTER TABLE DOCUMENT ADD ATTACH_TYPE INTEGER NOT NULL;

CREATE TABLE CONTACT_DOCUMENT (
    CONTACT_ID   INTEGER NOT NULL,
    DOCUMENT_ID  INTEGER NOT NULL
);

ALTER TABLE CONTACT_DOCUMENT ADD CONSTRAINT UNQ1_CONTACT_DOCUMENT UNIQUE (CONTACT_ID, DOCUMENT_ID);

ALTER TABLE CONTACT_DOCUMENT ADD CONSTRAINT FK_CONTACT_DOCUMENT_1 FOREIGN KEY (CONTACT_ID) REFERENCES CONTACT (ID) ON DELETE CASCADE;

ALTER TABLE CONTACT_DOCUMENT ADD CONSTRAINT FK_CONTACT_DOCUMENT_2 FOREIGN KEY (DOCUMENT_ID) REFERENCES DOCUMENT (ID) ON DELETE CASCADE;

CREATE TABLE EVENT_PROJECTS_DOCUMENT (
    EVENT_PROJECTS_ID  INTEGER NOT NULL,
    DOCUMENT_ID        INTEGER NOT NULL
);

ALTER TABLE EVENT_PROJECTS_DOCUMENT ADD CONSTRAINT UNQ1_EVENT_PROJECTS_DOCUMENT UNIQUE (EVENT_PROJECTS_ID, DOCUMENT_ID);

ALTER TABLE EVENT_PROJECTS_DOCUMENT ADD CONSTRAINT FK_EVENT_PROJECTS_DOCUMENT_1 FOREIGN KEY (EVENT_PROJECTS_ID) REFERENCES EVENT_PROJECTS (ID) ON DELETE CASCADE;

ALTER TABLE EVENT_PROJECTS_DOCUMENT ADD CONSTRAINT FK_EVENT_PROJECTS_DOCUMENT_2 FOREIGN KEY (DOCUMENT_ID) REFERENCES DOCUMENT (ID) ON DELETE CASCADE;

CREATE TABLE ORGANIZATION_DOCUMENT (
    ORGANIZATION_ID  INTEGER NOT NULL,
    DOCUMENT_ID      INTEGER NOT NULL
);

ALTER TABLE ORGANIZATION_DOCUMENT ADD CONSTRAINT UNQ1_ORGANIZATION_DOCUMENT UNIQUE (ORGANIZATION_ID, DOCUMENT_ID);

ALTER TABLE ORGANIZATION_DOCUMENT ADD CONSTRAINT FK_ORGANIZATION_DOCUMENT_1 FOREIGN KEY (ORGANIZATION_ID) REFERENCES ORGANIZATION (ID) ON DELETE CASCADE;

ALTER TABLE ORGANIZATION_DOCUMENT ADD CONSTRAINT FK_ORGANIZATION_DOCUMENT_2 FOREIGN KEY (DOCUMENT_ID) REFERENCES DOCUMENT (ID) ON DELETE CASCADE;

CREATE TABLE USERS_ACCESS (
    USERS_OWNER_ID   INTEGER NOT NULL,
    USERS_ACCESS_ID  INTEGER NOT NULL
);

ALTER TABLE USERS_ACCESS ADD CONSTRAINT UNQ1_USERS_ACCESS UNIQUE (USERS_OWNER_ID, USERS_ACCESS_ID);

ALTER TABLE USERS_ACCESS ADD CONSTRAINT FK_USERS_ACCESS_1 FOREIGN KEY (USERS_OWNER_ID) REFERENCES USERS (ID) ON DELETE CASCADE;

ALTER TABLE USERS_ACCESS ADD CONSTRAINT FK_USERS_ACCESS_2 FOREIGN KEY (USERS_ACCESS_ID) REFERENCES USERS (ID) ON DELETE CASCADE;

CREATE OR ALTER PROCEDURE CORRECT_DOCUMENT 
AS
DECLARE VARIABLE TABLE_ID INTEGER;
DECLARE VARIABLE DOCUMENT_ID INTEGER;
BEGIN
  FOR
    SELECT
      ID,
      DOCUMENT_ID
    FROM
      CONTACT
    WHERE
      DOCUMENT_ID IS NOT NULL
    INTO
      :TABLE_ID,
      :DOCUMENT_ID
  DO
    BEGIN
      INSERT INTO
        CONTACT_DOCUMENT
        (CONTACT_ID, DOCUMENT_ID)
      VALUES
        (:TABLE_ID, :DOCUMENT_ID);
    END

  FOR
    SELECT
      ID,
      DOCUMENT_ID
    FROM
      EVENT_PROJECTS
    WHERE
      DOCUMENT_ID IS NOT NULL
    INTO
      :TABLE_ID,
      :DOCUMENT_ID
  DO
    BEGIN
      INSERT INTO
        EVENT_PROJECTS_DOCUMENT
        (EVENT_PROJECTS_ID, DOCUMENT_ID)
      VALUES
        (:TABLE_ID, :DOCUMENT_ID);
    END

  FOR
    SELECT
      ID,
      DOCUMENT_ID
    FROM
      ORGANIZATION
    WHERE
      DOCUMENT_ID IS NOT NULL
    INTO
      :TABLE_ID,
      :DOCUMENT_ID
  DO
    BEGIN
      INSERT INTO
        ORGANIZATION_DOCUMENT
        (ORGANIZATION_ID, DOCUMENT_ID)
      VALUES
        (:TABLE_ID, :DOCUMENT_ID);
    END
END;

UPDATE DOCUMENT SET ATTACH_TYPE = 0;

EXECUTE PROCEDURE CORRECT_DOCUMENT;

DROP PROCEDURE CORRECT_DOCUMENT;

ALTER TABLE CONTACT DROP CONSTRAINT FK_CONTACT_3;

CREATE OR ALTER procedure ADD_REM_CONTACT (
    USERS_ID integer,
    CONTACT_TYPE_ID integer,
    PERSON_ID integer,
    CONTACT_RESULT_ID integer,
    DT_CONTACT timestamp,
    COMMENTS blob sub_type 0 segment size 80,
    ACCESS integer) 
returns (
    CONTACT_FINAL_ID integer,
    ERROR integer) 
as
BEGIN
/*
ERROR :
    0 - 
    1 -     
*/
    IF (EXISTS (SELECT
                    PERSON.USERS_ID
                FROM
                    PERSON
                WHERE
                (PERSON.ID =:PERSON_ID) AND
                (PERSON.USERS_ID =:USERS_ID)))
    THEN
        BEGIN
            ERROR = 0;
        END
    ELSE
    IF (ACCESS <> 0)
    THEN
        ERROR = 0;
    ELSE
        ERROR = 1;


    IF (ERROR = 0)
    THEN
        BEGIN
            CONTACT_FINAL_ID = GEN_ID(GEN_CONTACT_ID, 1);

            INSERT INTO CONTACT (
                ID,
                PERSON_ID,
                CONTACT_TYPE_ID,
                CONTACT_RESULT_ID,
                USERS_ID,
                DT_CONTACT,
                COMMENTS
            )
            VALUES (
                :CONTACT_FINAL_ID,
                :PERSON_ID,
                :CONTACT_TYPE_ID,
                :CONTACT_RESULT_ID,
                :USERS_ID,
                :DT_CONTACT,
                :COMMENTS
            );
        END

  SUSPEND;
END;

ALTER TABLE CONTACT DROP DOCUMENT_ID;

ALTER TABLE EVENT_PROJECTS DROP CONSTRAINT FK_EVENT_PROJECTS_7;

ALTER TABLE EVENT_PROJECTS DROP DOCUMENT_ID;

ALTER TABLE ORGANIZATION DROP CONSTRAINT FK_ORGANIZATION_7;

CREATE OR ALTER procedure ADD_ORGANIZATION (
    USERS_ID integer,
    NAME varchar(128),
    TEL varchar(64),
    FAX varchar(64),
    EMAIL varchar(64),
    WEB varchar(64),
    HOURS integer,
    REQ_INN varchar(16),
    REQ_KPP varchar(16),
    REQ_RACCOUNT varchar(32),
    REQ_OKPO varchar(16),
    REQ_OKVED varchar(128),
    REQ_BIK varchar(16),
    REQ_BANK varchar(128),
    REQ_KACCOUNT varchar(32),
    ACTIVITY_ID integer,
    DOCUMENT_ID integer,
    PROPERTY_ID integer,
    ORGANIZATION_TYPE integer,
    CHECKED integer,
    EMAIL_SEND integer,
    CONFIG blob sub_type 1 segment size 80,
    COMMENTS blob sub_type 1 segment size 80,
    ACTUAL_POST_INDEX varchar(16),
    LEGAL_POST_INDEX varchar(16),
    POSTAL_POST_INDEX varchar(16),
    ACTUAL_ADDRESS blob sub_type 1 segment size 80,
    LEGAL_ADDRESS blob sub_type 1 segment size 80,
    POSTAL_ADDRESS blob sub_type 1 segment size 80,
    ACTUAL_COUNTRY_ID integer,
    ACTUAL_REGION_ID integer,
    ACTUAL_CITY_ID integer,
    LEGAL_COUNTRY_ID integer,
    LEGAL_REGION_ID integer,
    LEGAL_CITY_ID integer,
    POSTAL_COUNTRY_ID integer,
    POSTAL_REGION_ID integer,
    POSTAL_CITY_ID integer) 
returns (
    ERROR integer,
    ID integer) 
as
declare variable ACTUAL_ADDRESS_ID integer;
declare variable LEGAL_ADDRESS_ID integer;
declare variable POSTAL_ADDRESS_ID integer;
BEGIN
  ERROR = -1;
  ID = -1;
  /****************************************************************************/
  IF ((:ACTUAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :ACTUAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 1; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_COUNTRY_ID = 0)
  THEN
    ACTUAL_COUNTRY_ID = NULL;

  IF ((:LEGAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :LEGAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 2; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_COUNTRY_ID = 0)
  THEN
    LEGAL_COUNTRY_ID = NULL;

  IF ((:POSTAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :POSTAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 3; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_COUNTRY_ID = 0)
  THEN
    POSTAL_COUNTRY_ID = NULL;

  IF ((:ACTUAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :ACTUAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 4; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_REGION_ID = 0)
  THEN
    ACTUAL_REGION_ID = NULL;

  IF ((:LEGAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :LEGAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 5; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_REGION_ID = 0)
  THEN
    LEGAL_REGION_ID = NULL;

  IF ((:POSTAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :POSTAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 6; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_REGION_ID = 0)
  THEN
    POSTAL_REGION_ID = NULL;

  IF ((:ACTUAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :ACTUAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 7; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_CITY_ID = 0)
  THEN
    ACTUAL_CITY_ID = NULL;

  IF ((:LEGAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :LEGAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 8; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_CITY_ID = 0)
  THEN
    LEGAL_CITY_ID = NULL;

  IF ((:POSTAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :POSTAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 9; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_CITY_ID = 0)
  THEN
    POSTAL_CITY_ID = NULL;


 /* IF (EXISTS (SELECT ID FROM ORGANIZATION WHERE NAME = :NAME))
  THEN
    BEGIN
      ERROR = 10;      
      SUSPEND;
      EXIT;
    END
  */
  IF ((:PROPERTY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM PROPERTY WHERE ID = :PROPERTY_ID)))
  THEN
    BEGIN
        ERROR = 11; /*     */
        SUSPEND;
        EXIT;
    END

  IF (:PROPERTY_ID = 0)
  THEN
    PROPERTY_ID = NULL;

  IF ((:ACTIVITY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM ACTIVITY WHERE ID = :ACTIVITY_ID)))
  THEN
    BEGIN
        ERROR = 12; /*   */
        SUSPEND;
        EXIT;
    END

  IF (:ACTIVITY_ID = 0)
  THEN
    PROPERTY_ID = NULL;

  /*******************************************************************/
  ACTUAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:ACTUAL_ADDRESS_ID,
     :ACTUAL_POST_INDEX,
     :ACTUAL_COUNTRY_ID,
     :ACTUAL_REGION_ID,
     :ACTUAL_CITY_ID,
     :ACTUAL_ADDRESS);
  /*******************************************************************/
  LEGAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:LEGAL_ADDRESS_ID,
     :LEGAL_POST_INDEX,
     :LEGAL_COUNTRY_ID,
     :LEGAL_REGION_ID,
     :LEGAL_CITY_ID,
     :LEGAL_ADDRESS);
  /*******************************************************************/
  POSTAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:POSTAL_ADDRESS_ID,
     :POSTAL_POST_INDEX,
     :POSTAL_COUNTRY_ID,
     :POSTAL_REGION_ID,
     :POSTAL_CITY_ID,
     :POSTAL_ADDRESS);
  /*********************************************************************/
  ID = GEN_ID(GEN_ORGANIZATION_ID, 1);
  INSERT INTO ORGANIZATION
    (ID, 
    USERS_ID,
    ACTUAL_ADDRESS_ID,
    LEGAL_ADDRESS_ID,
    POSTAL_ADDRESS_ID,
    NAME,
    TEL,
    FAX,
    EMAIL,
    WEB,
    HOURS,
    REQ_INN,
    REQ_KPP,
    REQ_RACCOUNT,
    REQ_OKPO,
    REQ_OKVED,
    REQ_BIK,
    REQ_BANK,
    REQ_KACCOUNT,
    ACTIVITY_ID,
    REMINDER_COUNT,
    DT_CREATE,
    DT_LASTMODIFY,
    ORGANIZATION_TYPE,
    CHECKED,
    EMAIL_SEND,
    CONFIG,
    COMMENTS,
    PROPERTY_ID)
  VALUES
    (:ID,
    :USERS_ID,
    :ACTUAL_ADDRESS_ID,
    :LEGAL_ADDRESS_ID,
    :POSTAL_ADDRESS_ID,
    :NAME,
    :TEL,
    :FAX,
    :EMAIL,
    :WEB,
    :HOURS,
    :REQ_INN,
    :REQ_KPP,
    :REQ_RACCOUNT,
    :REQ_OKPO,
    :REQ_OKVED,
    :REQ_BIK,
    :REQ_BANK,
    :REQ_KACCOUNT,
    :ACTIVITY_ID,
    0,
    'NOW',
    'NOW',
    :ORGANIZATION_TYPE,
    :CHECKED,
    :EMAIL_SEND,
    :CONFIG,
    :COMMENTS,
    :PROPERTY_ID);
  /****************************************************************************/
  ERROR = 0; /*   */
  SUSPEND;
END;

CREATE OR ALTER procedure EDIT_ORGANIZATION (
    ID integer,
    NAME varchar(128),
    TEL varchar(64),
    FAX varchar(64),
    EMAIL varchar(64),
    WEB varchar(64),
    HOURS integer,
    REQ_INN varchar(16),
    REQ_KPP varchar(16),
    REQ_RACCOUNT varchar(32),
    REQ_OKPO varchar(16),
    REQ_OKVED varchar(128),
    REQ_BIK varchar(16),
    REQ_BANK varchar(128),
    REQ_KACCOUNT varchar(32),
    ACTIVITY_ID integer,
    ORGANIZATION_TYPE integer,
    CHECKED integer,
    EMAIL_SEND integer,
    CONFIG blob sub_type 1 segment size 80,
    COMMENTS blob sub_type 1 segment size 80,
    ACTUAL_POST_INDEX varchar(16),
    LEGAL_POST_INDEX varchar(16),
    POSTAL_POST_INDEX varchar(16),
    ACTUAL_ADDRESS blob sub_type 1 segment size 80,
    LEGAL_ADDRESS blob sub_type 1 segment size 80,
    POSTAL_ADDRESS blob sub_type 1 segment size 80,
    ACTUAL_COUNTRY_ID integer,
    ACTUAL_REGION_ID integer,
    ACTUAL_CITY_ID integer,
    LEGAL_COUNTRY_ID integer,
    LEGAL_REGION_ID integer,
    LEGAL_CITY_ID integer,
    POSTAL_COUNTRY_ID integer,
    POSTAL_REGION_ID integer,
    POSTAL_CITY_ID integer,
    A_ADDRESS_ID integer,
    L_ADDRESS_ID integer,
    P_ADDRESS_ID integer,
    PROPERTY_ID integer,
    DOCUMENT_ID integer) 
returns (
    ERROR integer) 
as
BEGIN
  ERROR = -1;
  /****************************************************************************/
  IF ((:ACTUAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :ACTUAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 1; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_COUNTRY_ID = 0)
  THEN
    ACTUAL_COUNTRY_ID = NULL;

  IF ((:LEGAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :LEGAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 2; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_COUNTRY_ID = 0)
  THEN
    LEGAL_COUNTRY_ID = NULL;

  IF ((:POSTAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :POSTAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 3; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_COUNTRY_ID = 0)
  THEN
    POSTAL_COUNTRY_ID = NULL;

  IF ((:ACTUAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :ACTUAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 4; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_REGION_ID = 0)
  THEN
    ACTUAL_REGION_ID = NULL;

  IF ((:LEGAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :LEGAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 5; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_REGION_ID = 0)
  THEN
    LEGAL_REGION_ID = NULL;

  IF ((:POSTAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :POSTAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 6; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_REGION_ID = 0)
  THEN
    POSTAL_REGION_ID = NULL;

  IF ((:ACTUAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :ACTUAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 7; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_CITY_ID = 0)
  THEN
    ACTUAL_CITY_ID = NULL;

  IF ((:LEGAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :LEGAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 8; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_CITY_ID = 0)
  THEN
    LEGAL_CITY_ID = NULL;

  IF ((:POSTAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :POSTAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 9; /*    */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_CITY_ID = 0)
  THEN
    POSTAL_CITY_ID = NULL;


  IF ((:PROPERTY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM PROPERTY WHERE ID = :PROPERTY_ID)))
  THEN
    BEGIN
        ERROR = 10; /*     */
        SUSPEND;
        EXIT;
    END

  IF (:PROPERTY_ID = 0)
  THEN
    PROPERTY_ID = NULL;

  IF ((:ACTIVITY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM ACTIVITY WHERE ID = :ACTIVITY_ID)))
  THEN
    BEGIN
        ERROR = 11; /*   */
        SUSPEND;
        EXIT;
    END

  IF (:ACTIVITY_ID = 0)
  THEN
    PROPERTY_ID = NULL;

  /*******************************************************************/

  /*******************************************************************/
  UPDATE ADDRESS
    SET
        POST_INDEX = :ACTUAL_POST_INDEX,
        COUNTRY_ID = :ACTUAL_COUNTRY_ID,
        REGION_ID = :ACTUAL_REGION_ID,
        CITY_ID =:ACTUAL_CITY_ID,
        ADDRESS = :ACTUAL_ADDRESS
     WHERE
        ID = :A_ADDRESS_ID;

  /*******************************************************************/
  UPDATE ADDRESS
    SET
        POST_INDEX = :LEGAL_POST_INDEX,
        COUNTRY_ID = :LEGAL_COUNTRY_ID,
        REGION_ID = :LEGAL_REGION_ID,
        CITY_ID =:LEGAL_CITY_ID,
        ADDRESS = :LEGAL_ADDRESS
     WHERE
        ID = :L_ADDRESS_ID;
  /*********************************************************************/
  UPDATE ADDRESS
    SET
        POST_INDEX = :POSTAL_POST_INDEX,
        COUNTRY_ID = :POSTAL_COUNTRY_ID,
        REGION_ID = :POSTAL_REGION_ID,
        CITY_ID =:POSTAL_CITY_ID,
        ADDRESS = :POSTAL_ADDRESS
     WHERE
        ID = :P_ADDRESS_ID;
  /*********************************************************************/
  UPDATE ORGANIZATION
    SET
        NAME =: NAME,
        TEL =: TEL,
        FAX =: FAX,
        EMAIL =: EMAIL,
        WEB =: WEB,
        HOURS =: HOURS,
        REQ_INN =: REQ_INN,
        REQ_KPP =: REQ_KPP,
        REQ_RACCOUNT =: REQ_RACCOUNT,
        REQ_OKPO =: REQ_OKPO,
        REQ_OKVED =: REQ_OKVED,
        REQ_BIK =: REQ_BIK,
        REQ_BANK =: REQ_BANK,
        REQ_KACCOUNT =: REQ_KACCOUNT,
        ACTIVITY_ID =: ACTIVITY_ID,
        PROPERTY_ID =: PROPERTY_ID,
        DT_LASTMODIFY = 'NOW',
        ORGANIZATION_TYPE =:ORGANIZATION_TYPE,
        CHECKED =:CHECKED,
        EMAIL_SEND =: EMAIL_SEND,
        CONFIG =: CONFIG,
        COMMENTS =:COMMENTS
     WHERE
        ID =:ID ;

  /****************************************************************************/
  ERROR = 0; /*   */
  SUSPEND;
END;

CREATE OR ALTER procedure ADD_USER (
    DEPARTMENT_ID integer,
    LOGIN varchar(16),
    PASSW varchar(40),
    FIO varchar(128),
    JOBTITLE varchar(128),
    DIRECTORY varchar(256),
    GRANTS blob sub_type 1 segment size 80,
    CONFIG blob sub_type 1 segment size 80,
    COMMENTS blob sub_type 1 segment size 80) 
returns (
    ID integer) 
as
BEGIN


  ID = GEN_ID(GEN_USERS_ID, 1);

    INSERT INTO USERS
    (ID,
     DEPARTMENT_ID,
     LOGIN,
     PASSW,
     FIO, 
     JOBTITLE,
     GRANTS,
     CONFIG,
     COMMENTS,
     DIRECTORY
     )
  VALUES
    (:ID,
     :DEPARTMENT_ID,
     :LOGIN,
     :PASSW,
     :FIO,
     :JOBTITLE,
     :GRANTS,
     :CONFIG,
     :COMMENTS,
     :DIRECTORY
     );


  SUSPEND;
END;

CREATE OR ALTER procedure LINKS_DOC (
    TABLE_ID integer,
    DOCUMENT_ID integer,
    OPER integer,
    TABLE_INDEX integer) 
as
BEGIN


  IF (TABLE_INDEX = 0)
  THEN
    BEGIN
        IF (OPER = -1)
        THEN
            BEGIN
                DELETE
                FROM
                    ORGANIZATION_DOCUMENT
                WHERE
                    (ORGANIZATION_DOCUMENT.ORGANIZATION_ID =:TABLE_ID) AND
                    (ORGANIZATION_DOCUMENT.DOCUMENT_ID =:DOCUMENT_ID);
            END

        ELSE
        IF (OPER = 1)
        THEN
            BEGIN
                IF (NOT EXISTS(SELECT *
                            FROM
                                ORGANIZATION_DOCUMENT
                            WHERE
                                (ORGANIZATION_DOCUMENT.ORGANIZATION_ID =:TABLE_ID) AND
                                (ORGANIZATION_DOCUMENT.DOCUMENT_ID =:DOCUMENT_ID))
                                )
                THEN
                    BEGIN
                        INSERT INTO ORGANIZATION_DOCUMENT
                        (
                            ORGANIZATION_DOCUMENT.ORGANIZATION_ID,
                            ORGANIZATION_DOCUMENT.DOCUMENT_ID
                        )
                        VALUES
                        (
                            :TABLE_ID,
                            :DOCUMENT_ID
                        )  ;
                            
                    END
            END
    END

  ELSE
  IF (TABLE_INDEX = 1)
  THEN
    BEGIN
        IF (OPER = -1)
        THEN
            BEGIN
                DELETE
                FROM
                    CONTACT_DOCUMENT
                WHERE
                    (CONTACT_DOCUMENT.CONTACT_ID =:TABLE_ID) AND
                    (CONTACT_DOCUMENT.DOCUMENT_ID =:DOCUMENT_ID);
            END

        ELSE
        IF (OPER = 1)
        THEN
            BEGIN
                IF (NOT EXISTS(SELECT *
                            FROM
                                CONTACT_DOCUMENT
                            WHERE
                                (CONTACT_DOCUMENT.CONTACT_ID =:TABLE_ID) AND
                                (CONTACT_DOCUMENT.DOCUMENT_ID =:DOCUMENT_ID))
                                )
                THEN
                    BEGIN
                        INSERT INTO CONTACT_DOCUMENT
                        (
                            CONTACT_DOCUMENT.CONTACT_ID,
                            CONTACT_DOCUMENT.DOCUMENT_ID
                        )
                        VALUES
                        (
                            :TABLE_ID,
                            :DOCUMENT_ID
                        )  ;
                            
                    END
            END
    END
  ELSE
  IF (TABLE_INDEX = 2)
  THEN
    BEGIN
        IF (OPER = -1)
        THEN
            BEGIN
                DELETE
                FROM
                    EVENT_PROJECTS_DOCUMENT
                WHERE
                    (EVENT_PROJECTS_DOCUMENT.EVENT_PROJECTS_ID =:TABLE_ID) AND
                    (EVENT_PROJECTS_DOCUMENT.DOCUMENT_ID =:DOCUMENT_ID);
            END

        ELSE
        IF (OPER = 1)
        THEN
            BEGIN
                IF (NOT EXISTS(SELECT *
                            FROM
                                EVENT_PROJECTS_DOCUMENT
                            WHERE
                                (EVENT_PROJECTS_DOCUMENT.EVENT_PROJECTS_ID =:TABLE_ID) AND
                                (EVENT_PROJECTS_DOCUMENT.DOCUMENT_ID =:DOCUMENT_ID))
                                )
                THEN
                    BEGIN
                        INSERT INTO EVENT_PROJECTS_DOCUMENT
                        (
                            EVENT_PROJECTS_DOCUMENT.EVENT_PROJECTS_ID,
                            EVENT_PROJECTS_DOCUMENT.DOCUMENT_ID
                        )
                        VALUES
                        (
                            :TABLE_ID,
                            :DOCUMENT_ID
                        )  ;
                            
                    END
            END
    END
END;

CREATE OR ALTER procedure ADD_XOR_DEL_USERS_ACCESS (
    USERS_OWNER_ID integer,
    USERS_ACCESS_ID integer,
    OPERATION integer) 
as
BEGIN
 IF ((:OPERATION = 0) AND  (EXISTS (SELECT
                                        USERS_ACCESS.USERS_ACCESS_ID
                                    FROM
                                        USERS_ACCESS
                                    WHERE
                                        (USERS_ACCESS.USERS_OWNER_ID = :USERS_OWNER_ID) AND
                                        (USERS_ACCESS.USERS_ACCESS_ID =:USERS_ACCESS_ID))))
        THEN
            BEGIN
                DELETE
                    FROM
                        USERS_ACCESS
                    WHERE
                        (USERS_ACCESS.USERS_OWNER_ID = :USERS_OWNER_ID) AND
                        (USERS_ACCESS.USERS_ACCESS_ID =:USERS_ACCESS_ID);

            END




 IF ((:OPERATION = 1) AND (NOT EXISTS (SELECT
                                        USERS_ACCESS.USERS_ACCESS_ID
                                    FROM
                                        USERS_ACCESS
                                    WHERE
                                        (USERS_ACCESS.USERS_OWNER_ID = :USERS_OWNER_ID) AND
                                        (USERS_ACCESS.USERS_ACCESS_ID =:USERS_ACCESS_ID))))
        THEN
            BEGIN
                INSERT INTO USERS_ACCESS
                (
                    USERS_ACCESS.USERS_OWNER_ID,
                    USERS_ACCESS.USERS_ACCESS_ID
                 )
                VALUES
                (
                 :USERS_OWNER_ID,
                 :USERS_ACCESS_ID
                );
            END
  SUSPEND;
END;

ALTER TABLE ORGANIZATION DROP DOCUMENT_ID;