/* ----------------------------------------------------------------------------------------------- */
/*   " " : 3.1 RC20 (Final-2)                                           */
/*     : 3.1 RC21                                                */
/*                                                                                                 */
/*   RC21    RC20:                                                           */
/*                                                                                                 */
/* +   " " -      .      */
/*   ,      .                              */
/* +  " ",           */
/*      ,      ().          */
/*             .         */
/* +     " "   .              */
/* +   "  "    :        */
/*   1) , 2)   3)  .                                           */
/* +    ( !): , , ,  ,           */
/*       .                                                  */
/* +   .                                                                  */
/*                                                                                                 */
/*       ,     ,  , */
/*       RC20,   , , RC16.                               */
/*   ,         RC20,          */ 
/*      RC21.                                                                     */
/*                                                                                                 */
/* ----------------------------------------------------------------------------------------------- */
CREATE GENERATOR GEN_TASK_ID;

CREATE TABLE TASK (
    ID                 INTEGER NOT NULL,
    USERS_DIRECTOR_ID  INTEGER NOT NULL,
    USERS_MANAGER_ID   INTEGER NOT NULL,
    DT_CREATE          TIMESTAMP NOT NULL,
    DT_LIMIT           TIMESTAMP NOT NULL,
    DT_FINISH          TIMESTAMP,
    TASK_STATUS        INTEGER NOT NULL,
    COMMENTS_TASK      BLOB SUB_TYPE 1 SEGMENT SIZE 80 NOT NULL,
    COMMENTS_MANAGER   BLOB SUB_TYPE 1 SEGMENT SIZE 80,
    EDIT_USERS_ID      INTEGER
);

ALTER TABLE TASK ADD CONSTRAINT PK_TASK PRIMARY KEY (ID);

ALTER TABLE TASK ADD CONSTRAINT FK_TASK_1 FOREIGN KEY (USERS_DIRECTOR_ID) REFERENCES USERS (ID) ON DELETE CASCADE;
ALTER TABLE TASK ADD CONSTRAINT FK_TASK_2 FOREIGN KEY (USERS_MANAGER_ID) REFERENCES USERS (ID) ON DELETE CASCADE;
ALTER TABLE TASK ADD CONSTRAINT FK_TASK_3 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;


CREATE OR ALTER TRIGGER TASK_BI FOR TASK
ACTIVE BEFORE INSERT POSITION 0
as
begin
  if (new.id is null)
  then
    new.id = gen_id(gen_task_id, 1);

  new.DT_CREATE = 'NOW';
end;

CREATE GENERATOR GEN_ORGANIZATION_GROUP_ID;

CREATE TABLE ORGANIZATION_GROUP (
    ID             INTEGER NOT NULL,
    NAME           VARCHAR(128) NOT NULL,
    EDIT_USERS_ID  INTEGER
);

ALTER TABLE ORGANIZATION_GROUP ADD CONSTRAINT UNQ1_ORGANIZATION_GROUP UNIQUE (NAME);

ALTER TABLE ORGANIZATION_GROUP ADD CONSTRAINT PK_ORGANIZATION_GROUP PRIMARY KEY (ID);

ALTER TABLE ORGANIZATION_GROUP ADD CONSTRAINT FK_ORGANIZATION_GROUP_1 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;


CREATE OR ALTER TRIGGER ORGANIZATION_GROUP_BI FOR ORGANIZATION_GROUP
ACTIVE BEFORE INSERT POSITION 0
as
begin
  if (new.id is null)
  then
    new.id = gen_id(gen_organization_group_id, 1);
end;

ALTER TABLE CONTACT ADD DOCUMENT_COUNT INTEGER;
ALTER TABLE EVENT_PROJECTS ADD DOCUMENT_COUNT INTEGER;

ALTER TABLE ORGANIZATION ADD DOCUMENT_COUNT INTEGER;
ALTER TABLE ORGANIZATION ADD ORGANIZATION_GROUP_ID INTEGER;

ALTER TABLE ORGANIZATION ADD CONSTRAINT FK_ORGANIZATION_OG FOREIGN KEY (ORGANIZATION_GROUP_ID) REFERENCES ORGANIZATION_GROUP (ID) ON DELETE SET NULL;

CREATE OR ALTER procedure CLEAR_EDIT_USERS_ID (
    ID integer,
    IN_OUT integer,
    PROGRAM_ID integer) 
as
BEGIN
  UPDATE REMINDER
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    EVENT_PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE_NUMBERS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    REGION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    COUNTRY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TREND
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PERSON
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    JOB_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACTIVITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TASK
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_RESULT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;
  /*
  0 - 
  1 - 
  */
  IF (IN_OUT = 1)
  THEN
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = NULL
      WHERE
        USERS_ID = :ID;
    END
  ELSE
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = :ID
      WHERE
        ID = :PROGRAM_ID;
    END
END;

CREATE OR ALTER procedure ADD_ORGANIZATION (
    USERS_ID integer,
    NAME varchar(128),
    TEL varchar(64),
    FAX varchar(64),
    EMAIL varchar(64),
    WEB varchar(64),
    HOURS integer,
    REQ_INN varchar(16),
    REQ_KPP varchar(16),
    REQ_RACCOUNT varchar(32),
    REQ_OKPO varchar(16),
    REQ_OKVED varchar(128),
    REQ_BIK varchar(16),
    REQ_BANK varchar(128),
    REQ_KACCOUNT varchar(32),
    ACTIVITY_ID integer,
    PROPERTY_ID integer,
    ORGANIZATION_TYPE integer,
    CHECKED integer,
    EMAIL_SEND integer,
    CONFIG blob sub_type 1 segment size 80,
    COMMENTS blob sub_type 1 segment size 80,
    ACTUAL_POST_INDEX varchar(16),
    LEGAL_POST_INDEX varchar(16),
    POSTAL_POST_INDEX varchar(16),
    ACTUAL_ADDRESS blob sub_type 1 segment size 80,
    LEGAL_ADDRESS blob sub_type 1 segment size 80,
    POSTAL_ADDRESS blob sub_type 1 segment size 80,
    ACTUAL_COUNTRY_ID integer,
    ACTUAL_REGION_ID integer,
    ACTUAL_CITY_ID integer,
    LEGAL_COUNTRY_ID integer,
    LEGAL_REGION_ID integer,
    LEGAL_CITY_ID integer,
    POSTAL_COUNTRY_ID integer,
    POSTAL_REGION_ID integer,
    POSTAL_CITY_ID integer,
    X integer,
    Y integer,
    SUMMA_XY integer,
    ORGANIZATION_GROUP_ID integer) 
returns (
    ERROR integer,
    ID integer) 
as
declare variable ACTUAL_ADDRESS_ID integer;
declare variable LEGAL_ADDRESS_ID integer;
declare variable POSTAL_ADDRESS_ID integer;
BEGIN
  ERROR = -1;
  ID = -1;
  /****************************************************************************/
  IF ((:ACTUAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :ACTUAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 1; /* ?:' "??'?-:'??% ''???> */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_COUNTRY_ID = 0)
  THEN
    ACTUAL_COUNTRY_ID = NULL;

  IF ((:LEGAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :LEGAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 2; /* ?:' ???"?-:'??% ''???> */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_COUNTRY_ID = 0)
  THEN
    LEGAL_COUNTRY_ID = NULL;

  IF ((:POSTAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :POSTAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 3; /* ?:' ??-'?'?% ''???> */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_COUNTRY_ID = 0)
  THEN
    POSTAL_COUNTRY_ID = NULL;

  IF ((:ACTUAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :ACTUAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 4; /* ?:' "??'?-:'??% ?:????? */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_REGION_ID = 0)
  THEN
    ACTUAL_REGION_ID = NULL;

  IF ((:LEGAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :LEGAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 5; /* ?:' ???"?-:'??% ?:????? */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_REGION_ID = 0)
  THEN
    LEGAL_REGION_ID = NULL;

  IF ((:POSTAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :POSTAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 6; /* ?:' ??-'?'???????"? ?:????? */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_REGION_ID = 0)
  THEN
    POSTAL_REGION_ID = NULL;

  IF ((:ACTUAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :ACTUAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 7; /* ?:' "??'?-:'??% ????"? */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_CITY_ID = 0)
  THEN
    ACTUAL_CITY_ID = NULL;

  IF ((:LEGAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :LEGAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 8; /* ?:' ???"?-:'??% ????"? */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_CITY_ID = 0)
  THEN
    LEGAL_CITY_ID = NULL;

  IF ((:POSTAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :POSTAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 9; /* ?:' ??-'?'??? ????"? */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_CITY_ID = 0)
  THEN
    POSTAL_CITY_ID = NULL;


 /* IF (EXISTS (SELECT ID FROM ORGANIZATION WHERE NAME = :NAME))
  THEN
    BEGIN
      ERROR = 10;  :''? ???????-?? ' '???? ??:?:?
      SUSPEND;
      EXIT;
    END
  */
  IF ((:PROPERTY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM PROPERTY WHERE ID = :PROPERTY_ID)))
  THEN
    BEGIN
        ERROR = 11; /* ?:' ???????-????? ???'?'?% "???> */
        SUSPEND;
        EXIT;
    END

  IF (:PROPERTY_ID = 0)
  THEN
    PROPERTY_ID = NULL;

  IF ((:ACTIVITY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM ACTIVITY WHERE ID = :ACTIVITY_ID)))
  THEN
    BEGIN
        ERROR = 12; /* ?:' ??'?'??''? */
        SUSPEND;
        EXIT;
    END

  IF (:ACTIVITY_ID = 0)
  THEN
    PROPERTY_ID = NULL;

  /*******************************************************************/
  ACTUAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:ACTUAL_ADDRESS_ID,
     :ACTUAL_POST_INDEX,
     :ACTUAL_COUNTRY_ID,
     :ACTUAL_REGION_ID,
     :ACTUAL_CITY_ID,
     :ACTUAL_ADDRESS);
  /*******************************************************************/
  LEGAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:LEGAL_ADDRESS_ID,
     :LEGAL_POST_INDEX,
     :LEGAL_COUNTRY_ID,
     :LEGAL_REGION_ID,
     :LEGAL_CITY_ID,
     :LEGAL_ADDRESS);
  /*******************************************************************/
  POSTAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:POSTAL_ADDRESS_ID,
     :POSTAL_POST_INDEX,
     :POSTAL_COUNTRY_ID,
     :POSTAL_REGION_ID,
     :POSTAL_CITY_ID,
     :POSTAL_ADDRESS);
  /*********************************************************************/
  ID = GEN_ID(GEN_ORGANIZATION_ID, 1);
  INSERT INTO ORGANIZATION
    (ID, 
    USERS_ID,
    ACTUAL_ADDRESS_ID,
    LEGAL_ADDRESS_ID,
    POSTAL_ADDRESS_ID,
    NAME,
    TEL,
    FAX,
    EMAIL,
    WEB,
    HOURS,
    REQ_INN,
    REQ_KPP,
    REQ_RACCOUNT,
    REQ_OKPO,
    REQ_OKVED,
    REQ_BIK,
    REQ_BANK,
    REQ_KACCOUNT,
    ACTIVITY_ID,
    REMINDER_COUNT,
    DT_CREATE,
    DT_LASTMODIFY,
    ORGANIZATION_TYPE,
    CHECKED,
    EMAIL_SEND,
    CONFIG,
    COMMENTS,
    PROPERTY_ID,
    X,
    Y,
    SUMMA_XY,
    ORGANIZATION_GROUP_ID)
  VALUES
    (:ID,
    :USERS_ID,
    :ACTUAL_ADDRESS_ID,
    :LEGAL_ADDRESS_ID,
    :POSTAL_ADDRESS_ID,
    :NAME,
    :TEL,
    :FAX,
    :EMAIL,
    :WEB,
    :HOURS,
    :REQ_INN,
    :REQ_KPP,
    :REQ_RACCOUNT,
    :REQ_OKPO,
    :REQ_OKVED,
    :REQ_BIK,
    :REQ_BANK,
    :REQ_KACCOUNT,
    :ACTIVITY_ID,
    0,
    'NOW',
    'NOW',
    :ORGANIZATION_TYPE,
    :CHECKED,
    :EMAIL_SEND,
    :CONFIG,
    :COMMENTS,
    :PROPERTY_ID,
    :X,
    :Y,
    :SUMMA_XY,
    :ORGANIZATION_GROUP_ID);
  /****************************************************************************/
  ERROR = 0; /* '' ??:% */
  SUSPEND;
END;

CREATE OR ALTER procedure EDIT_ORGANIZATION (
    ID integer,
    NAME varchar(128),
    TEL varchar(64),
    FAX varchar(64),
    EMAIL varchar(64),
    WEB varchar(64),
    HOURS integer,
    REQ_INN varchar(16),
    REQ_KPP varchar(16),
    REQ_RACCOUNT varchar(32),
    REQ_OKPO varchar(16),
    REQ_OKVED varchar(128),
    REQ_BIK varchar(16),
    REQ_BANK varchar(128),
    REQ_KACCOUNT varchar(32),
    ACTIVITY_ID integer,
    ORGANIZATION_TYPE integer,
    CHECKED integer,
    EMAIL_SEND integer,
    CONFIG blob sub_type 1 segment size 80,
    COMMENTS blob sub_type 1 segment size 80,
    ACTUAL_POST_INDEX varchar(16),
    LEGAL_POST_INDEX varchar(16),
    POSTAL_POST_INDEX varchar(16),
    ACTUAL_ADDRESS blob sub_type 1 segment size 80,
    LEGAL_ADDRESS blob sub_type 1 segment size 80,
    POSTAL_ADDRESS blob sub_type 1 segment size 80,
    ACTUAL_COUNTRY_ID integer,
    ACTUAL_REGION_ID integer,
    ACTUAL_CITY_ID integer,
    LEGAL_COUNTRY_ID integer,
    LEGAL_REGION_ID integer,
    LEGAL_CITY_ID integer,
    POSTAL_COUNTRY_ID integer,
    POSTAL_REGION_ID integer,
    POSTAL_CITY_ID integer,
    A_ADDRESS_ID integer,
    L_ADDRESS_ID integer,
    P_ADDRESS_ID integer,
    PROPERTY_ID integer,
    X integer,
    Y integer,
    SUMMA_XY integer,
    ORGANIZATION_GROUP_ID integer) 
returns (
    ERROR integer) 
as
BEGIN
  ERROR = -1;
  /****************************************************************************/
  IF ((:ACTUAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :ACTUAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 1; /* ?:' "??'?-:'??% ''???> */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_COUNTRY_ID = 0)
  THEN
    ACTUAL_COUNTRY_ID = NULL;

  IF ((:LEGAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :LEGAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 2; /* ?:' ???"?-:'??% ''???> */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_COUNTRY_ID = 0)
  THEN
    LEGAL_COUNTRY_ID = NULL;

  IF ((:POSTAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :POSTAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 3; /* ?:' ??-'?'?% ''???> */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_COUNTRY_ID = 0)
  THEN
    POSTAL_COUNTRY_ID = NULL;

  IF ((:ACTUAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :ACTUAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 4; /* ?:' "??'?-:'??% ?:????? */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_REGION_ID = 0)
  THEN
    ACTUAL_REGION_ID = NULL;

  IF ((:LEGAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :LEGAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 5; /* ?:' ???"?-:'??% ?:????? */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_REGION_ID = 0)
  THEN
    LEGAL_REGION_ID = NULL;

  IF ((:POSTAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :POSTAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 6; /* ?:' ??-'?'???????"? ?:????? */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_REGION_ID = 0)
  THEN
    POSTAL_REGION_ID = NULL;

  IF ((:ACTUAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :ACTUAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 7; /* ?:' "??'?-:'??% ????"? */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_CITY_ID = 0)
  THEN
    ACTUAL_CITY_ID = NULL;

  IF ((:LEGAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :LEGAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 8; /* ?:' ???"?-:'??% ????"? */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_CITY_ID = 0)
  THEN
    LEGAL_CITY_ID = NULL;

  IF ((:POSTAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :POSTAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 9; /* ?:' ??-'?'??? ????"? */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_CITY_ID = 0)
  THEN
    POSTAL_CITY_ID = NULL;


  IF ((:PROPERTY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM PROPERTY WHERE ID = :PROPERTY_ID)))
  THEN
    BEGIN
        ERROR = 10; /* ?:' ???????-????? ???'?'?% "???> */
        SUSPEND;
        EXIT;
    END

  IF (:PROPERTY_ID = 0)
  THEN
    PROPERTY_ID = NULL;

  IF ((:ACTIVITY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM ACTIVITY WHERE ID = :ACTIVITY_ID)))
  THEN
    BEGIN
        ERROR = 11; /* ?:' ??'?'??''? */
        SUSPEND;
        EXIT;
    END

  IF (:ACTIVITY_ID = 0)
  THEN
    PROPERTY_ID = NULL;

  /*******************************************************************/

  /*******************************************************************/
  UPDATE ADDRESS
    SET
        POST_INDEX = :ACTUAL_POST_INDEX,
        COUNTRY_ID = :ACTUAL_COUNTRY_ID,
        REGION_ID = :ACTUAL_REGION_ID,
        CITY_ID =:ACTUAL_CITY_ID,
        ADDRESS = :ACTUAL_ADDRESS
     WHERE
        ID = :A_ADDRESS_ID;

  /*******************************************************************/
  UPDATE ADDRESS
    SET
        POST_INDEX = :LEGAL_POST_INDEX,
        COUNTRY_ID = :LEGAL_COUNTRY_ID,
        REGION_ID = :LEGAL_REGION_ID,
        CITY_ID =:LEGAL_CITY_ID,
        ADDRESS = :LEGAL_ADDRESS
     WHERE
        ID = :L_ADDRESS_ID;
  /*********************************************************************/
  UPDATE ADDRESS
    SET
        POST_INDEX = :POSTAL_POST_INDEX,
        COUNTRY_ID = :POSTAL_COUNTRY_ID,
        REGION_ID = :POSTAL_REGION_ID,
        CITY_ID =:POSTAL_CITY_ID,
        ADDRESS = :POSTAL_ADDRESS
     WHERE
        ID = :P_ADDRESS_ID;
  /*********************************************************************/
  UPDATE ORGANIZATION
    SET
        NAME =: NAME,
        TEL =: TEL,
        FAX =: FAX,
        EMAIL =: EMAIL,
        WEB =: WEB,
        HOURS =: HOURS,
        REQ_INN =: REQ_INN,
        REQ_KPP =: REQ_KPP,
        REQ_RACCOUNT =: REQ_RACCOUNT,
        REQ_OKPO =: REQ_OKPO,
        REQ_OKVED =: REQ_OKVED,
        REQ_BIK =: REQ_BIK,
        REQ_BANK =: REQ_BANK,
        REQ_KACCOUNT =: REQ_KACCOUNT,
        ACTIVITY_ID =: ACTIVITY_ID,
        PROPERTY_ID =: PROPERTY_ID,
        DT_LASTMODIFY = 'NOW',
        ORGANIZATION_TYPE =:ORGANIZATION_TYPE,
        CHECKED =:CHECKED,
        EMAIL_SEND =: EMAIL_SEND,
        CONFIG =: CONFIG,
        COMMENTS =:COMMENTS,
        X = :X,
        Y = :Y,
        SUMMA_XY = :SUMMA_XY,
        organization_group_id = :organization_group_id
     WHERE
        ID =:ID ;

  /****************************************************************************/
  ERROR = 0; /* '' ??:% */
  SUSPEND;
END;

CREATE OR ALTER TRIGGER CONTACT_BI FOR CONTACT
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_CONTACT_ID, 1);

  NEW.DOCUMENT_COUNT = 0;
END;

CREATE OR ALTER TRIGGER EVENT_PROJECTS_BI FOR EVENT_PROJECTS
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_EVENT_PROJECTS_ID, 1);

  NEW.DOCUMENT_COUNT = 0;
END;

CREATE OR ALTER TRIGGER ORGANIZATION_BI FOR ORGANIZATION
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_ORGANIZATION_ID, 1);

  NEW.DOCUMENT_COUNT = 0;
END;

CREATE OR ALTER TRIGGER CONTACT_DOCUMENT_AI FOR CONTACT_DOCUMENT
ACTIVE AFTER INSERT POSITION 0
AS
begin
  /* Trigger text */
  UPDATE CONTACT
  SET
    DOCUMENT_COUNT = DOCUMENT_COUNT + 1
  WHERE
    ID = NEW.contact_id;
end;

CREATE OR ALTER TRIGGER CONTACT_DOCUMENT_AD FOR CONTACT_DOCUMENT
ACTIVE AFTER DELETE POSITION 0
AS
begin
  /* Trigger text */
  UPDATE CONTACT
  SET
    DOCUMENT_COUNT = DOCUMENT_COUNT - 1
  WHERE
    ID = old.contact_id;
end;

CREATE OR ALTER TRIGGER EVENT_PROJECTS_DOCUMENT_AI FOR EVENT_PROJECTS_DOCUMENT
ACTIVE AFTER INSERT POSITION 0
AS
begin
  /* Trigger text */
  UPDATE EVENT_PROJECTS
  SET
    DOCUMENT_COUNT = DOCUMENT_COUNT + 1
  WHERE
    ID = NEW.EVENT_PROJECTS_id;

end;

CREATE OR ALTER TRIGGER EVENT_PROJECTS_DOCUMENT_AD FOR EVENT_PROJECTS_DOCUMENT
ACTIVE AFTER DELETE POSITION 0
AS
begin
  /* Trigger text */
  UPDATE EVENT_PROJECTS
  SET
    DOCUMENT_COUNT = DOCUMENT_COUNT - 1
  WHERE
    ID = OLD.EVENT_PROJECTS_id;
end;

CREATE OR ALTER TRIGGER ORGANIZATION_DOCUMENT_AI FOR ORGANIZATION_DOCUMENT
ACTIVE AFTER INSERT POSITION 0
AS
begin
  /* Trigger text */
  UPDATE ORGANIZATION
  SET
    DOCUMENT_COUNT = DOCUMENT_COUNT + 1
  WHERE
    ID = NEW.ORGANIZATION_id;
end;

CREATE OR ALTER TRIGGER ORGANIZATION_DOCUMENT_AD FOR ORGANIZATION_DOCUMENT
ACTIVE AFTER DELETE POSITION 0
AS
begin
  /* Trigger text */
  UPDATE ORGANIZATION
  SET
    DOCUMENT_COUNT = DOCUMENT_COUNT - 1
  WHERE
    ID = OLD.ORGANIZATION_id;
end;

UPDATE
  ORGANIZATION
SET
  DOCUMENT_COUNT = (SELECT COUNT(*) FROM ORGANIZATION_DOCUMENT WHERE ORGANIZATION_ID = ORGANIZATION.ID);

UPDATE
  CONTACT
SET
  DOCUMENT_COUNT = (SELECT COUNT(*) FROM CONTACT_DOCUMENT WHERE CONTACT_ID = CONTACT.ID);

UPDATE
  EVENT_PROJECTS
SET
  DOCUMENT_COUNT = (SELECT COUNT(*) FROM EVENT_PROJECTS_DOCUMENT WHERE EVENT_PROJECTS_ID = EVENT_PROJECTS.ID);

