/*........................................................*/
/*   */
/*         "  3.1".     */
/*       RC24   RC21, RC22  RC23.                 */
/*   3  2009 .  */
/*   */
/*........................................................*/
ALTER TABLE ORGANIZATION ADD ARM_COUNT INTEGER;
ALTER TABLE ORGANIZATION ADD PERSON_COUNT INTEGER;
ALTER TABLE ORGANIZATION ADD CONTRACT_SUMMA DOUBLE PRECISION;

CREATE GENERATOR GEN_ORGANIZATION_TYPE_ID;

CREATE TABLE ORGANIZATION_TYPE (
    ID             INTEGER NOT NULL,
    NAME           VARCHAR(64) NOT NULL,
    EDIT_USERS_ID  INTEGER
);

ALTER TABLE ORGANIZATION_TYPE ADD CONSTRAINT UNQ1_ORGANIZATION_TYPE UNIQUE (NAME);
ALTER TABLE ORGANIZATION_TYPE ADD CONSTRAINT PK_ORGANIZATION_TYPE PRIMARY KEY (ID);
ALTER TABLE ORGANIZATION_TYPE ADD CONSTRAINT FK_ORGANIZATION_TYPE_1 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;

CREATE OR ALTER TRIGGER ORGANIZATION_TYPE_BI FOR ORGANIZATION_TYPE
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_ORGANIZATION_TYPE_ID, 1);
END;


CREATE GENERATOR GEN_ORGANIZATION_SOURCE_ID;

CREATE TABLE ORGANIZATION_SOURCE (
    ID             INTEGER NOT NULL,
    NAME           VARCHAR(128) NOT NULL,
    EDIT_USERS_ID  INTEGER
);

ALTER TABLE ORGANIZATION_SOURCE ADD CONSTRAINT UNQ1_ORGANIZATION_SOURCE UNIQUE (NAME);
ALTER TABLE ORGANIZATION_SOURCE ADD CONSTRAINT PK_ORGANIZATION_SOURCE PRIMARY KEY (ID);
ALTER TABLE ORGANIZATION_SOURCE ADD CONSTRAINT FK_ORGANIZATION_SOURCE_1 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;


CREATE OR ALTER TRIGGER ORGANIZATION_SOURCE_BI FOR ORGANIZATION_SOURCE
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_ORGANIZATION_SOURCE_ID, 1);
END;

ALTER TABLE ORGANIZATION ADD ORGANIZATION_SOURCE_ID INTEGER;

alter table ORGANIZATION add constraint FK_ORGANIZATION_OS foreign key (ORGANIZATION_SOURCE_ID) references ORGANIZATION_SOURCE(ID) on delete SET NULL;

CREATE OR ALTER TRIGGER ORGANIZATION_BI FOR ORGANIZATION
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_ORGANIZATION_ID, 1);

  IF (NEW.ARM_COUNT IS NULL)
  THEN
    NEW.ARM_COUNT = 0;

  IF (NEW.PERSON_COUNT IS NULL)
  THEN
    NEW.PERSON_COUNT = 0;

  IF (NEW.CONTRACT_SUMMA IS NULL)
  THEN
    NEW.CONTRACT_SUMMA = 0;

  NEW.DOCUMENT_COUNT = 0;
END;

UPDATE
  ORGANIZATION
SET
  ARM_COUNT = 0,
  PERSON_COUNT = 0,
  CONTRACT_SUMMA = 0;

CREATE OR ALTER procedure ADD_ORGANIZATION (
    USERS_ID integer,
    NAME varchar(128),
    TEL varchar(64),
    FAX varchar(64),
    EMAIL varchar(64),
    WEB varchar(64),
    HOURS integer,
    REQ_INN varchar(16),
    REQ_KPP varchar(16),
    REQ_RACCOUNT varchar(32),
    REQ_OKPO varchar(16),
    REQ_OKVED varchar(128),
    REQ_BIK varchar(16),
    REQ_BANK varchar(128),
    REQ_KACCOUNT varchar(32),
    ACTIVITY_ID integer,
    PROPERTY_ID integer,
    ORGANIZATION_TYPE integer,
    CHECKED integer,
    EMAIL_SEND integer,
    CONFIG blob sub_type 1 segment size 80,
    COMMENTS blob sub_type 1 segment size 80,
    ACTUAL_POST_INDEX varchar(16),
    LEGAL_POST_INDEX varchar(16),
    POSTAL_POST_INDEX varchar(16),
    ACTUAL_ADDRESS blob sub_type 1 segment size 80,
    LEGAL_ADDRESS blob sub_type 1 segment size 80,
    POSTAL_ADDRESS blob sub_type 1 segment size 80,
    ACTUAL_COUNTRY_ID integer,
    ACTUAL_REGION_ID integer,
    ACTUAL_CITY_ID integer,
    LEGAL_COUNTRY_ID integer,
    LEGAL_REGION_ID integer,
    LEGAL_CITY_ID integer,
    POSTAL_COUNTRY_ID integer,
    POSTAL_REGION_ID integer,
    POSTAL_CITY_ID integer,
    X integer,
    Y integer,
    SUMMA_XY integer,
    ORGANIZATION_GROUP_ID integer,
    ARM_COUNT integer,
    PERSON_COUNT integer,
    CONTRACT_SUMMA double precision,
    ORGANIZATION_SOURCE_ID INTEGER)
returns (
    ERROR integer,
    ID integer)
as
declare variable ACTUAL_ADDRESS_ID integer;
declare variable LEGAL_ADDRESS_ID integer;
declare variable POSTAL_ADDRESS_ID integer;
BEGIN
  ERROR = -1;
  ID = -1;
  /****************************************************************************/
  IF ((:ACTUAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :ACTUAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 1; /* ?:' "??'?-:'??% ''???> */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_COUNTRY_ID = 0)
  THEN
    ACTUAL_COUNTRY_ID = NULL;

  IF ((:LEGAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :LEGAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 2; /* ?:' ???"?-:'??% ''???> */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_COUNTRY_ID = 0)
  THEN
    LEGAL_COUNTRY_ID = NULL;

  IF ((:POSTAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :POSTAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 3; /* ?:' ??-'?'?% ''???> */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_COUNTRY_ID = 0)
  THEN
    POSTAL_COUNTRY_ID = NULL;

  IF ((:ACTUAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :ACTUAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 4; /* ?:' "??'?-:'??% ?:????? */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_REGION_ID = 0)
  THEN
    ACTUAL_REGION_ID = NULL;

  IF ((:LEGAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :LEGAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 5; /* ?:' ???"?-:'??% ?:????? */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_REGION_ID = 0)
  THEN
    LEGAL_REGION_ID = NULL;

  IF ((:POSTAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :POSTAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 6; /* ?:' ??-'?'???????"? ?:????? */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_REGION_ID = 0)
  THEN
    POSTAL_REGION_ID = NULL;

  IF ((:ACTUAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :ACTUAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 7; /* ?:' "??'?-:'??% ????"? */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_CITY_ID = 0)
  THEN
    ACTUAL_CITY_ID = NULL;

  IF ((:LEGAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :LEGAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 8; /* ?:' ???"?-:'??% ????"? */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_CITY_ID = 0)
  THEN
    LEGAL_CITY_ID = NULL;

  IF ((:POSTAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :POSTAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 9; /* ?:' ??-'?'??? ????"? */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_CITY_ID = 0)
  THEN
    POSTAL_CITY_ID = NULL;


 /* IF (EXISTS (SELECT ID FROM ORGANIZATION WHERE NAME = :NAME))
  THEN
    BEGIN
      ERROR = 10;  :''? ????????-?? ' '???? ??:?:?
      SUSPEND;
      EXIT;
    END
  */
  IF ((:PROPERTY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM PROPERTY WHERE ID = :PROPERTY_ID)))
  THEN
    BEGIN
        ERROR = 11; /* ?:' ????????-????? ???'?'?% "???> */
        SUSPEND;
        EXIT;
    END

  IF (:PROPERTY_ID = 0)
  THEN
    PROPERTY_ID = NULL;

  IF ((:ACTIVITY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM ACTIVITY WHERE ID = :ACTIVITY_ID)))
  THEN
    BEGIN
        ERROR = 12; /* ?:' ??'?'??''? */
        SUSPEND;
        EXIT;
    END

  IF (:ACTIVITY_ID = 0)
  THEN
    ACTIVITY_ID = NULL;

  IF (:ORGANIZATION_SOURCE_ID = 0)
  THEN
    ORGANIZATION_SOURCE_ID = NULL;

  /*******************************************************************/
  ACTUAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:ACTUAL_ADDRESS_ID,
     :ACTUAL_POST_INDEX,
     :ACTUAL_COUNTRY_ID,
     :ACTUAL_REGION_ID,
     :ACTUAL_CITY_ID,
     :ACTUAL_ADDRESS);
  /*******************************************************************/
  LEGAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:LEGAL_ADDRESS_ID,
     :LEGAL_POST_INDEX,
     :LEGAL_COUNTRY_ID,
     :LEGAL_REGION_ID,
     :LEGAL_CITY_ID,
     :LEGAL_ADDRESS);
  /*******************************************************************/
  POSTAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:POSTAL_ADDRESS_ID,
     :POSTAL_POST_INDEX,
     :POSTAL_COUNTRY_ID,
     :POSTAL_REGION_ID,
     :POSTAL_CITY_ID,
     :POSTAL_ADDRESS);
  /*********************************************************************/
  ID = GEN_ID(GEN_ORGANIZATION_ID, 1);
  INSERT INTO ORGANIZATION
    (ID, 
    USERS_ID,
    ACTUAL_ADDRESS_ID,
    LEGAL_ADDRESS_ID,
    POSTAL_ADDRESS_ID,
    NAME,
    TEL,
    FAX,
    EMAIL,
    WEB,
    HOURS,
    REQ_INN,
    REQ_KPP,
    REQ_RACCOUNT,
    REQ_OKPO,
    REQ_OKVED,
    REQ_BIK,
    REQ_BANK,
    REQ_KACCOUNT,
    ACTIVITY_ID,
    REMINDER_COUNT,
    DT_CREATE,
    DT_LASTMODIFY,
    ORGANIZATION_TYPE,
    CHECKED,
    EMAIL_SEND,
    CONFIG,
    COMMENTS,
    PROPERTY_ID,
    X,
    Y,
    SUMMA_XY,
    ORGANIZATION_GROUP_ID,
    ARM_COUNT,
    PERSON_COUNT,
    CONTRACT_SUMMA,
    ORGANIZATION_SOURCE_ID)
  VALUES
    (:ID,
    :USERS_ID,
    :ACTUAL_ADDRESS_ID,
    :LEGAL_ADDRESS_ID,
    :POSTAL_ADDRESS_ID,
    :NAME,
    :TEL,
    :FAX,
    :EMAIL,
    :WEB,
    :HOURS,
    :REQ_INN,
    :REQ_KPP,
    :REQ_RACCOUNT,
    :REQ_OKPO,
    :REQ_OKVED,
    :REQ_BIK,
    :REQ_BANK,
    :REQ_KACCOUNT,
    :ACTIVITY_ID,
    0,
    'NOW',
    'NOW',
    :ORGANIZATION_TYPE,
    :CHECKED,
    :EMAIL_SEND,
    :CONFIG,
    :COMMENTS,
    :PROPERTY_ID,
    :X,
    :Y,
    :SUMMA_XY,
    :ORGANIZATION_GROUP_ID,
    :ARM_COUNT,
    :PERSON_COUNT,
    :CONTRACT_SUMMA,
    :ORGANIZATION_SOURCE_ID);
  /****************************************************************************/
  ERROR = 0; /* '' ??:% */
  SUSPEND;
END;

CREATE OR ALTER procedure EDIT_ORGANIZATION (
    ID integer,
    NAME varchar(128),
    TEL varchar(64),
    FAX varchar(64),
    EMAIL varchar(64),
    WEB varchar(64),
    HOURS integer,
    REQ_INN varchar(16),
    REQ_KPP varchar(16),
    REQ_RACCOUNT varchar(32),
    REQ_OKPO varchar(16),
    REQ_OKVED varchar(128),
    REQ_BIK varchar(16),
    REQ_BANK varchar(128),
    REQ_KACCOUNT varchar(32),
    ACTIVITY_ID integer,
    ORGANIZATION_TYPE integer,
    CHECKED integer,
    EMAIL_SEND integer,
    CONFIG blob sub_type 1 segment size 80,
    COMMENTS blob sub_type 1 segment size 80,
    ACTUAL_POST_INDEX varchar(16),
    LEGAL_POST_INDEX varchar(16),
    POSTAL_POST_INDEX varchar(16),
    ACTUAL_ADDRESS blob sub_type 1 segment size 80,
    LEGAL_ADDRESS blob sub_type 1 segment size 80,
    POSTAL_ADDRESS blob sub_type 1 segment size 80,
    ACTUAL_COUNTRY_ID integer,
    ACTUAL_REGION_ID integer,
    ACTUAL_CITY_ID integer,
    LEGAL_COUNTRY_ID integer,
    LEGAL_REGION_ID integer,
    LEGAL_CITY_ID integer,
    POSTAL_COUNTRY_ID integer,
    POSTAL_REGION_ID integer,
    POSTAL_CITY_ID integer,
    A_ADDRESS_ID integer,
    L_ADDRESS_ID integer,
    P_ADDRESS_ID integer,
    PROPERTY_ID integer,
    X integer,
    Y integer,
    SUMMA_XY integer,
    ORGANIZATION_GROUP_ID integer,
    ARM_COUNT integer,
    PERSON_COUNT integer,
    CONTRACT_SUMMA double precision,
    ORGANIZATION_SOURCE_ID INTEGER)
returns (
    ERROR integer)
as
BEGIN
  ERROR = -1;
  /****************************************************************************/
  IF ((:ACTUAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :ACTUAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 1; /* ?:' "??'?-:'??% ''???> */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_COUNTRY_ID = 0)
  THEN
    ACTUAL_COUNTRY_ID = NULL;

  IF ((:LEGAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :LEGAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 2; /* ?:' ???"?-:'??% ''???> */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_COUNTRY_ID = 0)
  THEN
    LEGAL_COUNTRY_ID = NULL;

  IF ((:POSTAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :POSTAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 3; /* ?:' ??-'?'?% ''???> */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_COUNTRY_ID = 0)
  THEN
    POSTAL_COUNTRY_ID = NULL;

  IF ((:ACTUAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :ACTUAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 4; /* ?:' "??'?-:'??% ?:????? */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_REGION_ID = 0)
  THEN
    ACTUAL_REGION_ID = NULL;

  IF ((:LEGAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :LEGAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 5; /* ?:' ???"?-:'??% ?:????? */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_REGION_ID = 0)
  THEN
    LEGAL_REGION_ID = NULL;

  IF ((:POSTAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :POSTAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 6; /* ?:' ??-'?'???????"? ?:????? */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_REGION_ID = 0)
  THEN
    POSTAL_REGION_ID = NULL;

  IF ((:ACTUAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :ACTUAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 7; /* ?:' "??'?-:'??% ????"? */
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_CITY_ID = 0)
  THEN
    ACTUAL_CITY_ID = NULL;

  IF ((:LEGAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :LEGAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 8; /* ?:' ???"?-:'??% ????"? */
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_CITY_ID = 0)
  THEN
    LEGAL_CITY_ID = NULL;

  IF ((:POSTAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :POSTAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 9; /* ?:' ??-'?'??? ????"? */
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_CITY_ID = 0)
  THEN
    POSTAL_CITY_ID = NULL;


  IF ((:PROPERTY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM PROPERTY WHERE ID = :PROPERTY_ID)))
  THEN
    BEGIN
        ERROR = 10; /* ?:' ????????-????? ???'?'?% "???> */
        SUSPEND;
        EXIT;
    END

  IF (:PROPERTY_ID = 0)
  THEN
    PROPERTY_ID = NULL;

  IF ((:ACTIVITY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM ACTIVITY WHERE ID = :ACTIVITY_ID)))
  THEN
    BEGIN
        ERROR = 11; /* ?:' ??'?'??''? */
        SUSPEND;
        EXIT;
    END

  IF (:ACTIVITY_ID = 0)
  THEN
    ACTIVITY_ID = NULL;

  IF (:ORGANIZATION_SOURCE_ID = 0)
  THEN
    ORGANIZATION_SOURCE_ID = NULL;

  /*******************************************************************/

  /*******************************************************************/
  UPDATE ADDRESS
    SET
        POST_INDEX = :ACTUAL_POST_INDEX,
        COUNTRY_ID = :ACTUAL_COUNTRY_ID,
        REGION_ID = :ACTUAL_REGION_ID,
        CITY_ID =:ACTUAL_CITY_ID,
        ADDRESS = :ACTUAL_ADDRESS
     WHERE
        ID = :A_ADDRESS_ID;

  /*******************************************************************/
  UPDATE ADDRESS
    SET
        POST_INDEX = :LEGAL_POST_INDEX,
        COUNTRY_ID = :LEGAL_COUNTRY_ID,
        REGION_ID = :LEGAL_REGION_ID,
        CITY_ID =:LEGAL_CITY_ID,
        ADDRESS = :LEGAL_ADDRESS
     WHERE
        ID = :L_ADDRESS_ID;
  /*********************************************************************/
  UPDATE ADDRESS
    SET
        POST_INDEX = :POSTAL_POST_INDEX,
        COUNTRY_ID = :POSTAL_COUNTRY_ID,
        REGION_ID = :POSTAL_REGION_ID,
        CITY_ID =:POSTAL_CITY_ID,
        ADDRESS = :POSTAL_ADDRESS
     WHERE
        ID = :P_ADDRESS_ID;
  /*********************************************************************/
  UPDATE ORGANIZATION
    SET
        NAME =: NAME,
        TEL =: TEL,
        FAX =: FAX,
        EMAIL =: EMAIL,
        WEB =: WEB,
        HOURS =: HOURS,
        REQ_INN =: REQ_INN,
        REQ_KPP =: REQ_KPP,
        REQ_RACCOUNT =: REQ_RACCOUNT,
        REQ_OKPO =: REQ_OKPO,
        REQ_OKVED =: REQ_OKVED,
        REQ_BIK =: REQ_BIK,
        REQ_BANK =: REQ_BANK,
        REQ_KACCOUNT =: REQ_KACCOUNT,
        ACTIVITY_ID =: ACTIVITY_ID,
        PROPERTY_ID =: PROPERTY_ID,
        DT_LASTMODIFY = 'NOW',
        ORGANIZATION_TYPE =:ORGANIZATION_TYPE,
        CHECKED =:CHECKED,
        EMAIL_SEND =: EMAIL_SEND,
        CONFIG =: CONFIG,
        COMMENTS =:COMMENTS,
        X = :X,
        Y = :Y,
        SUMMA_XY = :SUMMA_XY,
        organization_group_id = :organization_group_id,
        ARM_COUNT = :ARM_COUNT,
        PERSON_COUNT = :PERSON_COUNT,
        CONTRACT_SUMMA = :CONTRACT_SUMMA,
        ORGANIZATION_SOURCE_ID = :ORGANIZATION_SOURCE_ID
     WHERE
        ID =:ID ;

  /****************************************************************************/
  ERROR = 0; /* '' ??:% */
  SUSPEND;
END;

CREATE OR ALTER procedure CLEAR_EDIT_USERS_ID (
    ID integer,
    IN_OUT integer,
    PROGRAM_ID integer)
as
BEGIN
  UPDATE REMINDER
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    EVENT_PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE_NUMBERS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    REGION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    COUNTRY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TREND
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PERSON
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    JOB_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACTIVITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TASK
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_RESULT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_SOURCE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;
  /*
  0 - '?"
  1 - '>?"
  */
  IF (IN_OUT = 1)
  THEN
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = NULL
      WHERE
        USERS_ID = :ID;
    END
  ELSE
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = :ID
      WHERE
        ID = :PROGRAM_ID;
    END
END;

DELETE FROM ORGANIZATION_TYPE;

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(0, '  : ');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(1, '  : ');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(2, '  :   ');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(3, ' : VIP-');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(4, ' :  ');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(5, ' :  ');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(6, ' :  ');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(7, '  ');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(8, '  ');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(9, '  ');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(10, '');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(11, '');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(12, ' ');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(13, ' ');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(14, ' ');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(15, '');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(16, '');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(17, ' ');

INSERT INTO
  ORGANIZATION_TYPE
  (ID, NAME)
  VALUES(18, ' ');

ALTER SEQUENCE GEN_ORGANIZATION_TYPE_ID RESTART WITH 18;

alter table ORGANIZATION add constraint FK_ORGANIZATION_OT foreign key (ORGANIZATION_TYPE) references ORGANIZATION_TYPE(ID);

CREATE OR ALTER TRIGGER CONTACT_AU FOR CONTACT
ACTIVE AFTER UPDATE POSITION 0
AS
    DECLARE VARIABLE DAYSLEFT INTEGER;
    DECLARE VARIABLE USERS_ID INTEGER;
    DECLARE VARIABLE REMINDER BLOB SUB_TYPE 1 SEGMENT SIZE 80;
    DECLARE VARIABLE DT_REMINDER TIMESTAMP;
BEGIN
  IF (NEW.CONTACT_RESULT_ID <> OLD.CONTACT_RESULT_ID)
  THEN
    BEGIN

  FOR
    SELECT
        CONTACT_RESULT_AUTO.USERS_ID,
        CONTACT_RESULT_AUTO.DAYSLEFT,
        CONTACT_RESULT_AUTO.REMINDER
    FROM
        CONTACT_RESULT_AUTO
    WHERE
        CONTACT_RESULT_AUTO.CONTACT_RESULT_ID = NEW.CONTACT_RESULT_ID
  INTO
    :USERS_ID,
    :DAYSLEFT,
    :REMINDER
  DO
    BEGIN
        DT_REMINDER = CURRENT_DATE + DAYSLEFT;

        IF (USERS_ID IS NULL)
        THEN
            USERS_ID = NEW.USERS_ID;

        INSERT INTO REMINDER
        (
            REMINDER.PERSON_ID,
            REMINDER.FROM_USERS_ID,
            REMINDER.TO_USERS_ID,
            REMINDER.CONTACT_START_ID,
            REMINDER.CONTACT_FINAL_ID,
            REMINDER.DT_REMINDER,
            REMINDER.REMINDER,
            REMINDER.STATUS
        )
        VALUES
        (
            NEW.PERSON_ID,
            NEW.USERS_ID,
            :USERS_ID,
            NEW.ID,
            NULL, 
            :DT_REMINDER,
            :REMINDER, 
            0
        );

    END

    END
END;

CREATE OR ALTER procedure EXCHANGE_ORG (
    VAR_OLD_USERS_ID integer,
    VAR_NEW_USERS_ID integer,
    VAR_ORGAN_ID integer)
returns (
    VAR_PERS_COUNT integer,
    VAR_CONT_COUNT integer)
as
BEGIN

VAR_PERS_COUNT = 0;
VAR_CONT_COUNT = 0;

/*----------------------------------------------------------------------------*/
/*                                                       */
/*----------------------------------------------------------------------------*/
  UPDATE ORGANIZATION
  SET
    USERS_ID = :VAR_NEW_USERS_ID
  WHERE
    (USERS_ID = :VAR_OLD_USERS_ID)
    AND
    (ID = :VAR_ORGAN_ID);

/*----------------------------------------------------------------------------*/
/*                                                            */
/*----------------------------------------------------------------------------*/
 SELECT
    COUNT (P.ID)
  FROM
    PERSON P
  WHERE
    (P.USERS_ID = :VAR_OLD_USERS_ID)
    AND
    (P.ORGANIZATION_ID = :VAR_ORGAN_ID)
  INTO
    :VAR_PERS_COUNT;

  UPDATE PERSON
  SET
    USERS_ID = :VAR_NEW_USERS_ID
  WHERE
    (USERS_ID = :VAR_OLD_USERS_ID)
    AND
    (ORGANIZATION_ID = :VAR_ORGAN_ID);

  UPDATE AGREEMENT
  SET
    USERS_ID = :VAR_NEW_USERS_ID
  WHERE
    (USERS_ID = :VAR_OLD_USERS_ID)
    AND
    (ORGANIZATION_ID = :VAR_ORGAN_ID);

/*----------------------------------------------------------------------------*/
/*                                                          */
/*----------------------------------------------------------------------------*/
 SELECT
    COUNT (C.ID)
  FROM
    (CONTACT C LEFT JOIN PERSON P ON C.PERSON_ID = P.ID)
  WHERE
    (C.USERS_ID = :VAR_OLD_USERS_ID)
    AND
    (P.ORGANIZATION_ID = :VAR_ORGAN_ID)
  INTO
    :VAR_CONT_COUNT;

  UPDATE CONTACT
  SET
    USERS_ID = :VAR_NEW_USERS_ID
  WHERE
    (USERS_ID = :VAR_OLD_USERS_ID)
    AND
    (PERSON_ID IN (SELECT
                     ID
                   FROM
                     PERSON
                   WHERE
                     ORGANIZATION_ID = :VAR_ORGAN_ID));

  SUSPEND;

END;

CREATE OR ALTER procedure EXCHANGE_HINGING (
    VAR_USERS_ID integer)
as
declare variable VAR_ORGAN_ID integer;
declare variable VAR_ORGAN_USERS_ID integer;
declare variable VAR_PROJ_ID integer;
declare variable VAR_PROJ_USERS_ID integer;
BEGIN

/*----------------------------------------------------------------------------*/
/*                                         */
/*----------------------------------------------------------------------------*/
FOR
  SELECT
    ID,
    USERS_ID
  FROM
    ORGANIZATION
  WHERE
    (USERS_ID <> :VAR_USERS_ID)
  INTO
    :VAR_ORGAN_ID,
    :VAR_ORGAN_USERS_ID
DO
  BEGIN

    /*------------------------------------------------------------------------*/
    /*                                                      */
    /*------------------------------------------------------------------------*/
    UPDATE CONTACT
    SET
      USERS_ID = :VAR_ORGAN_USERS_ID
    WHERE
      (USERS_ID = :VAR_USERS_ID)
      AND
      (PERSON_ID IN (SELECT
                       ID
                     FROM
                       PERSON
                     WHERE
                       (ORGANIZATION_ID = :VAR_ORGAN_ID)));

    /*------------------------------------------------------------------------*/
    /*                                                        */
    /*------------------------------------------------------------------------*/
    UPDATE PERSON
    SET
      USERS_ID = :VAR_ORGAN_USERS_ID
    WHERE
      (USERS_ID = :VAR_USERS_ID)
      AND
      (ORGANIZATION_ID = :VAR_ORGAN_ID);

    UPDATE AGREEMENT
    SET
      USERS_ID = :VAR_ORGAN_USERS_ID
    WHERE
      (USERS_ID = :VAR_USERS_ID)
      AND
      (ORGANIZATION_ID = :VAR_ORGAN_ID);

  END

/*----------------------------------------------------------------------------*/
/*                                             */
/*----------------------------------------------------------------------------*/
FOR
  SELECT
    ID,
    USERS_ID
  FROM
    PROJECTS
  WHERE
    (USERS_ID <> :VAR_USERS_ID)
  INTO
    :VAR_PROJ_ID,
    :VAR_PROJ_USERS_ID
DO
  BEGIN

    /*------------------------------------------------------------------------*/
    /*                                              */
    /*------------------------------------------------------------------------*/
    UPDATE EVENT_PROJECTS
    SET
      USERS_ID = :VAR_PROJ_USERS_ID
    WHERE
      (USERS_ID = :VAR_USERS_ID)
      AND
      (PROJECTS_ID = :VAR_PROJ_ID);

  END

END;