/*----------------------------- procedure EXCHANGE_ORG -----------------------------------------------*/

CREATE OR ALTER procedure EXCHANGE_ORG (
    VAR_OLD_USERS_ID integer,
    VAR_NEW_USERS_ID integer,
    VAR_ORGAN_ID integer,
    VAR_DOWNLEVEL integer)
returns (
    VAR_PERS_COUNT integer,
    VAR_CONT_COUNT integer)
as
BEGIN

VAR_PERS_COUNT = 0;
VAR_CONT_COUNT = 0;

  UPDATE ORGANIZATION
  SET
    USERS_ID = :VAR_NEW_USERS_ID
  WHERE
    (USERS_ID = :VAR_OLD_USERS_ID)
    AND
    (ID = :VAR_ORGAN_ID);

  IF (VAR_DOWNLEVEL = 1)
  THEN
    SUSPEND;

 SELECT
    COUNT (P.ID)
  FROM
    PERSON P
  WHERE
    (P.USERS_ID = :VAR_OLD_USERS_ID)
    AND
    (P.ORGANIZATION_ID = :VAR_ORGAN_ID)
  INTO
    :VAR_PERS_COUNT;

  UPDATE PERSON
  SET
    USERS_ID = :VAR_NEW_USERS_ID
  WHERE
    (USERS_ID = :VAR_OLD_USERS_ID)
    AND
    (ORGANIZATION_ID = :VAR_ORGAN_ID);

  UPDATE AGREEMENT
  SET
    USERS_ID = :VAR_NEW_USERS_ID
  WHERE
    (USERS_ID = :VAR_OLD_USERS_ID)
    AND
    (ORGANIZATION_ID = :VAR_ORGAN_ID);

 SELECT
    COUNT (C.ID)
  FROM
    (CONTACT C LEFT JOIN PERSON P ON C.PERSON_ID = P.ID)
  WHERE
    (C.USERS_ID = :VAR_OLD_USERS_ID)
    AND
    (P.ORGANIZATION_ID = :VAR_ORGAN_ID)
  INTO
    :VAR_CONT_COUNT;

  UPDATE CONTACT
  SET
    USERS_ID = :VAR_NEW_USERS_ID
  WHERE
    (USERS_ID = :VAR_OLD_USERS_ID)
    AND
    (PERSON_ID IN (SELECT
                     ID
                   FROM
                     PERSON
                   WHERE
                     ORGANIZATION_ID = :VAR_ORGAN_ID));

  SUSPEND;

END;

/*----------------------------- procedure EXCHANGE_PROJ -----------------------------------------------*/

CREATE OR ALTER procedure EXCHANGE_PROJ (
    VAR_OLD_USERS_ID integer,
    VAR_NEW_USERS_ID integer,
    VAR_PROJ_ID integer,
    VAR_DOWNLEVEL integer)
returns (
    VAR_PEVENT_COUNT integer)
as
BEGIN

VAR_PEVENT_COUNT = 0;

  UPDATE PROJECTS
  SET
    USERS_ID = :VAR_NEW_USERS_ID
  WHERE
    (USERS_ID = :VAR_OLD_USERS_ID)
    AND
    (ID = :VAR_PROJ_ID);

  IF (VAR_DOWNLEVEL = 1)
  THEN
    SUSPEND;

 SELECT
    COUNT (EP.ID)
  FROM
    EVENT_PROJECTS EP
  WHERE
    (EP.USERS_ID = :VAR_OLD_USERS_ID)
    AND
    (EP.PROJECTS_ID = :VAR_PROJ_ID)
  INTO
    :VAR_PEVENT_COUNT;

  UPDATE EVENT_PROJECTS
  SET
    USERS_ID = :VAR_NEW_USERS_ID
  WHERE
    (USERS_ID = :VAR_OLD_USERS_ID)
    AND
    (PROJECTS_ID = :VAR_PROJ_ID);

  SUSPEND;

END;

/*---------------------------- procedure SELECT_BIRTHDAY ------------------------------------------------*/

CREATE OR ALTER procedure SELECT_BIRTHDAY (
    DAY_COUNT integer)
returns (
    ID integer,
    ORG_ID integer,
    FIO varchar(128),
    JOBTITLE varchar(128),
    ORGAN_NAME varchar(128),
    BIRTHDAY date,
    TEL_JOB varchar(64),
    TEL_MOBILE varchar(32),
    EMAIL varchar(64),
    ICQ varchar(32),
    SKYPE varchar(32),
    USER_ORG_LOGIN varchar(16),
    USER_ORG_FIO varchar(128))
as
declare variable DATE_NEXT date;
declare variable DATE_LAST date;
BEGIN

  DATE_LAST = CURRENT_DATE + DAY_COUNT;
  DATE_NEXT = CURRENT_DATE;

  WHILE (DATE_NEXT <= DATE_LAST)
  DO
    BEGIN
        FOR
        SELECT
            PERSON.ID,
            PERSON.FIO,
            PERSON.JOBTITLE,
            PERSON.BIRTHDAY,
            PERSON.TEL_JOB,
            PERSON.TEL_MOBILE,
            PERSON.EMAIL,
            PERSON.ICQ,
            PERSON.SKYPE,
            ORGANIZATION.NAME,
            ORGANIZATION.ID,
            USERS.LOGIN,
            USERS.FIO
        FROM
            PERSON
            INNER JOIN ORGANIZATION ON ORGANIZATION.ID = PERSON.ORGANIZATION_ID
            LEFT JOIN USERS ON ORGANIZATION.USERS_ID = USERS.ID
        WHERE
            (EXTRACT (DAY FROM PERSON.BIRTHDAY)) = (EXTRACT (DAY FROM :DATE_NEXT))
            AND
            (EXTRACT (MONTH FROM PERSON.BIRTHDAY)) = (EXTRACT (MONTH FROM :DATE_NEXT))
        INTO
            :ID,
            :FIO,
            :JOBTITLE,
            :BIRTHDAY,
            :TEL_JOB,
            :TEL_MOBILE,
            :EMAIL,
            :ICQ,
            :SKYPE,
            :ORGAN_NAME,
            :ORG_ID,
            :USER_ORG_LOGIN,
            :USER_ORG_FIO
        DO
          SUSPEND;

        DATE_NEXT = DATE_NEXT + 1;

     END

END;

/*------------------------------ TABLE PROJECTS_STATUS_MAIN -------------------------------------------*/

CREATE GENERATOR GEN_PROJECTS_STATUS_MAIN_ID;

CREATE TABLE PROJECTS_STATUS_MAIN (
    ID             INTEGER NOT NULL,
    NAME           VARCHAR(64) NOT NULL,
    EDIT_USERS_ID  INTEGER
);

ALTER TABLE PROJECTS_STATUS_MAIN ADD CONSTRAINT UNQ1_PROJECTS_STATUS_MAIN UNIQUE (NAME);

ALTER TABLE PROJECTS_STATUS_MAIN ADD CONSTRAINT PK_PROJECTS_STATUS_MAIN PRIMARY KEY (ID);

ALTER TABLE PROJECTS_STATUS_MAIN ADD CONSTRAINT FK_PROJECTS_STATUS_MAIN_1 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;

CREATE OR ALTER TRIGGER PROJECTS_STATUS_MAIN_BI FOR PROJECTS_STATUS_MAIN
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_PROJECTS_STATUS_MAIN_ID, 1);
END;

/*------------------------------ TABLE PROJECTS -------------------------------------------*/

ALTER TABLE PROJECTS ADD ORGANIZATION_ID INTEGER;
ALTER TABLE PROJECTS ADD PROJECTS_STATUS_MAIN_ID INTEGER;
ALTER TABLE PROJECTS ADD CURRENCY_ID INTEGER;
ALTER TABLE PROJECTS ADD SUMMA DOUBLE PRECISION;
ALTER TABLE PROJECTS ADD DT_NEXT TIMESTAMP;

ALTER TABLE PROJECTS ADD CONSTRAINT FK_PROJECTS_3 FOREIGN KEY (ORGANIZATION_ID) REFERENCES ORGANIZATION (ID) ON DELETE SET NULL;
ALTER TABLE PROJECTS ADD CONSTRAINT FK_PROJECTS_4 FOREIGN KEY (PROJECTS_STATUS_MAIN_ID) REFERENCES PROJECTS_STATUS_MAIN (ID) ON DELETE SET NULL;

CREATE OR ALTER TRIGGER PROJECTS_BI FOR PROJECTS
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_PROJECTS_ID, 1);

  IF (NEW.ORGANIZATION_ID < 1)
  THEN
    NEW.ORGANIZATION_ID = NULL;

  IF (NEW.PROJECTS_STATUS_MAIN_ID < 1)
  THEN
    NEW.PROJECTS_STATUS_MAIN_ID = NULL;

END;

CREATE OR ALTER TRIGGER PROJECTS_BU FOR PROJECTS
ACTIVE BEFORE UPDATE POSITION 0
AS
BEGIN

  IF (NEW.ORGANIZATION_ID < 1)
  THEN
    NEW.ORGANIZATION_ID = NULL;

  IF (NEW.PROJECTS_STATUS_MAIN_ID < 1)
  THEN
    NEW.PROJECTS_STATUS_MAIN_ID = NULL;

END;

/*-------------------------- procedure CLEAR_EDIT_USERS_ID -----------------------------------------------*/

CREATE OR ALTER procedure CLEAR_EDIT_USERS_ID (
    ID integer,
    IN_OUT integer,
    PROGRAM_ID integer)
as
BEGIN

  UPDATE SPAM
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE PROJECTS_STATUS_MAIN
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE REMINDER
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    EVENT_PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE_NUMBERS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    REGION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    COUNTRY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TREND
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PERSON
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    JOB_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACTIVITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TASK
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_RESULT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_SOURCE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  IF (IN_OUT = 1)
  THEN
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = NULL
      WHERE
        USERS_ID = :ID;
    END
  ELSE
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = :ID
      WHERE
        ID = :PROGRAM_ID;
    END
END;

/*-------------------------- TABLE USERS_ACCESS -----------------------------------------------*/

ALTER TABLE USERS_ACCESS ADD GRANTS BLOB SUB_TYPE 1 SEGMENT SIZE 80 CHARACTER SET WIN1251;

/*------------------------------ procedure ADD_XOR_DEL_USERS_ACCESS ----------------------------------*/

CREATE OR ALTER procedure ADD_XOR_DEL_USERS_ACCESS (
    USERS_OWNER_ID integer,
    USERS_ACCESS_ID integer,
    GRANTS blob sub_type 1 segment size 80,
    OPERATION integer)
as
BEGIN
 /* DELETE */
 IF (:OPERATION = 0)
 THEN
   BEGIN
     DELETE
       FROM
         USERS_ACCESS
       WHERE
         (USERS_ACCESS.USERS_OWNER_ID = :USERS_OWNER_ID)
         AND
         (USERS_ACCESS.USERS_ACCESS_ID =:USERS_ACCESS_ID);
   END

 /* INSERT */
 ELSE
 IF ((:OPERATION = 1) AND (NOT EXISTS (SELECT
                                        USERS_ACCESS.USERS_ACCESS_ID
                                    FROM
                                        USERS_ACCESS
                                    WHERE
                                        (USERS_ACCESS.USERS_OWNER_ID = :USERS_OWNER_ID) AND
                                        (USERS_ACCESS.USERS_ACCESS_ID =:USERS_ACCESS_ID))))
        THEN
            BEGIN
                INSERT INTO USERS_ACCESS
                (
                    USERS_ACCESS.USERS_OWNER_ID,
                    USERS_ACCESS.USERS_ACCESS_ID,
                    USERS_ACCESS.GRANTS
                 )
                VALUES
                (
                 :USERS_OWNER_ID,
                 :USERS_ACCESS_ID,
                 :GRANTS
                );
            END

  SUSPEND;
END;

/*----------------------- TABLE ORGANIZATION -----------------------------------------*/

ALTER TABLE ORGANIZATION ADD CONTRACT_CURRENCY_ID INTEGER;

/*----------------------- procedure ADD_ORGANIZATION -----------------------------------------*/

CREATE OR ALTER procedure ADD_ORGANIZATION (
    USERS_ID integer,
    NAME varchar(128),
    TEL varchar(64),
    FAX varchar(64),
    EMAIL varchar(64),
    WEB varchar(64),
    HOURS integer,
    REQ_INN varchar(16),
    REQ_KPP varchar(16),
    REQ_RACCOUNT varchar(32),
    REQ_OKPO varchar(16),
    REQ_OKVED varchar(128),
    REQ_BIK varchar(16),
    REQ_BANK varchar(128),
    REQ_KACCOUNT varchar(32),
    ACTIVITY_ID integer,
    PROPERTY_ID integer,
    ORGANIZATION_TYPE integer,
    CHECKED integer,
    EMAIL_SEND integer,
    CONFIG blob sub_type 1 segment size 80,
    COMMENTS blob sub_type 1 segment size 80,
    ACTUAL_POST_INDEX varchar(16),
    LEGAL_POST_INDEX varchar(16),
    POSTAL_POST_INDEX varchar(16),
    ACTUAL_ADDRESS blob sub_type 1 segment size 80,
    LEGAL_ADDRESS blob sub_type 1 segment size 80,
    POSTAL_ADDRESS blob sub_type 1 segment size 80,
    ACTUAL_COUNTRY_ID integer,
    ACTUAL_REGION_ID integer,
    ACTUAL_CITY_ID integer,
    LEGAL_COUNTRY_ID integer,
    LEGAL_REGION_ID integer,
    LEGAL_CITY_ID integer,
    POSTAL_COUNTRY_ID integer,
    POSTAL_REGION_ID integer,
    POSTAL_CITY_ID integer,
    X integer,
    Y integer,
    SUMMA_XY integer,
    ORGANIZATION_GROUP_ID integer,
    ARM_COUNT integer,
    PERSON_COUNT integer,
    CONTRACT_SUMMA double precision,
    ORGANIZATION_SOURCE_ID integer,
    CONTRACT_CURRENCY_ID integer)
returns (
    ERROR integer,
    ID integer)
as
declare variable ACTUAL_ADDRESS_ID integer;
declare variable LEGAL_ADDRESS_ID integer;
declare variable POSTAL_ADDRESS_ID integer;
BEGIN
  ERROR = -1;
  ID = -1;
  /****************************************************************************/
  IF ((:ACTUAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :ACTUAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 1; 
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_COUNTRY_ID = 0)
  THEN
    ACTUAL_COUNTRY_ID = NULL;

  IF ((:LEGAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :LEGAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 2; 
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_COUNTRY_ID = 0)
  THEN
    LEGAL_COUNTRY_ID = NULL;

  IF ((:POSTAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :POSTAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 3; 
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_COUNTRY_ID = 0)
  THEN
    POSTAL_COUNTRY_ID = NULL;

  IF ((:ACTUAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :ACTUAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 4; 
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_REGION_ID = 0)
  THEN
    ACTUAL_REGION_ID = NULL;

  IF ((:LEGAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :LEGAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 5; 
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_REGION_ID = 0)
  THEN
    LEGAL_REGION_ID = NULL;

  IF ((:POSTAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :POSTAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 6; 
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_REGION_ID = 0)
  THEN
    POSTAL_REGION_ID = NULL;

  IF ((:ACTUAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :ACTUAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 7; 
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_CITY_ID = 0)
  THEN
    ACTUAL_CITY_ID = NULL;

  IF ((:LEGAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :LEGAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 8; 
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_CITY_ID = 0)
  THEN
    LEGAL_CITY_ID = NULL;

  IF ((:POSTAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :POSTAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 9; 
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_CITY_ID = 0)
  THEN
    POSTAL_CITY_ID = NULL;


 /* IF (EXISTS (SELECT ID FROM ORGANIZATION WHERE NAME = :NAME))
  THEN
    BEGIN
      ERROR = 10;  
      SUSPEND;
      EXIT;
    END
  */
  IF ((:PROPERTY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM PROPERTY WHERE ID = :PROPERTY_ID)))
  THEN
    BEGIN
        ERROR = 11;
        SUSPEND;
        EXIT;
    END

  IF (:PROPERTY_ID = 0)
  THEN
    PROPERTY_ID = NULL;

  IF ((:ACTIVITY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM ACTIVITY WHERE ID = :ACTIVITY_ID)))
  THEN
    BEGIN
        ERROR = 12;
        SUSPEND;
        EXIT;
    END

  IF (:ACTIVITY_ID = 0)
  THEN
    ACTIVITY_ID = NULL;

  IF (:ORGANIZATION_SOURCE_ID = 0)
  THEN
    ORGANIZATION_SOURCE_ID = NULL;

  /*******************************************************************/
  ACTUAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:ACTUAL_ADDRESS_ID,
     :ACTUAL_POST_INDEX,
     :ACTUAL_COUNTRY_ID,
     :ACTUAL_REGION_ID,
     :ACTUAL_CITY_ID,
     :ACTUAL_ADDRESS);
  /*******************************************************************/
  LEGAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:LEGAL_ADDRESS_ID,
     :LEGAL_POST_INDEX,
     :LEGAL_COUNTRY_ID,
     :LEGAL_REGION_ID,
     :LEGAL_CITY_ID,
     :LEGAL_ADDRESS);
  /*******************************************************************/
  POSTAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:POSTAL_ADDRESS_ID,
     :POSTAL_POST_INDEX,
     :POSTAL_COUNTRY_ID,
     :POSTAL_REGION_ID,
     :POSTAL_CITY_ID,
     :POSTAL_ADDRESS);
  /*********************************************************************/
  ID = GEN_ID(GEN_ORGANIZATION_ID, 1);
  INSERT INTO ORGANIZATION
    (ID, 
    USERS_ID,
    ACTUAL_ADDRESS_ID,
    LEGAL_ADDRESS_ID,
    POSTAL_ADDRESS_ID,
    NAME,
    TEL,
    FAX,
    EMAIL,
    WEB,
    HOURS,
    REQ_INN,
    REQ_KPP,
    REQ_RACCOUNT,
    REQ_OKPO,
    REQ_OKVED,
    REQ_BIK,
    REQ_BANK,
    REQ_KACCOUNT,
    ACTIVITY_ID,
    REMINDER_COUNT,
    DT_CREATE,
    DT_LASTMODIFY,
    ORGANIZATION_TYPE,
    CHECKED,
    EMAIL_SEND,
    CONFIG,
    COMMENTS,
    PROPERTY_ID,
    X,
    Y,
    SUMMA_XY,
    ORGANIZATION_GROUP_ID,
    ARM_COUNT,
    PERSON_COUNT,
    CONTRACT_SUMMA,
    ORGANIZATION_SOURCE_ID,
    CONTRACT_CURRENCY_ID)
  VALUES
    (:ID,
    :USERS_ID,
    :ACTUAL_ADDRESS_ID,
    :LEGAL_ADDRESS_ID,
    :POSTAL_ADDRESS_ID,
    :NAME,
    :TEL,
    :FAX,
    :EMAIL,
    :WEB,
    :HOURS,
    :REQ_INN,
    :REQ_KPP,
    :REQ_RACCOUNT,
    :REQ_OKPO,
    :REQ_OKVED,
    :REQ_BIK,
    :REQ_BANK,
    :REQ_KACCOUNT,
    :ACTIVITY_ID,
    0,
    'NOW',
    'NOW',
    :ORGANIZATION_TYPE,
    :CHECKED,
    :EMAIL_SEND,
    :CONFIG,
    :COMMENTS,
    :PROPERTY_ID,
    :X,
    :Y,
    :SUMMA_XY,
    :ORGANIZATION_GROUP_ID,
    :ARM_COUNT,
    :PERSON_COUNT,
    :CONTRACT_SUMMA,
    :ORGANIZATION_SOURCE_ID,
    :CONTRACT_CURRENCY_ID);
  /****************************************************************************/
  ERROR = 0;
  SUSPEND;
END;

/*---------------------  procedure EDIT_ORGANIZATION  ------------------------------------------*/

CREATE OR ALTER procedure EDIT_ORGANIZATION (
    ID integer,
    NAME varchar(128),
    TEL varchar(64),
    FAX varchar(64),
    EMAIL varchar(64),
    WEB varchar(64),
    HOURS integer,
    REQ_INN varchar(16),
    REQ_KPP varchar(16),
    REQ_RACCOUNT varchar(32),
    REQ_OKPO varchar(16),
    REQ_OKVED varchar(128),
    REQ_BIK varchar(16),
    REQ_BANK varchar(128),
    REQ_KACCOUNT varchar(32),
    ACTIVITY_ID integer,
    ORGANIZATION_TYPE integer,
    CHECKED integer,
    EMAIL_SEND integer,
    CONFIG blob sub_type 1 segment size 80,
    COMMENTS blob sub_type 1 segment size 80,
    ACTUAL_POST_INDEX varchar(16),
    LEGAL_POST_INDEX varchar(16),
    POSTAL_POST_INDEX varchar(16),
    ACTUAL_ADDRESS blob sub_type 1 segment size 80,
    LEGAL_ADDRESS blob sub_type 1 segment size 80,
    POSTAL_ADDRESS blob sub_type 1 segment size 80,
    ACTUAL_COUNTRY_ID integer,
    ACTUAL_REGION_ID integer,
    ACTUAL_CITY_ID integer,
    LEGAL_COUNTRY_ID integer,
    LEGAL_REGION_ID integer,
    LEGAL_CITY_ID integer,
    POSTAL_COUNTRY_ID integer,
    POSTAL_REGION_ID integer,
    POSTAL_CITY_ID integer,
    A_ADDRESS_ID integer,
    L_ADDRESS_ID integer,
    P_ADDRESS_ID integer,
    PROPERTY_ID integer,
    X integer,
    Y integer,
    SUMMA_XY integer,
    ORGANIZATION_GROUP_ID integer,
    ARM_COUNT integer,
    PERSON_COUNT integer,
    CONTRACT_SUMMA double precision,
    ORGANIZATION_SOURCE_ID integer,
    CONTRACT_CURRENCY_ID integer)
returns (
    ERROR integer)
as
BEGIN
  ERROR = -1;
  /****************************************************************************/
  IF ((:ACTUAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :ACTUAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 1;
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_COUNTRY_ID = 0)
  THEN
    ACTUAL_COUNTRY_ID = NULL;

  IF ((:LEGAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :LEGAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 2;
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_COUNTRY_ID = 0)
  THEN
    LEGAL_COUNTRY_ID = NULL;

  IF ((:POSTAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :POSTAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 3;
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_COUNTRY_ID = 0)
  THEN
    POSTAL_COUNTRY_ID = NULL;

  IF ((:ACTUAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :ACTUAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 4;
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_REGION_ID = 0)
  THEN
    ACTUAL_REGION_ID = NULL;

  IF ((:LEGAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :LEGAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 5;
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_REGION_ID = 0)
  THEN
    LEGAL_REGION_ID = NULL;

  IF ((:POSTAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :POSTAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 6;
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_REGION_ID = 0)
  THEN
    POSTAL_REGION_ID = NULL;

  IF ((:ACTUAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :ACTUAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 7;
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_CITY_ID = 0)
  THEN
    ACTUAL_CITY_ID = NULL;

  IF ((:LEGAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :LEGAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 8;
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_CITY_ID = 0)
  THEN
    LEGAL_CITY_ID = NULL;

  IF ((:POSTAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :POSTAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 9;
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_CITY_ID = 0)
  THEN
    POSTAL_CITY_ID = NULL;


  IF ((:PROPERTY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM PROPERTY WHERE ID = :PROPERTY_ID)))
  THEN
    BEGIN
        ERROR = 10;
        SUSPEND;
        EXIT;
    END

  IF (:PROPERTY_ID = 0)
  THEN
    PROPERTY_ID = NULL;

  IF ((:ACTIVITY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM ACTIVITY WHERE ID = :ACTIVITY_ID)))
  THEN
    BEGIN
        ERROR = 11; 
        SUSPEND;
        EXIT;
    END

  IF (:ACTIVITY_ID = 0)
  THEN
    ACTIVITY_ID = NULL;

  IF (:ORGANIZATION_SOURCE_ID = 0)
  THEN
    ORGANIZATION_SOURCE_ID = NULL;

  /*******************************************************************/

  /*******************************************************************/
  UPDATE ADDRESS
    SET
        POST_INDEX = :ACTUAL_POST_INDEX,
        COUNTRY_ID = :ACTUAL_COUNTRY_ID,
        REGION_ID = :ACTUAL_REGION_ID,
        CITY_ID =:ACTUAL_CITY_ID,
        ADDRESS = :ACTUAL_ADDRESS
     WHERE
        ID = :A_ADDRESS_ID;

  /*******************************************************************/
  UPDATE ADDRESS
    SET
        POST_INDEX = :LEGAL_POST_INDEX,
        COUNTRY_ID = :LEGAL_COUNTRY_ID,
        REGION_ID = :LEGAL_REGION_ID,
        CITY_ID =:LEGAL_CITY_ID,
        ADDRESS = :LEGAL_ADDRESS
     WHERE
        ID = :L_ADDRESS_ID;
  /*********************************************************************/
  UPDATE ADDRESS
    SET
        POST_INDEX = :POSTAL_POST_INDEX,
        COUNTRY_ID = :POSTAL_COUNTRY_ID,
        REGION_ID = :POSTAL_REGION_ID,
        CITY_ID =:POSTAL_CITY_ID,
        ADDRESS = :POSTAL_ADDRESS
     WHERE
        ID = :P_ADDRESS_ID;
  /*********************************************************************/
  UPDATE ORGANIZATION
    SET
        NAME =: NAME,
        TEL =: TEL,
        FAX =: FAX,
        EMAIL =: EMAIL,
        WEB =: WEB,
        HOURS =: HOURS,
        REQ_INN =: REQ_INN,
        REQ_KPP =: REQ_KPP,
        REQ_RACCOUNT =: REQ_RACCOUNT,
        REQ_OKPO =: REQ_OKPO,
        REQ_OKVED =: REQ_OKVED,
        REQ_BIK =: REQ_BIK,
        REQ_BANK =: REQ_BANK,
        REQ_KACCOUNT =: REQ_KACCOUNT,
        ACTIVITY_ID =: ACTIVITY_ID,
        PROPERTY_ID =: PROPERTY_ID,
        DT_LASTMODIFY = 'NOW',
        ORGANIZATION_TYPE =:ORGANIZATION_TYPE,
        CHECKED =:CHECKED,
        EMAIL_SEND =: EMAIL_SEND,
        CONFIG =: CONFIG,
        COMMENTS =:COMMENTS,
        X = :X,
        Y = :Y,
        SUMMA_XY = :SUMMA_XY,
        organization_group_id = :organization_group_id,
        ARM_COUNT = :ARM_COUNT,
        PERSON_COUNT = :PERSON_COUNT,
        CONTRACT_SUMMA = :CONTRACT_SUMMA,
        ORGANIZATION_SOURCE_ID = :ORGANIZATION_SOURCE_ID,
        CONTRACT_CURRENCY_ID = :CONTRACT_CURRENCY_ID
     WHERE
        ID =:ID ;

  /****************************************************************************/
  ERROR = 0;
  SUSPEND;
END;

/*--------------------- TABLE AGREEMENT ------------------------------------------*/

ALTER TABLE AGREEMENT ADD SUMMA_CURRENCY_ID INTEGER;
ALTER TABLE AGREEMENT ADD DISCOUNT_TYPE INTEGER;
