ALTER TABLE ORGANIZATION
ADD USERS_BUSY_ID INTEGER;

alter table ORGANIZATION
add constraint FK_ORGANIZATION_7
foreign key (USERS_BUSY_ID)
references USERS(ID)
on delete SET NULL;

ALTER TABLE PERSON
ADD MAIN_PERSON INTEGER;

ALTER TABLE PERSON
ADD EMAIL2 VARCHAR(64);

ALTER TABLE PERSON
ADD USERS_LASTEDIT_ID INTEGER;

alter table PERSON
add constraint FK_PERSON_5
foreign key (USERS_LASTEDIT_ID)
references USERS(ID)
on delete SET NULL;

ALTER TABLE CONTACT
ADD USERS_LASTEDIT_ID INTEGER;

alter table CONTACT
add constraint FK_CONTACT_7
foreign key (USERS_LASTEDIT_ID)
references USERS(ID)
on delete SET NULL;

ALTER TABLE REMINDER
ADD PRIORITY INTEGER;

ALTER TABLE TASK
ADD PERSON_ID INTEGER;

alter table TASK
add constraint FK_TASK_4
foreign key (PERSON_ID)
references PERSON(ID)
on delete SET NULL;

CREATE TABLE PERSON_NOMENCLATURE (
    PERSON_ID        INTEGER NOT NULL,
    NOMENCLATURE_ID  INTEGER NOT NULL
);

ALTER TABLE PERSON_NOMENCLATURE ADD CONSTRAINT UNQ1_PERSON_NOMENCLATURE UNIQUE (PERSON_ID, NOMENCLATURE_ID);

ALTER TABLE PERSON_NOMENCLATURE ADD CONSTRAINT FK_PERSON_NOMENCLATURE_1 FOREIGN KEY (PERSON_ID) REFERENCES PERSON (ID) ON DELETE CASCADE;
ALTER TABLE PERSON_NOMENCLATURE ADD CONSTRAINT FK_PERSON_NOMENCLATURE_2 FOREIGN KEY (NOMENCLATURE_ID) REFERENCES NOMENCLATURE (ID) ON DELETE CASCADE;

CREATE OR ALTER TRIGGER ORGANIZATION_BI FOR ORGANIZATION
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_ORGANIZATION_ID, 1);

  IF (NEW.ARM_COUNT IS NULL)
  THEN
    NEW.ARM_COUNT = 0;

  IF (NEW.PERSON_COUNT IS NULL)
  THEN
    NEW.PERSON_COUNT = 0;

  IF (NEW.CONTRACT_SUMMA IS NULL)
  THEN
    NEW.CONTRACT_SUMMA = 0;

  NEW.DOCUMENT_COUNT = 0;

  IF (NEW.USERS_BUSY_ID < 0)
  THEN
    NEW.USERS_BUSY_ID = NULL;
END;

CREATE OR ALTER procedure ADD_ORGANIZATION (
    USERS_ID integer,
    NAME varchar(128),
    TEL varchar(64),
    FAX varchar(64),
    EMAIL varchar(64),
    WEB varchar(64),
    HOURS integer,
    REQ_INN varchar(16),
    REQ_KPP varchar(16),
    REQ_RACCOUNT varchar(32),
    REQ_OKPO varchar(16),
    REQ_OKVED varchar(128),
    REQ_BIK varchar(16),
    REQ_BANK varchar(128),
    REQ_KACCOUNT varchar(32),
    ACTIVITY_ID integer,
    PROPERTY_ID integer,
    ORGANIZATION_TYPE integer,
    CHECKED integer,
    EMAIL_SEND integer,
    CONFIG blob sub_type 1 segment size 80,
    COMMENTS blob sub_type 1 segment size 80,
    ACTUAL_POST_INDEX varchar(16),
    LEGAL_POST_INDEX varchar(16),
    POSTAL_POST_INDEX varchar(16),
    ACTUAL_ADDRESS blob sub_type 1 segment size 80,
    LEGAL_ADDRESS blob sub_type 1 segment size 80,
    POSTAL_ADDRESS blob sub_type 1 segment size 80,
    ACTUAL_COUNTRY_ID integer,
    ACTUAL_REGION_ID integer,
    ACTUAL_CITY_ID integer,
    LEGAL_COUNTRY_ID integer,
    LEGAL_REGION_ID integer,
    LEGAL_CITY_ID integer,
    POSTAL_COUNTRY_ID integer,
    POSTAL_REGION_ID integer,
    POSTAL_CITY_ID integer,
    X integer,
    Y integer,
    SUMMA_XY integer,
    ORGANIZATION_GROUP_ID integer,
    ARM_COUNT integer,
    PERSON_COUNT integer,
    CONTRACT_SUMMA double precision,
    ORGANIZATION_SOURCE_ID integer,
    CONTRACT_CURRENCY_ID integer)
returns (
    ERROR integer,
    ID integer)
as
declare variable ACTUAL_ADDRESS_ID integer;
declare variable LEGAL_ADDRESS_ID integer;
declare variable POSTAL_ADDRESS_ID integer;
BEGIN
  ERROR = -1;
  ID = -1;
  /****************************************************************************/
  IF ((:ACTUAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :ACTUAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 1; 
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_COUNTRY_ID = 0)
  THEN
    ACTUAL_COUNTRY_ID = NULL;

  IF ((:LEGAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :LEGAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 2; 
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_COUNTRY_ID = 0)
  THEN
    LEGAL_COUNTRY_ID = NULL;

  IF ((:POSTAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :POSTAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 3; 
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_COUNTRY_ID = 0)
  THEN
    POSTAL_COUNTRY_ID = NULL;

  IF ((:ACTUAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :ACTUAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 4; 
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_REGION_ID = 0)
  THEN
    ACTUAL_REGION_ID = NULL;

  IF ((:LEGAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :LEGAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 5; 
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_REGION_ID = 0)
  THEN
    LEGAL_REGION_ID = NULL;

  IF ((:POSTAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :POSTAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 6; 
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_REGION_ID = 0)
  THEN
    POSTAL_REGION_ID = NULL;

  IF ((:ACTUAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :ACTUAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 7; 
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_CITY_ID = 0)
  THEN
    ACTUAL_CITY_ID = NULL;

  IF ((:LEGAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :LEGAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 8; 
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_CITY_ID = 0)
  THEN
    LEGAL_CITY_ID = NULL;

  IF ((:POSTAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :POSTAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 9; 
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_CITY_ID = 0)
  THEN
    POSTAL_CITY_ID = NULL;


 /* IF (EXISTS (SELECT ID FROM ORGANIZATION WHERE NAME = :NAME))
  THEN
    BEGIN
      ERROR = 10;  
      SUSPEND;
      EXIT;
    END
  */
  IF ((:PROPERTY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM PROPERTY WHERE ID = :PROPERTY_ID)))
  THEN
    BEGIN
        ERROR = 11;
        SUSPEND;
        EXIT;
    END

  IF (:PROPERTY_ID = 0)
  THEN
    PROPERTY_ID = NULL;

  IF ((:ACTIVITY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM ACTIVITY WHERE ID = :ACTIVITY_ID)))
  THEN
    BEGIN
        ERROR = 12;
        SUSPEND;
        EXIT;
    END

  IF (:ACTIVITY_ID = 0)
  THEN
    ACTIVITY_ID = NULL;

  IF (:ORGANIZATION_SOURCE_ID = 0)
  THEN
    ORGANIZATION_SOURCE_ID = NULL;

  /*******************************************************************/
  ACTUAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:ACTUAL_ADDRESS_ID,
     :ACTUAL_POST_INDEX,
     :ACTUAL_COUNTRY_ID,
     :ACTUAL_REGION_ID,
     :ACTUAL_CITY_ID,
     :ACTUAL_ADDRESS);
  /*******************************************************************/
  LEGAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:LEGAL_ADDRESS_ID,
     :LEGAL_POST_INDEX,
     :LEGAL_COUNTRY_ID,
     :LEGAL_REGION_ID,
     :LEGAL_CITY_ID,
     :LEGAL_ADDRESS);
  /*******************************************************************/
  POSTAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:POSTAL_ADDRESS_ID,
     :POSTAL_POST_INDEX,
     :POSTAL_COUNTRY_ID,
     :POSTAL_REGION_ID,
     :POSTAL_CITY_ID,
     :POSTAL_ADDRESS);
  /*********************************************************************/
  ID = GEN_ID(GEN_ORGANIZATION_ID, 1);
  INSERT INTO ORGANIZATION
    (ID, 
    USERS_ID,
    ACTUAL_ADDRESS_ID,
    LEGAL_ADDRESS_ID,
    POSTAL_ADDRESS_ID,
    NAME,
    TEL,
    FAX,
    EMAIL,
    WEB,
    HOURS,
    REQ_INN,
    REQ_KPP,
    REQ_RACCOUNT,
    REQ_OKPO,
    REQ_OKVED,
    REQ_BIK,
    REQ_BANK,
    REQ_KACCOUNT,
    ACTIVITY_ID,
    REMINDER_COUNT,
    DT_CREATE,
    DT_LASTMODIFY,
    ORGANIZATION_TYPE,
    CHECKED,
    EMAIL_SEND,
    CONFIG,
    COMMENTS,
    PROPERTY_ID,
    X,
    Y,
    SUMMA_XY,
    ORGANIZATION_GROUP_ID,
    ARM_COUNT,
    PERSON_COUNT,
    CONTRACT_SUMMA,
    ORGANIZATION_SOURCE_ID,
    CONTRACT_CURRENCY_ID,
    USERS_BUSY_ID)
  VALUES
    (:ID,
    :USERS_ID,
    :ACTUAL_ADDRESS_ID,
    :LEGAL_ADDRESS_ID,
    :POSTAL_ADDRESS_ID,
    :NAME,
    :TEL,
    :FAX,
    :EMAIL,
    :WEB,
    :HOURS,
    :REQ_INN,
    :REQ_KPP,
    :REQ_RACCOUNT,
    :REQ_OKPO,
    :REQ_OKVED,
    :REQ_BIK,
    :REQ_BANK,
    :REQ_KACCOUNT,
    :ACTIVITY_ID,
    0,
    'NOW',
    'NOW',
    :ORGANIZATION_TYPE,
    :CHECKED,
    :EMAIL_SEND,
    :CONFIG,
    :COMMENTS,
    :PROPERTY_ID,
    :X,
    :Y,
    :SUMMA_XY,
    :ORGANIZATION_GROUP_ID,
    :ARM_COUNT,
    :PERSON_COUNT,
    :CONTRACT_SUMMA,
    :ORGANIZATION_SOURCE_ID,
    :CONTRACT_CURRENCY_ID,
    NULL);
  /****************************************************************************/
  ERROR = 0;
  SUSPEND;
END;

CREATE GENERATOR GEN_ACTIONS_ID;

CREATE TABLE ACTIONS (
    ID             INTEGER NOT NULL,
    DT_ACTION      TIMESTAMP NOT NULL,
    NAME           VARCHAR(256) NOT NULL,
    EDIT_USERS_ID  INTEGER
);

ALTER TABLE ACTIONS ADD CONSTRAINT UNQ1_ACTIONS UNIQUE (NAME);

ALTER TABLE ACTIONS ADD CONSTRAINT PK_ACTIONS PRIMARY KEY (ID);

ALTER TABLE ACTIONS ADD CONSTRAINT FK_ACTIONS_1 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;

CREATE OR ALTER TRIGGER ACTIONS_BI FOR ACTIONS
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_ACTIONS_ID, 1);
END;

CREATE GENERATOR GEN_PERSON_ACTIONS_ID;

CREATE TABLE PERSON_ACTIONS (
    ID                 INTEGER NOT NULL,
    PERSON_ID          INTEGER NOT NULL,
    ACTIONS_ID         INTEGER NOT NULL,
    USERS_ID           INTEGER NOT NULL,
    USERS_LASTEDIT_ID  INTEGER NOT NULL
);


ALTER TABLE PERSON_ACTIONS ADD CONSTRAINT PK_PERSON_ACTIONS PRIMARY KEY (ID);

ALTER TABLE PERSON_ACTIONS ADD CONSTRAINT FK_PERSON_ACTIONS_1 FOREIGN KEY (PERSON_ID) REFERENCES PERSON (ID) ON DELETE CASCADE;
ALTER TABLE PERSON_ACTIONS ADD CONSTRAINT FK_PERSON_ACTIONS_2 FOREIGN KEY (ACTIONS_ID) REFERENCES ACTIONS (ID) ON DELETE CASCADE;
ALTER TABLE PERSON_ACTIONS ADD CONSTRAINT FK_PERSON_ACTIONS_3 FOREIGN KEY (USERS_ID) REFERENCES USERS (ID) ON DELETE CASCADE;
ALTER TABLE PERSON_ACTIONS ADD CONSTRAINT FK_PERSON_ACTIONS_4 FOREIGN KEY (USERS_LASTEDIT_ID) REFERENCES USERS (ID) ON DELETE CASCADE;

CREATE OR ALTER TRIGGER PERSON_ACTIONS_BI FOR PERSON_ACTIONS
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_PERSON_ACTIONS_ID, 1);
END;

CREATE OR ALTER procedure CLEAR_EDIT_USERS_ID (
    ID integer,
    IN_OUT integer,
    PROGRAM_ID integer)
as
BEGIN

  UPDATE SPAM
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE PROJECTS_STATUS_MAIN
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE REMINDER
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    EVENT_PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE_NUMBERS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    REGION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    COUNTRY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TREND
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PERSON
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    JOB_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACTIVITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TASK
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_RESULT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_SOURCE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACTIONS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  IF (IN_OUT = 1)
  THEN
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = NULL
      WHERE
        USERS_ID = :ID;
    END
  ELSE
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = :ID
      WHERE
        ID = :PROGRAM_ID;
    END
END;

DROP TRIGGER DOCUMENT_AIUD;

CREATE OR ALTER TRIGGER DOCUMENT_BI FOR DOCUMENT
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_DOCUMENT_ID, 1);

  IF ((NEW.NUMBER IS NULL) or (NEW.NUMBER = ''))
  THEN
    NEW.NUMBER = NEW.ID;
END;

alter table PERSON_ACTIONS
add constraint UNQ1_PERSON_ACTIONS
unique (PERSON_ID,ACTIONS_ID);
