CREATE OR ALTER procedure GET_SPAM_MAIL (
    SHOW_STAT integer,
    SPAM_ID integer)
returns (
    COUNT_NOW integer,
    COUNT_ALL integer,
    SPAM_MAILS_ID varchar(128),
    PERSON_ORG_MAIL varchar(128),
    PERSON_ORG_NAME varchar(128),
    ORG_NAME varchar(128),
    PERSON_LASTNAME varchar(42),
    PERSON_FIRSNAME varchar(42),
    PERSON_SECONDNAME varchar(42))
as
declare variable PERSON_ID integer;
declare variable ORGANIZATION_ID integer;
BEGIN
  FOR
  SELECT
    SPAM_EMAILS.ID,
    SPAM_EMAILS.ORGANIZATION_ID,
    SPAM_EMAILS.PERSON_ID
  FROM
    SPAM_EMAILS
  WHERE
    (SPAM_EMAILS.SPAM_ID =:SPAM_ID) AND
    (SPAM_EMAILS.STATUS = 0)
  INTO
    :SPAM_MAILS_ID,
    :ORGANIZATION_ID,
    :PERSON_ID
  DO
    BREAK;

  IF (SPAM_MAILS_ID IS NULL)
  THEN
    BEGIN
        SUSPEND;
        EXIT;
    END
  IF (PERSON_ID IS NULL)
  THEN
    BEGIN
        SELECT
            ORGANIZATION.NAME,
            ORGANIZATION.EMAIL,
            ORGANIZATION.NAME
        FROM
            ORGANIZATION
        WHERE
            ORGANIZATION.ID =:ORGANIZATION_ID
        INTO
            :PERSON_ORG_NAME,
            :PERSON_ORG_MAIL,
            :ORG_NAME;
    END
  ELSE
    BEGIN

        SELECT
            PERSON.FIO,
            PERSON.LASTNAME,
            PERSON.FIRSTNAME,
            PERSON.SECONDNAME,
            PERSON.EMAIL,
            ORGANIZATION.NAME
        FROM
            PERSON
            INNER JOIN ORGANIZATION ON ORGANIZATION.ID =PERSON.ORGANIZATION_ID
        WHERE
            PERSON.ID =:PERSON_ID
        INTO
            :PERSON_ORG_NAME,
            :PERSON_LASTNAME,
            :PERSON_FIRSNAME,
            :PERSON_SECONDNAME,
            :PERSON_ORG_MAIL,
            :ORG_NAME;
    END
/*
  UPDATE SPAM_EMAILS
  SET
    SPAM_EMAILS.STATUS = 1
  WHERE
    SPAM_EMAILS.ID =:SPAM_MAILS_ID;
*/
  IF (SHOW_STAT = 1)
  THEN
    BEGIN
        SELECT
            COUNT(SPAM_EMAILS.ID)
        FROM
            SPAM_EMAILS
        WHERE
            (SPAM_EMAILS.SPAM_ID = :SPAM_ID) AND
            (SPAM_EMAILS.STATUS <> 0 )
        INTO
        :COUNT_NOW;

        SELECT
            COUNT(SPAM_EMAILS.ID)
        FROM
            SPAM_EMAILS
        WHERE
            (SPAM_EMAILS.SPAM_ID = :SPAM_ID)
        INTO
            :COUNT_ALL;
    END


  SUSPEND;
END;

CREATE GENERATOR GEN_ORGANIZATION_DISCOUNT_ID;

CREATE TABLE ORGANIZATION_DISCOUNT (
    ID               INTEGER NOT NULL,
    USERS_ID         INTEGER NOT NULL,
    ORGANIZATION_ID  INTEGER NOT NULL,
    NOMENCLATURE_ID  INTEGER NOT NULL,
    DISCOUNT         DOUBLE PRECISION NOT NULL,
    DAYS             INTEGER NOT NULL,
    COMMENTS         BLOB SUB_TYPE 1 SEGMENT SIZE 80,
    EDIT_USERS_ID    INTEGER
);

ALTER TABLE ORGANIZATION_DISCOUNT ADD CONSTRAINT UNQ1_ORGANIZATION_DISCOUNT UNIQUE (ORGANIZATION_ID, NOMENCLATURE_ID);
ALTER TABLE ORGANIZATION_DISCOUNT ADD CONSTRAINT PK_ORGANIZATION_DISCOUNT PRIMARY KEY (ID);
ALTER TABLE ORGANIZATION_DISCOUNT ADD CONSTRAINT FK_ORGANIZATION_DISCOUNT_1 FOREIGN KEY (ORGANIZATION_ID) REFERENCES ORGANIZATION (ID) ON DELETE CASCADE;
ALTER TABLE ORGANIZATION_DISCOUNT ADD CONSTRAINT FK_ORGANIZATION_DISCOUNT_2 FOREIGN KEY (NOMENCLATURE_ID) REFERENCES NOMENCLATURE (ID) ON DELETE CASCADE;
ALTER TABLE ORGANIZATION_DISCOUNT ADD CONSTRAINT FK_ORGANIZATION_DISCOUNT_3 FOREIGN KEY (USERS_ID) REFERENCES USERS (ID) ON DELETE CASCADE;
ALTER TABLE ORGANIZATION_DISCOUNT ADD CONSTRAINT FK_ORGANIZATION_DISCOUNT_4 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;

CREATE OR ALTER TRIGGER ORGANIZATION_DISCOUNT_BI FOR ORGANIZATION_DISCOUNT
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN

  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_ORGANIZATION_DISCOUNT_ID, 1);

  IF (NEW.DISCOUNT IS NULL)
  THEN
    NEW.DISCOUNT = 0;

  IF (NEW.DAYS IS NULL)
  THEN
    NEW.DAYS = 0;

END;

CREATE OR ALTER procedure LIST_DISCOUNT (
    NOMENCLATURE_ID_01 integer,
    NOMENCLATURE_ID_02 integer,
    NOMENCLATURE_ID_03 integer,
    NOMENCLATURE_ID_04 integer,
    NOMENCLATURE_ID_05 integer,
    NOMENCLATURE_ID_06 integer,
    NOMENCLATURE_ID_07 integer,
    NOMENCLATURE_ID_08 integer,
    NOMENCLATURE_ID_09 integer,
    NOMENCLATURE_ID_10 integer,
    NOMENCLATURE_ID_11 integer,
    NOMENCLATURE_ID_12 integer,
    NOMENCLATURE_ID_13 integer,
    NOMENCLATURE_ID_14 integer,
    NOMENCLATURE_ID_15 integer,
    NOMENCLATURE_ID_16 integer,
    NOMENCLATURE_ID_17 integer,
    NOMENCLATURE_ID_18 integer,
    NOMENCLATURE_ID_19 integer,
    NOMENCLATURE_ID_20 integer,
    NOMENCLATURE_ID_21 integer)
returns (
    ORGANIZATION_ID integer,
    ORGANIZATION_NAME varchar(128),
    NOMENCLATURE_DISCOUNT_01 double precision,
    NOMENCLATURE_DISCOUNT_02 double precision,
    NOMENCLATURE_DISCOUNT_03 double precision,
    NOMENCLATURE_DISCOUNT_04 double precision,
    NOMENCLATURE_DISCOUNT_05 double precision,
    NOMENCLATURE_DISCOUNT_06 double precision,
    NOMENCLATURE_DISCOUNT_07 double precision,
    NOMENCLATURE_DISCOUNT_08 double precision,
    NOMENCLATURE_DISCOUNT_09 double precision,
    NOMENCLATURE_DISCOUNT_10 double precision,
    NOMENCLATURE_DISCOUNT_11 double precision,
    NOMENCLATURE_DISCOUNT_12 double precision,
    NOMENCLATURE_DISCOUNT_13 double precision,
    NOMENCLATURE_DISCOUNT_14 double precision,
    NOMENCLATURE_DISCOUNT_15 double precision,
    NOMENCLATURE_DISCOUNT_16 double precision,
    NOMENCLATURE_DISCOUNT_17 double precision,
    NOMENCLATURE_DISCOUNT_18 double precision,
    NOMENCLATURE_DISCOUNT_19 double precision,
    NOMENCLATURE_DISCOUNT_20 double precision,
    NOMENCLATURE_DISCOUNT_21 double precision)
as
declare variable NOMENCLATURE_ID integer;
declare variable I integer;
declare variable DISCOUNT double precision;
BEGIN
  FOR
    SELECT
      ID,
      NAME
    FROM
      ORGANIZATION
    ORDER BY
      NAME
    INTO
      :ORGANIZATION_ID,
      :ORGANIZATION_NAME
  DO
    BEGIN
      NOMENCLATURE_DISCOUNT_01 = NULL;
      NOMENCLATURE_DISCOUNT_02 = NULL;
      NOMENCLATURE_DISCOUNT_03 = NULL;
      NOMENCLATURE_DISCOUNT_04 = NULL;
      NOMENCLATURE_DISCOUNT_05 = NULL;
      NOMENCLATURE_DISCOUNT_06 = NULL;
      NOMENCLATURE_DISCOUNT_07 = NULL;
      NOMENCLATURE_DISCOUNT_08 = NULL;
      NOMENCLATURE_DISCOUNT_09 = NULL;
      NOMENCLATURE_DISCOUNT_10 = NULL;
      NOMENCLATURE_DISCOUNT_11 = NULL;
      NOMENCLATURE_DISCOUNT_12 = NULL;
      NOMENCLATURE_DISCOUNT_13 = NULL;
      NOMENCLATURE_DISCOUNT_14 = NULL;
      NOMENCLATURE_DISCOUNT_15 = NULL;
      NOMENCLATURE_DISCOUNT_16 = NULL;
      NOMENCLATURE_DISCOUNT_17 = NULL;
      NOMENCLATURE_DISCOUNT_18 = NULL;
      NOMENCLATURE_DISCOUNT_19 = NULL;
      NOMENCLATURE_DISCOUNT_20 = NULL;
      NOMENCLATURE_DISCOUNT_21 = NULL;
      I = 0;
      FOR
        SELECT
          ID
        FROM
          NOMENCLATURE
        WHERE
          (ID IN (:NOMENCLATURE_ID_01, :NOMENCLATURE_ID_02, :NOMENCLATURE_ID_03,
                  :NOMENCLATURE_ID_04, :NOMENCLATURE_ID_05, :NOMENCLATURE_ID_06,
                  :NOMENCLATURE_ID_07, :NOMENCLATURE_ID_08, :NOMENCLATURE_ID_09,
                  :NOMENCLATURE_ID_10, :NOMENCLATURE_ID_11, :NOMENCLATURE_ID_12,
                  :NOMENCLATURE_ID_13, :NOMENCLATURE_ID_14, :NOMENCLATURE_ID_15,
                  :NOMENCLATURE_ID_16, :NOMENCLATURE_ID_17, :NOMENCLATURE_ID_18,
                  :NOMENCLATURE_ID_19, :NOMENCLATURE_ID_20, :NOMENCLATURE_ID_21))
        ORDER BY
          NAME
        INTO
          :NOMENCLATURE_ID
      DO
        BEGIN
          I = I + 1;
          DISCOUNT = NULL;

          SELECT
            DISCOUNT
          FROM
            ORGANIZATION_DISCOUNT
          WHERE
            (ORGANIZATION_ID = :ORGANIZATION_ID) AND
            (NOMENCLATURE_ID = :NOMENCLATURE_ID)
          INTO
            :DISCOUNT;

          IF (I =  1) THEN NOMENCLATURE_DISCOUNT_01 = :DISCOUNT;
          IF (I =  2) THEN NOMENCLATURE_DISCOUNT_02 = :DISCOUNT;
          IF (I =  3) THEN NOMENCLATURE_DISCOUNT_03 = :DISCOUNT;
          IF (I =  4) THEN NOMENCLATURE_DISCOUNT_04 = :DISCOUNT;
          IF (I =  5) THEN NOMENCLATURE_DISCOUNT_05 = :DISCOUNT;
          IF (I =  6) THEN NOMENCLATURE_DISCOUNT_06 = :DISCOUNT;
          IF (I =  7) THEN NOMENCLATURE_DISCOUNT_07 = :DISCOUNT;
          IF (I =  8) THEN NOMENCLATURE_DISCOUNT_08 = :DISCOUNT;
          IF (I =  9) THEN NOMENCLATURE_DISCOUNT_09 = :DISCOUNT;
          IF (I = 10) THEN NOMENCLATURE_DISCOUNT_10 = :DISCOUNT;
          IF (I = 11) THEN NOMENCLATURE_DISCOUNT_11 = :DISCOUNT;
          IF (I = 12) THEN NOMENCLATURE_DISCOUNT_12 = :DISCOUNT;
          IF (I = 13) THEN NOMENCLATURE_DISCOUNT_13 = :DISCOUNT;
          IF (I = 14) THEN NOMENCLATURE_DISCOUNT_14 = :DISCOUNT;
          IF (I = 15) THEN NOMENCLATURE_DISCOUNT_15 = :DISCOUNT;
          IF (I = 16) THEN NOMENCLATURE_DISCOUNT_16 = :DISCOUNT;
          IF (I = 17) THEN NOMENCLATURE_DISCOUNT_17 = :DISCOUNT;
          IF (I = 18) THEN NOMENCLATURE_DISCOUNT_18 = :DISCOUNT;
          IF (I = 19) THEN NOMENCLATURE_DISCOUNT_19 = :DISCOUNT;
          IF (I = 20) THEN NOMENCLATURE_DISCOUNT_20 = :DISCOUNT;
          IF (I = 21) THEN NOMENCLATURE_DISCOUNT_21 = :DISCOUNT;
        END
      suspend;
    END
END;

EXECUTE PROCEDURE SET_VERSION('3.1.32');
