CREATE GENERATOR GEN_NOMENCLATURE_GROUP_ID;

CREATE TABLE NOMENCLATURE_GROUP (
    ID             INTEGER NOT NULL,
    NAME           VARCHAR(128) NOT NULL,
    CODE           VARCHAR(32),
    DISCOUNT_1     DOUBLE PRECISION NOT NULL,
    DISCOUNT_2     DOUBLE PRECISION NOT NULL,
    EDIT_USERS_ID  INTEGER
);

ALTER TABLE NOMENCLATURE_GROUP ADD CONSTRAINT UNQ1_NOMENCLATURE_GROUP UNIQUE (NAME);
ALTER TABLE NOMENCLATURE_GROUP ADD CONSTRAINT PK_NOMENCLATURE_GROUP PRIMARY KEY (ID);

ALTER TABLE NOMENCLATURE_GROUP ADD CONSTRAINT FK_NOMENCLATURE_GROUP_1 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;

CREATE OR ALTER TRIGGER NOMENCLATURE_GROUP_BI FOR NOMENCLATURE_GROUP
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_NOMENCLATURE_GROUP_ID, 1);
END;

INSERT INTO NOMENCLATURE_GROUP(ID, NAME, DISCOUNT_1, DISCOUNT_2) VALUES(0, ' ', 0, 0);

CREATE GENERATOR GEN_ED_ID;

CREATE TABLE ED (
    ID             INTEGER NOT NULL,
    NAME           VARCHAR(8) NOT NULL,
    FULL_NAME      VARCHAR(64),
    EDIT_USERS_ID  INTEGER
);

ALTER TABLE ED ADD CONSTRAINT UNQ1_ED UNIQUE (NAME);
ALTER TABLE ED ADD CONSTRAINT PK_ED PRIMARY KEY (ID);
ALTER TABLE ED ADD CONSTRAINT FK_ED_1 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;

CREATE OR ALTER TRIGGER ED_BI FOR ED
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_ED_ID, 1);
END;

CREATE GENERATOR GEN_CURRENCY_ID;

CREATE TABLE CURRENCY (
    ID             INTEGER NOT NULL,
    NAME           VARCHAR(16) NOT NULL,
    PRICE_RUB      DOUBLE PRECISION NOT NULL,
    EDIT_USERS_ID  INTEGER
);

ALTER TABLE CURRENCY ADD CONSTRAINT UNQ1_CURRENCY UNIQUE (NAME);
ALTER TABLE CURRENCY ADD CONSTRAINT PK_CURRENCY PRIMARY KEY (ID);
ALTER TABLE CURRENCY ADD CONSTRAINT FK_CURRENCY_1 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;

CREATE OR ALTER TRIGGER CURRENCY_BI FOR CURRENCY
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_CURRENCY_ID, 1);
END;

INSERT INTO CURRENCY (ID, NAME, PRICE_RUB) VALUES(0, '.', 1);

CREATE GENERATOR GEN_PAKAGE_ID;

CREATE TABLE PAKAGE (
    ID             INTEGER NOT NULL,
    NAME           VARCHAR(64) NOT NULL,
    EDIT_USERS_ID  INTEGER
);

ALTER TABLE PAKAGE ADD CONSTRAINT UNQ1_PAKAGE UNIQUE (NAME);
ALTER TABLE PAKAGE ADD CONSTRAINT PK_PAKAGE PRIMARY KEY (ID);
ALTER TABLE PAKAGE ADD CONSTRAINT FK_PAKAGE_1 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;

CREATE OR ALTER TRIGGER PAKAGE_BI FOR PAKAGE
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_PAKAGE_ID, 1);
END;


CREATE OR ALTER procedure CLEAR_EDIT_USERS_ID (
    ID integer,
    IN_OUT integer,
    PROGRAM_ID integer)
as
BEGIN

  UPDATE SPAM
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE PROJECTS_STATUS_MAIN
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE REMINDER
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    EVENT_PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE_NUMBERS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    REGION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    COUNTRY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TREND
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PERSON
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    JOB_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACTIVITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TASK
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_RESULT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_SOURCE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACTIONS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  IF (IN_OUT = 1)
  THEN
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = NULL
      WHERE
        USERS_ID = :ID;
    END
  ELSE
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = :ID
      WHERE
        ID = :PROGRAM_ID;
    END

  UPDATE
    ORGANIZATION_DISCOUNT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CURRENCY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ED
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PAKAGE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

END;

ALTER TABLE NOMENCLATURE ADD NOMENCLATURE_GROUP_ID INTEGER NOT NULL;
ALTER TABLE NOMENCLATURE ADD PAKAGE_ID INTEGER;
ALTER TABLE NOMENCLATURE ADD ED_ID INTEGER;
ALTER TABLE NOMENCLATURE ADD CURRENCY_ID INTEGER;
ALTER TABLE NOMENCLATURE ADD COUNTRY_ID INTEGER;
ALTER TABLE NOMENCLATURE ADD PRICE DOUBLE PRECISION NOT NULL;
ALTER TABLE NOMENCLATURE ADD ARTICLE VARCHAR(32);
ALTER TABLE NOMENCLATURE ADD MASSA DOUBLE PRECISION NOT NULL;
ALTER TABLE NOMENCLATURE ADD GTD VARCHAR(32);
ALTER TABLE NOMENCLATURE ADD NDS_TYPE INTEGER NOT NULL;
ALTER TABLE NOMENCLATURE ADD NDS DOUBLE PRECISION NOT NULL;
ALTER TABLE NOMENCLATURE ADD FACTOR DOUBLE PRECISION NOT NULL;
ALTER TABLE NOMENCLATURE ADD COMMENTS_1 BLOB SUB_TYPE 1 SEGMENT SIZE 80;
ALTER TABLE NOMENCLATURE ADD COMMENTS_2 BLOB SUB_TYPE 1 SEGMENT SIZE 80;

ALTER TABLE NOMENCLATURE ADD CONSTRAINT FK_NOMENCLATURE_2 FOREIGN KEY (NOMENCLATURE_GROUP_ID) REFERENCES NOMENCLATURE_GROUP (ID);
ALTER TABLE NOMENCLATURE ADD CONSTRAINT FK_NOMENCLATURE_3 FOREIGN KEY (PAKAGE_ID) REFERENCES PAKAGE (ID) ON DELETE SET NULL;
ALTER TABLE NOMENCLATURE ADD CONSTRAINT FK_NOMENCLATURE_4 FOREIGN KEY (ED_ID) REFERENCES ED (ID) ON DELETE SET NULL;
ALTER TABLE NOMENCLATURE ADD CONSTRAINT FK_NOMENCLATURE_5 FOREIGN KEY (CURRENCY_ID) REFERENCES CURRENCY (ID) ON DELETE SET NULL;
ALTER TABLE NOMENCLATURE ADD CONSTRAINT FK_NOMENCLATURE_6 FOREIGN KEY (COUNTRY_ID) REFERENCES COUNTRY (ID) ON DELETE SET NULL;

UPDATE
  NOMENCLATURE
SET
  NOMENCLATURE_GROUP_ID = 0,
  PRICE = 0,
  MASSA = 0,
  NDS_TYPE = 0,
  NDS = 0,
  FACTOR = 0;

EXECUTE PROCEDURE SET_VERSION('3.1.36');