ALTER TABLE PROJECTS ADD DT_END TIMESTAMP;

ALTER TABLE EVENT_PROJECTS ADD PAYMENT_NUM INTEGER;
ALTER TABLE EVENT_PROJECTS ADD PAYMENT_DATE TIMESTAMP;

ALTER TABLE PERSON ADD SMS_SEND INTEGER;
UPDATE PERSON SET SMS_SEND = 0;

CREATE GENERATOR GEN_SMS_OPERATOR_ID;

CREATE TABLE SMS_OPERATOR (
    ID            INTEGER NOT NULL,
    NAME          VARCHAR(64) NOT NULL,
    HTTP_REQUEST  BLOB SUB_TYPE 1 SEGMENT SIZE 80 NOT NULL,
    HTTP_ANSWER   VARCHAR(128),
    STATUS        INTEGER NOT NULL);

ALTER TABLE SMS_OPERATOR ADD CONSTRAINT UNQ1_SMS_OPERATOR UNIQUE (NAME);
ALTER TABLE SMS_OPERATOR ADD CONSTRAINT PK_SMS_OPERATOR PRIMARY KEY (ID);

CREATE OR ALTER TRIGGER SMS_OPERATOR_BI FOR SMS_OPERATOR
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_SMS_OPERATOR_ID, 1);
END;

CREATE GENERATOR GEN_SMS_SPAM_ID;

CREATE TABLE SMS_SPAM (
    ID               INTEGER NOT NULL,
    SMS_OPERATOR_ID  INTEGER NOT NULL,
    USERS_ID         INTEGER NOT NULL,
    NAME             VARCHAR(64) NOT NULL,
    MSG              BLOB SUB_TYPE 1 SEGMENT SIZE 80 NOT NULL,
    DT_CREATE        TIMESTAMP NOT NULL,
    STATUS           INTEGER NOT NULL,
    EDIT_USERS_ID    INTEGER);

ALTER TABLE SMS_SPAM ADD CONSTRAINT UNQ1_SMS_SPAM UNIQUE (NAME);
ALTER TABLE SMS_SPAM ADD CONSTRAINT PK_SMS_SPAM PRIMARY KEY (ID);
ALTER TABLE SMS_SPAM ADD CONSTRAINT FK_SMS_SPAM_1 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;
ALTER TABLE SMS_SPAM ADD CONSTRAINT FK_SMS_SPAM_2 FOREIGN KEY (USERS_ID) REFERENCES USERS (ID) ON DELETE CASCADE;
ALTER TABLE SMS_SPAM ADD CONSTRAINT FK_SMS_SPAM_3 FOREIGN KEY (SMS_OPERATOR_ID) REFERENCES SMS_OPERATOR (ID) ON DELETE CASCADE;

CREATE OR ALTER TRIGGER SMS_SPAM_BI FOR SMS_SPAM
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_SMS_SPAM_ID, 1);
END;

CREATE GENERATOR GEN_SMS_SPAM_TELS_ID;

CREATE TABLE SMS_SPAM_TELS (
    ID           INTEGER NOT NULL,
    SMS_SPAM_ID  INTEGER NOT NULL,
    PERSON_ID    INTEGER NOT NULL,
    STATUS       INTEGER NOT NULL,
    HTTP_ANSWER  BLOB SUB_TYPE 1 SEGMENT SIZE 80);

ALTER TABLE SMS_SPAM_TELS ADD CONSTRAINT UNQ1_SMS_SPAM_TELS UNIQUE (SMS_SPAM_ID, PERSON_ID);
ALTER TABLE SMS_SPAM_TELS ADD CONSTRAINT PK_SMS_SPAM_TELS PRIMARY KEY (ID);
ALTER TABLE SMS_SPAM_TELS ADD CONSTRAINT FK_SMS_SPAM_TELS_1 FOREIGN KEY (SMS_SPAM_ID) REFERENCES SMS_SPAM (ID) ON DELETE CASCADE;
ALTER TABLE SMS_SPAM_TELS ADD CONSTRAINT FK_SMS_SPAM_TELS_2 FOREIGN KEY (PERSON_ID) REFERENCES PERSON (ID) ON DELETE CASCADE;

CREATE OR ALTER TRIGGER SMS_SPAM_TELS_BI FOR SMS_SPAM_TELS
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_SMS_SPAM_TELS_ID, 1);
END;

CREATE OR ALTER procedure CLEAR_EDIT_USERS_ID (
    ID integer,
    IN_OUT integer,
    PROGRAM_ID integer)
as
BEGIN
  UPDATE SPAM
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE PROJECTS_STATUS_MAIN
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE REMINDER
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    EVENT_PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE_NUMBERS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    REGION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    COUNTRY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TREND
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PERSON
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    JOB_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACTIVITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TASK
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_RESULT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_SOURCE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACTIONS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  IF (IN_OUT = 1)
  THEN
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = NULL
      WHERE
        USERS_ID = :ID;
    END
  ELSE
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = :ID
      WHERE
        ID = :PROGRAM_ID;
    END

  UPDATE
    ORGANIZATION_DISCOUNT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CURRENCY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ED
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PAKAGE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CITY_DIRECT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    SMS_SPAM
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;
END;

CREATE OR ALTER procedure GET_SPAM_MAIL (
    SHOW_STAT integer,
    SPAM_ID integer)
returns (
    COUNT_NOW integer,
    COUNT_ALL integer,
    SPAM_MAILS_ID varchar(128),
    PERSON_ORG_MAIL varchar(128),
    PERSON_ORG_NAME varchar(128),
    ORG_NAME varchar(128),
    PERSON_LASTNAME varchar(42),
    PERSON_FIRSNAME varchar(42),
    PERSON_SECONDNAME varchar(42))
as
declare variable PERSON_ID integer;
declare variable ORGANIZATION_ID integer;
BEGIN
  IF (SHOW_STAT = 1)
  THEN
    BEGIN
        SELECT
            COUNT(SPAM_EMAILS.ID)
        FROM
            SPAM_EMAILS
        WHERE
            (SPAM_EMAILS.SPAM_ID = :SPAM_ID) AND
            (SPAM_EMAILS.STATUS <> 0 )
        INTO
        :COUNT_NOW;

        SELECT
            COUNT(SPAM_EMAILS.ID)
        FROM
            SPAM_EMAILS
        WHERE
            (SPAM_EMAILS.SPAM_ID = :SPAM_ID)
        INTO
            :COUNT_ALL;
    END

  FOR
  SELECT
    SPAM_EMAILS.ID,
    SPAM_EMAILS.ORGANIZATION_ID,
    SPAM_EMAILS.PERSON_ID
  FROM
    SPAM_EMAILS
  WHERE
    (SPAM_EMAILS.SPAM_ID =:SPAM_ID) AND
    (SPAM_EMAILS.STATUS = 0)
  INTO
    :SPAM_MAILS_ID,
    :ORGANIZATION_ID,
    :PERSON_ID
  DO
    BREAK;

  IF (SPAM_MAILS_ID IS NULL)
  THEN
    BEGIN
        SUSPEND;
        EXIT;
    END

  IF (PERSON_ID IS NULL)
  THEN
    BEGIN
        SELECT
            ORGANIZATION.NAME,
            ORGANIZATION.EMAIL,
            ORGANIZATION.NAME
        FROM
            ORGANIZATION
        WHERE
            ORGANIZATION.ID =:ORGANIZATION_ID
        INTO
            :PERSON_ORG_NAME,
            :PERSON_ORG_MAIL,
            :ORG_NAME;
    END
  ELSE
    BEGIN

        SELECT
            PERSON.FIO,
            PERSON.LASTNAME,
            PERSON.FIRSTNAME,
            PERSON.SECONDNAME,
            PERSON.EMAIL,
            ORGANIZATION.NAME
        FROM
            PERSON
            INNER JOIN ORGANIZATION ON ORGANIZATION.ID =PERSON.ORGANIZATION_ID
        WHERE
            PERSON.ID =:PERSON_ID
        INTO
            :PERSON_ORG_NAME,
            :PERSON_LASTNAME,
            :PERSON_FIRSNAME,
            :PERSON_SECONDNAME,
            :PERSON_ORG_MAIL,
            :ORG_NAME;
    END

  SUSPEND;
END;

EXECUTE PROCEDURE SET_VERSION('3.1.39');