ALTER TABLE SPAM ADD MAIL_FROM VARCHAR(128) NOT NULL;
ALTER TABLE SPAM ADD ADD_CONTACT INTEGER NOT NULL;
ALTER TABLE SPAM ADD CONTACT_TYPE_ID INTEGER;
ALTER TABLE SPAM ADD CONTACT_RESULT_ID INTEGER;
ALTER TABLE SPAM ADD NOMENCLATURE_ID INTEGER;

ALTER TABLE SPAM ADD CONSTRAINT FK_SPAM_3 FOREIGN KEY (CONTACT_TYPE_ID) REFERENCES CONTACT_TYPE (ID) ON DELETE SET NULL;
ALTER TABLE SPAM ADD CONSTRAINT FK_SPAM_4 FOREIGN KEY (CONTACT_RESULT_ID) REFERENCES CONTACT_RESULT (ID) ON DELETE SET NULL;
ALTER TABLE SPAM ADD CONSTRAINT FK_SPAM_5 FOREIGN KEY (NOMENCLATURE_ID) REFERENCES NOMENCLATURE (ID) ON DELETE SET NULL;

UPDATE
  SPAM
SET
  STATUS = 0,
  ADD_CONTACT = 0,
  MAIL_FROM = ' ';

ALTER TABLE SPAM_EMAILS ADD ERROR_SMTP BLOB SUB_TYPE 1 SEGMENT SIZE 80;

CREATE OR ALTER procedure SELECT_SPAM_EMAILS
returns (
    ID integer,
    EMAIL_SEND integer,
    EMAIL varchar(64),
    ORGANIZATION_NAME varchar(128),
    BRIEF_NAME varchar(8),
    POST_INDEX varchar(16),
    ADDRESS blob sub_type 1 segment size 80,
    COUNTRY_NAME varchar(64),
    REGION_NAME varchar(64),
    CITY_NAME varchar(64),
    FIO varchar(128),
    JOB_STATUS_NAME varchar(64),
    JOBTITLE varchar(128),
    LASTNAME varchar(42),
    FIRSTNAME varchar(42),
    SECONDNAME varchar(42),
    TEL varchar(64),
    SEX integer,
    SPAM_ID integer,
    STATUS integer,
    ERROR_SMTP blob sub_type 1 segment size 80)
as
declare variable ORGANIZATION_ID integer;
declare variable PERSON_ID integer;
BEGIN
    FOR
        SELECT
            SPAM_EMAILS.ID,
            SPAM_EMAILS.ORGANIZATION_ID,
            SPAM_EMAILS.PERSON_ID,
            SPAM_EMAILS.STATUS,
            SPAM_EMAILS.SPAM_ID,
            SPAM_EMAILS.ERROR_SMTP
        FROM
            SPAM_EMAILS
        INTO
            :ID,
            :ORGANIZATION_ID,
            :PERSON_ID,
            :STATUS,
            :SPAM_ID,
            :ERROR_SMTP
    DO
        BEGIN
          SELECT
            ORGANIZATION.NAME,
            ORGANIZATION.EMAIL,
            ORGANIZATION.EMAIL_SEND,
            ORGANIZATION.TEL,
            PROPERTY.BRIEF_NAME,
            POSTAL_ADDRESS.POST_INDEX,
            POSTAL_ADDRESS.ADDRESS,
            COUNTRY.NAME,
            REGION.NAME,
            CITY.NAME
          FROM
            ORGANIZATION
            LEFT JOIN PROPERTY ON PROPERTY.ID = ORGANIZATION.PROPERTY_ID
            LEFT JOIN ADDRESS POSTAL_ADDRESS ON POSTAL_ADDRESS.ID = ORGANIZATION.POSTAL_ADDRESS_ID
            LEFT JOIN COUNTRY ON COUNTRY.ID = POSTAL_ADDRESS.COUNTRY_ID
            LEFT JOIN REGION ON REGION.ID = POSTAL_ADDRESS.REGION_ID
            LEFT JOIN CITY ON CITY.ID = POSTAL_ADDRESS.CITY_ID
          WHERE
            ORGANIZATION.ID = :ORGANIZATION_ID
          INTO
            :ORGANIZATION_NAME,
            :EMAIL,
            :EMAIL_SEND,
            :TEL,
            :BRIEF_NAME,
            :POST_INDEX,
            :ADDRESS,
            :COUNTRY_NAME,
            :REGION_NAME,
            :CITY_NAME;

          IF (PERSON_ID IS NOT NULL)
          THEN
            BEGIN
              SELECT
                PERSON.JOBTITLE,
                PERSON.FIO,
                PERSON.LASTNAME,
                PERSON.FIRSTNAME,
                PERSON.SECONDNAME,
                PERSON.EMAIL,
                PERSON.EMAIL_SEND,
                PERSON.TEL_JOB,
                PERSON.SEX,
                JOB_STATUS.NAME
              FROM
                PERSON
                LEFT JOIN JOB_STATUS ON JOB_STATUS.ID = PERSON.JOB_STATUS_ID
              WHERE
                PERSON.ID = :PERSON_ID
              INTO
                :JOBTITLE,
                :FIO,
                :LASTNAME,
                :FIRSTNAME,
                :SECONDNAME,
                :EMAIL,
                :EMAIL_SEND,
                :TEL,
                :SEX,
                :JOB_STATUS_NAME;
            END
          ELSE
            BEGIN
              JOBTITLE = NULL;
              FIO = NULL;
              LASTNAME = NULL;
              FIRSTNAME = NULL;
              SECONDNAME = NULL;
              SEX = NULL;
              JOB_STATUS_NAME = NULL;
            END

          SUSPEND;
        END
END;

CREATE OR ALTER procedure GET_SPAM_MAIL (
    SHOW_STAT integer,
    SPAM_ID integer)
returns (
    COUNT_NOW integer,
    COUNT_ALL integer,
    SPAM_MAILS_ID integer,
    PERSON_ORG_MAIL varchar(128),
    PERSON_ORG_NAME varchar(128),
    ORG_NAME varchar(128),
    PERSON_ID integer,
    PERSON_LASTNAME varchar(42),
    PERSON_FIRSNAME varchar(42),
    PERSON_SECONDNAME varchar(42))
as
declare variable ORGANIZATION_ID integer;
BEGIN
  IF (SHOW_STAT = 1)
  THEN
    BEGIN
        SELECT
            COUNT(SPAM_EMAILS.ID)
        FROM
            SPAM_EMAILS
        WHERE
            (SPAM_EMAILS.SPAM_ID = :SPAM_ID) AND
            (SPAM_EMAILS.STATUS <> 0 )
        INTO
        :COUNT_NOW;

        SELECT
            COUNT(SPAM_EMAILS.ID)
        FROM
            SPAM_EMAILS
        WHERE
            (SPAM_EMAILS.SPAM_ID = :SPAM_ID)
        INTO
            :COUNT_ALL;
    END

  FOR
  SELECT
    SPAM_EMAILS.ID,
    SPAM_EMAILS.ORGANIZATION_ID,
    SPAM_EMAILS.PERSON_ID
  FROM
    SPAM_EMAILS
  WHERE
    (SPAM_EMAILS.SPAM_ID =:SPAM_ID) AND
    (SPAM_EMAILS.STATUS = 0)
  INTO
    :SPAM_MAILS_ID,
    :ORGANIZATION_ID,
    :PERSON_ID
  DO
    BREAK;

  IF (SPAM_MAILS_ID IS NULL)
  THEN
    BEGIN
        SUSPEND;
        EXIT;
    END

  IF (PERSON_ID IS NULL)
  THEN
    BEGIN
        PERSON_ID = -1;
        SELECT
            ORGANIZATION.NAME,
            ORGANIZATION.EMAIL,
            ORGANIZATION.NAME
        FROM
            ORGANIZATION
        WHERE
            ORGANIZATION.ID =:ORGANIZATION_ID
        INTO
            :PERSON_ORG_NAME,
            :PERSON_ORG_MAIL,
            :ORG_NAME;
    END
  ELSE
    BEGIN
        SELECT
            PERSON.FIO,
            PERSON.LASTNAME,
            PERSON.FIRSTNAME,
            PERSON.SECONDNAME,
            PERSON.EMAIL,
            ORGANIZATION.NAME
        FROM
            PERSON
            INNER JOIN ORGANIZATION ON ORGANIZATION.ID = PERSON.ORGANIZATION_ID
        WHERE
            PERSON.ID =:PERSON_ID
        INTO
            :PERSON_ORG_NAME,
            :PERSON_LASTNAME,
            :PERSON_FIRSNAME,
            :PERSON_SECONDNAME,
            :PERSON_ORG_MAIL,
            :ORG_NAME;
    END

  SUSPEND;
END;

ALTER TABLE CONTACT_TYPE ADD MAIN_TYPE INTEGER NOT NULL;
UPDATE CONTACT_TYPE SET MAIN_TYPE = 0;

CREATE OR ALTER TRIGGER CONTACT_TYPE_BU FOR CONTACT_TYPE
ACTIVE BEFORE UPDATE POSITION 0
AS
begin
  IF (NEW.MAIN_TYPE = 1)
  THEN
    BEGIN
      UPDATE
        CONTACT_TYPE
      SET
        MAIN_TYPE = 0
      WHERE
        ID <> OLD.ID;
    END
end;

ALTER TABLE CONTACT_RESULT ADD MAIN_RESULT INTEGER NOT NULL;
UPDATE CONTACT_RESULT SET MAIN_RESULT = 0;

CREATE OR ALTER TRIGGER CONTACT_RESULT_BU FOR CONTACT_RESULT
ACTIVE BEFORE UPDATE POSITION 0
AS
begin
  IF (NEW.MAIN_RESULT = 1)
  THEN
    BEGIN
      UPDATE
        CONTACT_RESULT
      SET
        MAIN_RESULT = 0
      WHERE
        ID <> OLD.ID;
    END
end;

create or alter procedure EXCHANGE_ORGANIZATION (
    ORGANIZATION_ID integer,
    OLD_OWNER_ID integer,
    NEW_OWNER_ID integer,
    PERSONS_ACTION integer,
    AGREEMENT_ACTION integer,
    DISCOUNT_ACTION integer,
    COTACTS_ACTION integer)
as
begin
  UPDATE ORGANIZATION
  SET
    USERS_ID = :NEW_OWNER_ID,
    USERS_BUSY_ID = NULL
  WHERE
    (ID = :ORGANIZATION_ID);

  IF (:PERSONS_ACTION > 0)
  THEN
    BEGIN
      UPDATE
        PERSON
      SET
        USERS_ID = :NEW_OWNER_ID
      WHERE
       (ORGANIZATION_ID = :ORGANIZATION_ID)
       AND
       ((:PERSONS_ACTION = 2) OR (USERS_ID = :OLD_OWNER_ID));
    END

  IF (:AGREEMENT_ACTION > 0)
  THEN
    BEGIN
      UPDATE
        AGREEMENT
      SET
        USERS_ID = :NEW_OWNER_ID
      WHERE
       (ORGANIZATION_ID = :ORGANIZATION_ID)
       AND
       ((:AGREEMENT_ACTION = 2) OR (USERS_ID = :OLD_OWNER_ID));
    END

  IF (:DISCOUNT_ACTION > 0)
  THEN
    BEGIN
      UPDATE
        ORGANIZATION_DISCOUNT
      SET
        USERS_ID = :NEW_OWNER_ID
      WHERE
       (ORGANIZATION_ID = :ORGANIZATION_ID)
       AND
       ((:DISCOUNT_ACTION = 2) OR (USERS_ID = :OLD_OWNER_ID));
    END

  IF (:COTACTS_ACTION > 0)
  THEN
    BEGIN
      UPDATE
        CONTACT
      SET
        USERS_ID = :NEW_OWNER_ID
      WHERE
       (PERSON_ID IN (SELECT ID FROM PERSON WHERE ORGANIZATION_ID = :ORGANIZATION_ID))
       AND
       ((:COTACTS_ACTION = 2) OR (USERS_ID = :OLD_OWNER_ID));
    END
end;

CREATE GENERATOR GEN_EVENT_PROJECTS_SOURCE_PAY;

CREATE TABLE EVENT_PROJECTS_SOURCE_PAY (
    ID             INTEGER NOT NULL,
    NAME           VARCHAR(128) NOT NULL,
    EDIT_USERS_ID  INTEGER
);

ALTER TABLE EVENT_PROJECTS_SOURCE_PAY ADD CONSTRAINT UNQ1_EVENT_PROJECTS_SOURCE_PAY UNIQUE (NAME);
ALTER TABLE EVENT_PROJECTS_SOURCE_PAY ADD CONSTRAINT PK_EVENT_PROJECTS_SOURCE_PAY PRIMARY KEY (ID);
ALTER TABLE EVENT_PROJECTS_SOURCE_PAY ADD CONSTRAINT FK_EVENT_PROJECTS_SOURCE_PAY_1 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;

CREATE OR ALTER TRIGGER EVENT_PROJECTS_SOURCE_PAY_BI FOR EVENT_PROJECTS_SOURCE_PAY
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_EVENT_PROJECTS_SOURCE_PAY, 1);
END;

ALTER TABLE EVENT_PROJECTS DROP CONSTRAINT FK_EVENT_PROJECTS_2;
ALTER TABLE EVENT_PROJECTS ADD CONSTRAINT FK_EVENT_PROJECTS_2 FOREIGN KEY (NOMENCLATURE_ID) REFERENCES NOMENCLATURE(ID) ON DELETE SET NULL USING INDEX FK_EVENT_PROJECTS_2;

ALTER TABLE EVENT_PROJECTS ADD EVENT_PROJECTS_SOURCE_PAY_ID INTEGER;
ALTER TABLE EVENT_PROJECTS ADD CONSTRAINT FK_EVENT_PROJECTS_7 FOREIGN KEY (EVENT_PROJECTS_SOURCE_PAY_ID) REFERENCES EVENT_PROJECTS_SOURCE_PAY (ID) ON DELETE SET NULL;

CREATE OR ALTER procedure CLEAR_EDIT_USERS_ID (
    ID integer,
    IN_OUT integer,
    PROGRAM_ID integer)
as
BEGIN
  UPDATE SPAM
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE PROJECTS_STATUS_MAIN
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE REMINDER
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    EVENT_PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE_NUMBERS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    REGION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    COUNTRY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TREND
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PERSON
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    JOB_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACTIVITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TASK
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_RESULT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_SOURCE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACTIONS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  IF (IN_OUT = 1)
  THEN
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = NULL
      WHERE
        USERS_ID = :ID;
    END
  ELSE
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = :ID
      WHERE
        ID = :PROGRAM_ID;
    END

  UPDATE
    ORGANIZATION_DISCOUNT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CURRENCY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ED
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PAKAGE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CITY_DIRECT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    SMS_SPAM
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    EVENT_PROJECTS_SOURCE_PAY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

END;

CREATE GENERATOR GEN_FILTER_SHABLON_ID;

CREATE TABLE FILTER_SHABLON (
    ID             INTEGER NOT NULL,
    USERS_ID       INTEGER NOT NULL,
    NAME           VARCHAR(128) NOT NULL,
    SHABLON        BLOB SUB_TYPE 1 SEGMENT SIZE 80 NOT NULL,
    SHABLON_DEF    INTEGER NOT NULL,
    SHABLON_POINT  INTEGER NOT NULL
);

ALTER TABLE FILTER_SHABLON ADD CONSTRAINT UNQ1_FILTER_SHABLON UNIQUE (NAME, USERS_ID);
ALTER TABLE FILTER_SHABLON ADD CONSTRAINT PK_FILTER_SHABLON PRIMARY KEY (ID);
ALTER TABLE FILTER_SHABLON ADD CONSTRAINT FK_FILTER_SHABLON_1 FOREIGN KEY (USERS_ID) REFERENCES USERS (ID) ON DELETE CASCADE;

CREATE OR ALTER TRIGGER FILTER_SHABLON_BI FOR FILTER_SHABLON
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_FILTER_SHABLON_ID, 1);
END;

CREATE OR ALTER TRIGGER FILTER_SHABLON_BU FOR FILTER_SHABLON
ACTIVE BEFORE UPDATE POSITION 0
AS
begin
  IF (NEW.SHABLON_DEF = 1)
  THEN
    BEGIN
      UPDATE
        FILTER_SHABLON
      SET
        SHABLON_DEF = 0
      WHERE
        ID <> OLD.ID;
    END
end;

CREATE GENERATOR GEN_PLANS_ID;

CREATE TABLE PLANS (
    ID          INTEGER NOT NULL,
    USERS_ID    INTEGER NOT NULL,
    DT_BEGIN    TIMESTAMP NOT NULL,
    DT_END      TIMESTAMP NOT NULL,
    NAME        VARCHAR(128) NOT NULL,
    COMMENTS    BLOB SUB_TYPE 1 SEGMENT SIZE 80,
    STATUS_FIX  DATE,
    PLAN_TYPE   INTEGER NOT NULL
);

ALTER TABLE PLANS ADD CONSTRAINT UNQ1_PLANS UNIQUE (USERS_ID, NAME);
ALTER TABLE PLANS ADD CONSTRAINT PK_PLANS PRIMARY KEY (ID);
ALTER TABLE PLANS ADD CONSTRAINT FK_PLANS_1 FOREIGN KEY (USERS_ID) REFERENCES USERS (ID) ON DELETE CASCADE;

CREATE OR ALTER TRIGGER PLANS_BI FOR PLANS
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_PLANS_ID, 1);
END;

CREATE GENERATOR GEN_PLANS_EVENT_ID;

CREATE TABLE PLANS_EVENT (
    ID               INTEGER NOT NULL,
    PLANS_ID         INTEGER NOT NULL,
    ORGANIZATION_ID  INTEGER NOT NULL,
    DT_BEGIN         TIMESTAMP NOT NULL,
    DT_END           TIMESTAMP NOT NULL,
    DATE_PAY         DATE,
    STATUS           INTEGER NOT NULL,
    COMMENTS_BEFORE  BLOB SUB_TYPE 1 SEGMENT SIZE 80 NOT NULL,
    COMMENTS_AFTER   BLOB SUB_TYPE 1 SEGMENT SIZE 80,
    STATUS_FIX       DATE
);

ALTER TABLE PLANS_EVENT ADD CONSTRAINT PK_PLANS_EVENT PRIMARY KEY (ID);
ALTER TABLE PLANS_EVENT ADD CONSTRAINT FK_PLANS_EVENT_1 FOREIGN KEY (PLANS_ID) REFERENCES PLANS (ID) ON DELETE CASCADE;
ALTER TABLE PLANS_EVENT ADD CONSTRAINT FK_PLANS_EVENT_2 FOREIGN KEY (ORGANIZATION_ID) REFERENCES ORGANIZATION (ID) ON DELETE CASCADE;

CREATE OR ALTER TRIGGER PLANS_EVENT_BI FOR PLANS_EVENT
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_PLANS_EVENT_ID, 1);
END;

CREATE GENERATOR GEN_PLANS_NOMENCLATURE_ID;

CREATE TABLE PLANS_NOMENCLATURE (
    ID               INTEGER NOT NULL,
    PLANS_EVENT_ID   INTEGER NOT NULL,
    NOMENCLATURE_ID  INTEGER NOT NULL,
    NOMENC_COUNT     INTEGER NOT NULL,
    NOMENC_VOLUME    DOUBLE PRECISION NOT NULL,
    COMMENTS_BEFORE  BLOB SUB_TYPE 1 SEGMENT SIZE 80,
    COMMENTS_AFTER   BLOB SUB_TYPE 1 SEGMENT SIZE 80
);

ALTER TABLE PLANS_NOMENCLATURE ADD CONSTRAINT UNQ1_PLANS_NOMENCLATURE UNIQUE (PLANS_EVENT_ID, NOMENCLATURE_ID);
ALTER TABLE PLANS_NOMENCLATURE ADD CONSTRAINT PK_PLANS_NOMENCLATURE PRIMARY KEY (ID);
ALTER TABLE PLANS_NOMENCLATURE ADD CONSTRAINT FK_PLANS_NOMENCLATURE_1 FOREIGN KEY (PLANS_EVENT_ID) REFERENCES PLANS_EVENT (ID) ON DELETE CASCADE;
ALTER TABLE PLANS_NOMENCLATURE ADD CONSTRAINT FK_PLANS_NOMENCLATURE_2 FOREIGN KEY (NOMENCLATURE_ID) REFERENCES NOMENCLATURE (ID) ON DELETE CASCADE;

CREATE OR ALTER TRIGGER PLANS_NOMENCLATURE_BI FOR PLANS_NOMENCLATURE
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_PLANS_NOMENCLATURE_ID, 1);
END;

CREATE OR ALTER procedure ADD_PLAN (
    SOURCE_PLAN_ID integer,
    RECEIVER_PLAN_ID integer,
    ADD_PLAN_NOMENCLATURE integer)
as
declare variable SOURCE_PLANS_EVENT_ID integer;
declare variable RECEIVER_PLANS_EVENT_ID integer;
declare variable ORGANIZATION_ID integer;
declare variable DT_BEGIN timestamp;
declare variable DT_END timestamp;
declare variable DATE_PAY date;
declare variable STATUS integer;
declare variable COMMENTS_BEFORE blob sub_type 1 segment size 80;
declare variable COMMENTS_AFTER blob sub_type 1 segment size 80;
declare variable STATUS_FIX date;
declare variable NOMENCLATURE_ID integer;
declare variable NOMENC_COUNT integer;
declare variable NOMENC_VOLUME double precision;
declare variable NOMENC_COMMENTS_BEFORE blob sub_type 1 segment size 80;
declare variable NOMENC_COMMENTS_AFTER blob sub_type 1 segment size 80;
BEGIN
  FOR
    SELECT
      ID,
      ORGANIZATION_ID,
      DT_BEGIN,
      DT_END,
      DATE_PAY,
      STATUS,
      COMMENTS_BEFORE,
      COMMENTS_AFTER,
      STATUS_FIX
    FROM
      PLANS_EVENT
    WHERE
      PLANS_ID = :SOURCE_PLAN_ID
    INTO
      :SOURCE_PLANS_EVENT_ID,
      :ORGANIZATION_ID,
      :DT_BEGIN,
      :DT_END,
      :DATE_PAY,
      :STATUS,
      :COMMENTS_BEFORE,
      :COMMENTS_AFTER,
      :STATUS_FIX
  DO
    BEGIN

      RECEIVER_PLANS_EVENT_ID = GEN_ID(GEN_PLANS_EVENT_ID, 1);
      INSERT INTO
         PLANS_EVENT
         (ID,
          PLANS_ID,
          ORGANIZATION_ID,
          DT_BEGIN,
          DT_END,
          DATE_PAY,
          STATUS,
          COMMENTS_BEFORE,
          COMMENTS_AFTER,
          STATUS_FIX)
      VALUES
         (:RECEIVER_PLANS_EVENT_ID,
          :RECEIVER_PLAN_ID,
          :ORGANIZATION_ID,
          :DT_BEGIN,
          :DT_END,
          :DATE_PAY,
          :STATUS,
          :COMMENTS_BEFORE,
          :COMMENTS_AFTER,
          :STATUS_FIX);

       IF (:ADD_PLAN_NOMENCLATURE = 1)
       THEN
         BEGIN
           FOR
             SELECT
               NOMENCLATURE_ID,
               NOMENC_COUNT,
               NOMENC_VOLUME,
               COMMENTS_BEFORE,
               COMMENTS_AFTER
             FROM
               PLANS_NOMENCLATURE
             WHERE
               PLANS_EVENT_ID = :SOURCE_PLANS_EVENT_ID
             INTO
               :NOMENCLATURE_ID,
               :NOMENC_COUNT,
               :NOMENC_VOLUME,
               :NOMENC_COMMENTS_BEFORE,
               :NOMENC_COMMENTS_AFTER
           DO
             BEGIN
               INSERT INTO
                 PLANS_NOMENCLATURE
                 (PLANS_EVENT_ID,
                  NOMENCLATURE_ID,
                  NOMENC_COUNT,
                  NOMENC_VOLUME,
                  COMMENTS_BEFORE,
                  COMMENTS_AFTER)
               VALUES
                 (:RECEIVER_PLANS_EVENT_ID,
                  :NOMENCLATURE_ID,
                  :NOMENC_COUNT,
                  :NOMENC_VOLUME,
                  :NOMENC_COMMENTS_BEFORE,
                  :NOMENC_COMMENTS_AFTER);
             END
         END
    END
END;

CREATE OR ALTER procedure PLAN_TABLE (
    DT_BEGIN timestamp,
    DT_END timestamp,
    PLAN_TYPE integer)
returns (
    USERS_ID integer,
    FIO varchar(128),
    JOBTITLE varchar(129),
    DAYS_WORK integer,
    DAYS_KOMAND integer,
    COUNT_KOMAND integer,
    PERCENT double precision,
    ORG_COUNT integer,
    ORG_STATUS_0 integer,
    ORG_STATUS_1 integer,
    ORG_STATUS_2 integer,
    ORG_STATUS_3 integer)
as
declare variable PLAN_DT_BEGIN timestamp;
declare variable PLAN_DT_END timestamp;
declare variable PLAN_ID integer;
declare variable STATUS integer;
BEGIN
  DAYS_WORK = (DT_END - DT_BEGIN + 1) / 7 * 5;
  FOR
    SELECT DISTINCT
      PLANS.USERS_ID,
      USERS.FIO,
      USERS.JOBTITLE
    FROM
      PLANS
      LEFT JOIN USERS ON USERS.ID = PLANS.USERS_ID
    WHERE
      (((PLANS.DT_BEGIN <= :DT_BEGIN) AND (:DT_END <= PLANS.DT_END)) OR
       ((PLANS.DT_BEGIN <= :DT_BEGIN) AND (:DT_BEGIN <= PLANS.DT_END)) OR
       ((PLANS.DT_BEGIN <= :DT_END) AND (:DT_END <= PLANS.DT_END)) OR
       ((:DT_BEGIN <= PLANS.DT_BEGIN) AND (PLANS.DT_END <= :DT_END)))
      AND
      ((:PLAN_TYPE = -1) OR (PLANS.PLAN_TYPE = :PLAN_TYPE))
    INTO
      :USERS_ID,
      :FIO,
      :JOBTITLE
  DO
    BEGIN
      /******************** 1 ********************/
      DAYS_KOMAND = 0;
      COUNT_KOMAND = 0;
      FOR
        SELECT
          PLANS.ID,
          PLANS.DT_BEGIN,
          PLANS.DT_END
        FROM
          PLANS
        WHERE
          (PLANS.USERS_ID = :USERS_ID)
          AND
          (((PLANS.DT_BEGIN <= :DT_BEGIN) AND (:DT_END <= PLANS.DT_END)) OR
           ((PLANS.DT_BEGIN <= :DT_BEGIN) AND (:DT_BEGIN <= PLANS.DT_END)) OR
           ((PLANS.DT_BEGIN <= :DT_END) AND (:DT_END <= PLANS.DT_END)) OR
           ((:DT_BEGIN <= PLANS.DT_BEGIN) AND (PLANS.DT_END <= :DT_END)))
          AND
          ((:PLAN_TYPE = -1) OR (PLANS.PLAN_TYPE = :PLAN_TYPE))
        INTO
          :PLAN_ID,
          :PLAN_DT_BEGIN,
          :PLAN_DT_END
        DO
          BEGIN
            COUNT_KOMAND = COUNT_KOMAND + 1;

            IF ((:PLAN_DT_BEGIN <= :DT_BEGIN) AND (:DT_END <= :PLAN_DT_END))
            THEN
              DAYS_KOMAND = DAYS_KOMAND + (:DT_END - :DT_BEGIN + 1);
            ELSE
              IF ((:PLAN_DT_BEGIN <= :DT_BEGIN) AND (:DT_BEGIN <= :PLAN_DT_END))
              THEN
                DAYS_KOMAND = DAYS_KOMAND + (:PLAN_DT_END - :DT_BEGIN + 1);
              ELSE
                IF ((:PLAN_DT_BEGIN <= :DT_END) AND (:DT_END <= :PLAN_DT_END))
                THEN
                  DAYS_KOMAND = DAYS_KOMAND + (:DT_END - :PLAN_DT_BEGIN + 1);
                ELSE
                  DAYS_KOMAND = DAYS_KOMAND + (:PLAN_DT_END - :PLAN_DT_BEGIN + 1);
          END
      /***************************************** 2 **********************/
      ORG_COUNT = 0;
      ORG_STATUS_0 = 0;
      ORG_STATUS_1 = 0;
      ORG_STATUS_2 = 0;
      ORG_STATUS_3 = 0;
      FOR
        SELECT
          PLANS_EVENT.STATUS
        FROM
          PLANS
          LEFT JOIN PLANS_EVENT ON PLANS_EVENT.PLANS_ID = PLANS.ID
        WHERE
          (PLANS.USERS_ID = :USERS_ID)
          AND
          ((:PLAN_TYPE = -1) OR (PLANS.PLAN_TYPE = :PLAN_TYPE))
          AND
          (((PLANS_EVENT.DT_BEGIN <= :DT_BEGIN) AND (:DT_END <= PLANS_EVENT.DT_END)) OR
           ((PLANS_EVENT.DT_BEGIN <= :DT_BEGIN) AND (:DT_BEGIN <= PLANS_EVENT.DT_END)) OR
           ((PLANS_EVENT.DT_BEGIN <= :DT_END) AND (:DT_END <= PLANS_EVENT.DT_END)) OR
           ((:DT_BEGIN <= PLANS_EVENT.DT_BEGIN) AND (PLANS_EVENT.DT_END <= :DT_END)))
        INTO
          :STATUS
      DO
        BEGIN
          ORG_COUNT = ORG_COUNT + 1;

          IF (:STATUS = 0)
          THEN
            ORG_STATUS_0 = ORG_STATUS_0 + 1;
          ELSE
            IF (:STATUS = 1)
            THEN
              ORG_STATUS_1 = ORG_STATUS_1 + 1;
            ELSE
              IF (:STATUS = 2)
              THEN
                ORG_STATUS_2 = ORG_STATUS_2 + 1;
              ELSE
                ORG_STATUS_3 = ORG_STATUS_3 + 1;
        END
      /******************************************************************/
      PERCENT = DAYS_KOMAND * 100 / DAYS_WORK;
      SUSPEND;
    END
END;

CREATE GENERATOR GEN_NOMENCLATURE_RIVAL_ID;

CREATE TABLE NOMENCLATURE_RIVAL (
    ID               INTEGER NOT NULL,
    NOMENCLATURE_ID  INTEGER NOT NULL,
    ORGANIZATION_ID  INTEGER NOT NULL,
    PRICE            DOUBLE PRECISION NOT NULL,
    COMMENTS         BLOB SUB_TYPE 1 SEGMENT SIZE 80
);

ALTER TABLE NOMENCLATURE_RIVAL ADD CONSTRAINT UNQ1_NOMENCLATURE_RIVAL UNIQUE (NOMENCLATURE_ID, ORGANIZATION_ID);
ALTER TABLE NOMENCLATURE_RIVAL ADD CONSTRAINT PK_NOMENCLATURE_RIVAL PRIMARY KEY (ID);
ALTER TABLE NOMENCLATURE_RIVAL ADD CONSTRAINT FK_NOMENCLATURE_RIVAL_1 FOREIGN KEY (NOMENCLATURE_ID) REFERENCES NOMENCLATURE (ID) ON DELETE CASCADE;
ALTER TABLE NOMENCLATURE_RIVAL ADD CONSTRAINT FK_NOMENCLATURE_RIVAL_2 FOREIGN KEY (ORGANIZATION_ID) REFERENCES ORGANIZATION (ID) ON DELETE CASCADE;

CREATE OR ALTER TRIGGER NOMENCLATURE_RIVAL_BI FOR NOMENCLATURE_RIVAL
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_NOMENCLATURE_RIVAL_ID, 1);
END;

CREATE GENERATOR GEN_CONTACT_NOMENCLATURE_ID;

CREATE TABLE CONTACT_NOMENCLATURE (
    ID               INTEGER NOT NULL,
    CONTACT_ID       INTEGER NOT NULL,
    NOMENCLATURE_ID  INTEGER NOT NULL,
    NOMENC_COUNT     INTEGER NOT NULL
);

ALTER TABLE CONTACT_NOMENCLATURE ADD CONSTRAINT UNQ1_CONTACT_NOMENCLATURE UNIQUE (CONTACT_ID, NOMENCLATURE_ID);
ALTER TABLE CONTACT_NOMENCLATURE ADD CONSTRAINT PK_CONTACT_NOMENCLATURE PRIMARY KEY (ID);
ALTER TABLE CONTACT_NOMENCLATURE ADD CONSTRAINT FK_CONTACT_NOMENCLATURE_1 FOREIGN KEY (CONTACT_ID) REFERENCES CONTACT (ID) ON DELETE CASCADE;
ALTER TABLE CONTACT_NOMENCLATURE ADD CONSTRAINT FK_CONTACT_NOMENCLATURE_2 FOREIGN KEY (NOMENCLATURE_ID) REFERENCES NOMENCLATURE (ID) ON DELETE CASCADE;

CREATE OR ALTER TRIGGER CONTACT_NOMENCLATURE_BI FOR CONTACT_NOMENCLATURE
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_CONTACT_NOMENCLATURE_ID, 1);
END;

EXECUTE PROCEDURE SET_VERSION('3.1.40');