CREATE GENERATOR GEN_FILTER_TREE_ID;

CREATE TABLE FILTER_TREE (
    ID              INTEGER NOT NULL,
    NAME            VARCHAR(128) NOT NULL,
    FILTER_TREE_ID  INTEGER
);

ALTER TABLE FILTER_TREE ADD CONSTRAINT UNQ1_FILTER_TREE UNIQUE (NAME, FILTER_TREE_ID);
ALTER TABLE FILTER_TREE ADD CONSTRAINT PK_FILTER_TREE PRIMARY KEY (ID);
ALTER TABLE FILTER_TREE ADD CONSTRAINT FK_FILTER_TREE_1 FOREIGN KEY (FILTER_TREE_ID) REFERENCES FILTER_TREE (ID) ON DELETE CASCADE;

CREATE OR ALTER TRIGGER FILTER_TREE_BI FOR FILTER_TREE
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_FILTER_TREE_ID, 1);
END;

CREATE TABLE FILTER_TREE_ORGANIZATION (
    FILTER_TREE_ID   INTEGER NOT NULL,
    ORGANIZATION_ID  INTEGER NOT NULL
);

ALTER TABLE FILTER_TREE_ORGANIZATION ADD CONSTRAINT UNQ1_FILTER_TREE_ORGANIZATION UNIQUE (FILTER_TREE_ID, ORGANIZATION_ID);
ALTER TABLE FILTER_TREE_ORGANIZATION ADD CONSTRAINT FK_FILTER_TREE_ORGANIZATION_1 FOREIGN KEY (FILTER_TREE_ID) REFERENCES FILTER_TREE (ID) ON DELETE CASCADE;
ALTER TABLE FILTER_TREE_ORGANIZATION ADD CONSTRAINT FK_FILTER_TREE_ORGANIZATION_2 FOREIGN KEY (ORGANIZATION_ID) REFERENCES ORGANIZATION (ID) ON DELETE CASCADE;

create or alter procedure AE_FILTER_TREE (
    ID integer,
    NAME varchar(64),
    FILTER_TREE_ID integer)
returns (
    ADD_ID integer)
as
begin
  if (ID > 0)
  then
    begin
      UPDATE
        FILTER_TREE
      SET
        NAME = :NAME
      WHERE
        ID = :ID;
    end
  else
    begin
      ADD_ID = GEN_ID(GEN_FILTER_TREE_ID, 1);
      INSERT INTO
        FILTER_TREE
        (ID, NAME, FILTER_TREE_ID)
      VALUES
        (:ADD_ID, :NAME, :FILTER_TREE_ID);
      SUSPEND;
    end
end;

EXECUTE PROCEDURE SET_VERSION('3.1.42');
