CREATE GENERATOR GEN_ACCOUNT_GROUP_ID;

CREATE TABLE ACCOUNT_GROUP (
    ID                      INTEGER NOT NULL,
    USERS_ID                INTEGER NOT NULL,
    ORGANIZATION_POLUCH_ID  INTEGER,
    NAME                    VARCHAR(64) NOT NULL,
    ACCOUNT_FORMAT          VARCHAR(32),
    LAST_NUMBER             INTEGER NOT NULL,
    COMMENTS                BLOB SUB_TYPE 1 SEGMENT SIZE 80,
    EDIT_USERS_ID           INTEGER
);

ALTER TABLE ACCOUNT_GROUP ADD CONSTRAINT UNQ1_ACCOUNT_GROUP UNIQUE (ORGANIZATION_POLUCH_ID, NAME);
ALTER TABLE ACCOUNT_GROUP ADD CONSTRAINT PK_ACCOUNT_GROUP PRIMARY KEY (ID);
ALTER TABLE ACCOUNT_GROUP ADD CONSTRAINT FK_ACCOUNT_GROUP_1 FOREIGN KEY (ORGANIZATION_POLUCH_ID) REFERENCES ORGANIZATION (ID) ON DELETE SET NULL;
ALTER TABLE ACCOUNT_GROUP ADD CONSTRAINT FK_ACCOUNT_GROUP_2 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;

CREATE OR ALTER TRIGGER ACCOUNT_GROUP_BI FOR ACCOUNT_GROUP
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_ACCOUNT_GROUP_ID, 1);
END;

CREATE GENERATOR GEN_ACCOUNT_ID;

CREATE TABLE ACCOUNT (
    ID                      INTEGER NOT NULL,
    USERS_ID                INTEGER NOT NULL,
    ACCOUNT_GROUP_ID        INTEGER NOT NULL,
    NUMBER                  VARCHAR(32) NOT NULL,
    DT                      TIMESTAMP NOT NULL,
    ORGANIZATION_PLATEG_ID  INTEGER,
    SUMMA_WITHNDS           DOUBLE PRECISION NOT NULL,
    SUMMA_NONDS             DOUBLE PRECISION NOT NULL,
    SUMMA_NDS               DOUBLE PRECISION NOT NULL,
    SUMMA_PAYMENTS          DOUBLE PRECISION NOT NULL,
    STATUS                  INTEGER NOT NULL,
    DAYS                    INTEGER NOT NULL,
    COMMENTS                BLOB SUB_TYPE 1 SEGMENT SIZE 80,
    EDIT_USERS_ID           INTEGER
);

ALTER TABLE ACCOUNT ADD CONSTRAINT UNQ1_ACCOUNT UNIQUE (ACCOUNT_GROUP_ID, NUMBER);
ALTER TABLE ACCOUNT ADD CONSTRAINT PK_ACCOUNT PRIMARY KEY (ID);
ALTER TABLE ACCOUNT ADD CONSTRAINT FK_ACCOUNT_1 FOREIGN KEY (ACCOUNT_GROUP_ID) REFERENCES ACCOUNT_GROUP (ID) ON DELETE CASCADE;
ALTER TABLE ACCOUNT ADD CONSTRAINT FK_ACCOUNT_2 FOREIGN KEY (ORGANIZATION_PLATEG_ID) REFERENCES ORGANIZATION (ID) ON DELETE SET NULL;
ALTER TABLE ACCOUNT ADD CONSTRAINT FK_ACCOUNT_3 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;

CREATE OR ALTER TRIGGER ACCOUNT_BI FOR ACCOUNT
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_ACCOUNT_ID, 1);
END;

CREATE GENERATOR GEN_ACCOUNT_NOMENCLATURE_ID;

CREATE TABLE ACCOUNT_NOMENCLATURE (
    ID               INTEGER NOT NULL,
    USERS_ID         INTEGER NOT NULL,
    ACCOUNT_ID       INTEGER NOT NULL,
    NOMENCLATURE_ID  INTEGER NOT NULL,
    PRICE            DOUBLE PRECISION NOT NULL,
    NOMENC_COUNT     DOUBLE PRECISION NOT NULL,
    NDS_TYPE         INTEGER NOT NULL,
    NDS              DOUBLE PRECISION NOT NULL,
    SUMMA_NDS        DOUBLE PRECISION NOT NULL,
    SUMMA_WITHNDS    DOUBLE PRECISION NOT NULL,
    SUMMA_NONDS      DOUBLE PRECISION NOT NULL
);

ALTER TABLE ACCOUNT_NOMENCLATURE ADD CONSTRAINT UNQ1_ACCOUNT_NOMENCLATURE UNIQUE (ACCOUNT_ID, NOMENCLATURE_ID);
ALTER TABLE ACCOUNT_NOMENCLATURE ADD CONSTRAINT PK_ACCOUNT_NOMENCLATURE PRIMARY KEY (ID);
ALTER TABLE ACCOUNT_NOMENCLATURE ADD CONSTRAINT FK_ACCOUNT_NOMENCLATURE_1 FOREIGN KEY (ACCOUNT_ID) REFERENCES ACCOUNT (ID) ON DELETE CASCADE;
ALTER TABLE ACCOUNT_NOMENCLATURE ADD CONSTRAINT FK_ACCOUNT_NOMENCLATURE_2 FOREIGN KEY (NOMENCLATURE_ID) REFERENCES NOMENCLATURE (ID) ON DELETE CASCADE;

CREATE OR ALTER TRIGGER ACCOUNT_NOMENCLATURE_BI FOR ACCOUNT_NOMENCLATURE
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_ACCOUNT_NOMENCLATURE_ID, 1);
END;

CREATE GENERATOR GEN_ACCOUNT_PAYMENT_ID;

CREATE TABLE ACCOUNT_PAYMENT (
    ID            INTEGER NOT NULL,
    USERS_ID      INTEGER NOT NULL,
    ACCOUNT_ID    INTEGER NOT NULL,
    SUMMA         DOUBLE PRECISION NOT NULL,
    PAYMENT_DATE  TIMESTAMP NOT NULL,
    NUMBER        INTEGER NOT NULL,
    COMMENTS      BLOB SUB_TYPE 1 SEGMENT SIZE 80
);

ALTER TABLE ACCOUNT_PAYMENT ADD CONSTRAINT UNQ1_ACCOUNT_PAYMENT UNIQUE (ACCOUNT_ID, NUMBER, PAYMENT_DATE);
ALTER TABLE ACCOUNT_PAYMENT ADD CONSTRAINT PK_ACCOUNT_PAYMENT PRIMARY KEY (ID);
ALTER TABLE ACCOUNT_PAYMENT ADD CONSTRAINT FK_ACCOUNT_PAYMENT_1 FOREIGN KEY (ACCOUNT_ID) REFERENCES ACCOUNT (ID) ON DELETE CASCADE ON UPDATE CASCADE;

CREATE OR ALTER TRIGGER ACCOUNT_PAYMENT_BI FOR ACCOUNT_PAYMENT
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_ACCOUNT_PAYMENT_ID, 1);
END;

CREATE OR ALTER PROCEDURE CLEAR_EDIT_USERS_ID (
    ID INTEGER,
    IN_OUT INTEGER,
    PROGRAM_ID INTEGER)
AS
BEGIN
  UPDATE SPAM
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE PROJECTS_STATUS_MAIN
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE REMINDER
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    EVENT_PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE_NUMBERS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    REGION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    COUNTRY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TREND
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PERSON
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    JOB_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACTIVITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TASK
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_RESULT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_SOURCE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACTIONS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  IF (IN_OUT = 1)
  THEN
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = NULL
      WHERE
        USERS_ID = :ID;
    END
  ELSE
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = :ID
      WHERE
        ID = :PROGRAM_ID;
    END

  UPDATE
    ORGANIZATION_DISCOUNT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CURRENCY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ED
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PAKAGE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CITY_DIRECT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    SMS_SPAM
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    EVENT_PROJECTS_SOURCE_PAY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACCOUNT_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACCOUNT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;
END;

EXECUTE PROCEDURE SET_VERSION('3.1.43');