ALTER TABLE CONTACT_RESULT_AUTO ADD TEXT_OPTION INTEGER NOT NULL;
ALTER TABLE CONTACT_RESULT_AUTO ADD CONTACT_USERS_ID INTEGER;
ALTER TABLE CONTACT_RESULT_AUTO ADD PRIORITY INTEGER NOT NULL;

UPDATE 
  CONTACT_RESULT_AUTO 
SET 
  PRIORITY = 1,
  TEXT_OPTION = 0;

ALTER TABLE CONTACT_RESULT_AUTO ADD CONSTRAINT UNQ1_CONTACT_RESULT_AUTO UNIQUE (CONTACT_RESULT_ID,USERS_ID,CONTACT_USERS_ID);
ALTER TABLE CONTACT_RESULT_AUTO ADD CONSTRAINT FK_CONTACT_RESULT_AUTO_3 FOREIGN KEY (CONTACT_USERS_ID) REFERENCES USERS(ID) ON DELETE CASCADE;

CREATE TABLE CONTACT_REMINDER (
    CONTACT_ID              INTEGER NOT NULL,
    REMINDER_ID             INTEGER NOT NULL,
    CONTACT_RESULT_AUTO_ID  INTEGER NOT NULL);

ALTER TABLE CONTACT_REMINDER ADD CONSTRAINT FK_CONTACT_REMINDER_1 FOREIGN KEY (CONTACT_ID) REFERENCES CONTACT(ID) ON DELETE CASCADE;
ALTER TABLE CONTACT_REMINDER ADD CONSTRAINT FK_CONTACT_REMINDER_2 FOREIGN KEY (REMINDER_ID) REFERENCES REMINDER(ID) ON DELETE CASCADE;
ALTER TABLE CONTACT_REMINDER ADD CONSTRAINT FK_CONTACT_REMINDER_3 FOREIGN KEY (CONTACT_RESULT_AUTO_ID) REFERENCES CONTACT_RESULT_AUTO(ID) ON DELETE CASCADE;

CREATE OR ALTER TRIGGER REMINDER_AIUD FOR REMINDER
ACTIVE AFTER INSERT OR UPDATE OR DELETE POSITION 0
AS
    DECLARE VARIABLE ORG_ID INTEGER;
    DECLARE VARIABLE REM_COUNT INTEGER;
BEGIN

  SELECT
    PERSON.ORGANIZATION_ID
  FROM
    PERSON
  WHERE
    (PERSON.ID = NEW.PERSON_ID) or (PERSON.ID = OLD.PERSON_ID)
  INTO
    :ORG_ID;

  SELECT
    COUNT(REMINDER.ID)
  FROM
    REMINDER
    LEFT JOIN PERSON ON PERSON.ID = REMINDER.PERSON_ID
  WHERE
    (PERSON.ORGANIZATION_ID =:ORG_ID) AND
    (REMINDER.STATUS IN (0,1,2))
  INTO
    :REM_COUNT;

  UPDATE ORGANIZATION
  SET
    ORGANIZATION.REMINDER_COUNT =:REM_COUNT
  WHERE
    ORGANIZATION.ID =:ORG_ID;

  IF ((OLD.STATUS IN (0,2)) AND (NEW.STATUS NOT IN (0,2)))
  THEN
    BEGIN
      DELETE
      FROM
        CONTACT_REMINDER
      WHERE
        REMINDER_ID = NEW.ID;
    END
END;

CREATE OR ALTER TRIGGER CONTACT_AI FOR CONTACT
ACTIVE AFTER INSERT POSITION 0
AS
    DECLARE VARIABLE CONTACT_RESULT_AUTO_ID INTEGER;
    DECLARE VARIABLE REMINDER_ID INTEGER;
    DECLARE VARIABLE DAYSLEFT INTEGER;
    DECLARE VARIABLE PRIORITY INTEGER;
    DECLARE VARIABLE USERS_ID INTEGER;
    DECLARE VARIABLE CONTACT_USERS_ID INTEGER;
    DECLARE VARIABLE TEXT_OPTION INTEGER;
    DECLARE VARIABLE REMINDER BLOB SUB_TYPE 1 SEGMENT SIZE 80;
    DECLARE VARIABLE REMINDER_COMMENT VARCHAR(2048);
    DECLARE VARIABLE DT_REMINDER TIMESTAMP;
BEGIN
  UPDATE ORGANIZATION
  SET
    ORGANIZATION.DT_LASTMODIFY = 'NOW'
  WHERE
    ORGANIZATION.ID = (SELECT PERSON.ORGANIZATION_ID
                        FROM
                            PERSON
                        WHERE
                            PERSON.ID =  NEW.PERSON_ID);

  FOR
    SELECT
      ID,
      USERS_ID,
      DAYSLEFT,
      PRIORITY,
      CONTACT_USERS_ID,
      TEXT_OPTION,
      REMINDER
    FROM
      CONTACT_RESULT_AUTO
    WHERE
      (CONTACT_RESULT_ID = NEW.CONTACT_RESULT_ID) AND
      ((CONTACT_USERS_ID IS NULL) OR (CONTACT_USERS_ID = NEW.USERS_ID))
    INTO
      :CONTACT_RESULT_AUTO_ID,
      :USERS_ID,
      :DAYSLEFT,
      :PRIORITY,
      :CONTACT_USERS_ID,
      :TEXT_OPTION,
      :REMINDER
  DO
    BEGIN
      DT_REMINDER = CURRENT_DATE + DAYSLEFT;

      IF (CONTACT_USERS_ID IS NULL)
      THEN
        CONTACT_USERS_ID = NEW.USERS_ID;

      IF (USERS_ID IS NULL)
      THEN
        USERS_ID = NEW.USERS_ID;

      REMINDER_COMMENT = SUBSTRING(REMINDER FROM 1 FOR 2048);

      IF (TEXT_OPTION = 1)
      THEN
        REMINDER_COMMENT = SUBSTRING(NEW.COMMENTS FROM 1 FOR 2048);

      IF (TEXT_OPTION = 2)
      THEN
        REMINDER_COMMENT = REMINDER_COMMENT || '
' || SUBSTRING(NEW.COMMENTS FROM 1 FOR 2048);

      REMINDER_ID = GEN_ID(GEN_REMINDER_ID, 1);

      INSERT INTO REMINDER
        (
          ID,
          PERSON_ID,
          FROM_USERS_ID,
          TO_USERS_ID,
          CONTACT_START_ID,
          CONTACT_FINAL_ID,
          DT_REMINDER,
          REMINDER,
          STATUS,
          PRIORITY
        )
      VALUES
        (
          :REMINDER_ID,
          NEW.PERSON_ID,
          :CONTACT_USERS_ID,
          :USERS_ID,
          NEW.ID,
          NULL,
          :DT_REMINDER,
          :REMINDER_COMMENT,
          0,
          :PRIORITY
        );

      INSERT INTO CONTACT_REMINDER
        (
          CONTACT_ID,
          REMINDER_ID,
          CONTACT_RESULT_AUTO_ID
        )
      VALUES
        (
          NEW.ID,
          :REMINDER_ID,
          :CONTACT_RESULT_AUTO_ID
        );

    END
END;

CREATE OR ALTER TRIGGER CONTACT_AU FOR CONTACT
ACTIVE AFTER UPDATE POSITION 0
AS
    DECLARE VARIABLE CONTACT_RESULT_AUTO_ID INTEGER;
    DECLARE VARIABLE REMINDER_ID INTEGER;
    DECLARE VARIABLE DAYSLEFT INTEGER;
    DECLARE VARIABLE PRIORITY INTEGER;
    DECLARE VARIABLE USERS_ID INTEGER;
    DECLARE VARIABLE CONTACT_USERS_ID INTEGER;
    DECLARE VARIABLE TEXT_OPTION INTEGER;
    DECLARE VARIABLE REMINDER BLOB SUB_TYPE 1 SEGMENT SIZE 80;
    DECLARE VARIABLE REMINDER_COMMENT VARCHAR(2048);
    DECLARE VARIABLE DT_REMINDER TIMESTAMP;
BEGIN
  IF (NEW.CONTACT_RESULT_ID <> OLD.CONTACT_RESULT_ID)
  THEN
    BEGIN
      /* udalim napominania */
      DELETE
      FROM
        REMINDER
      WHERE
        ID IN (SELECT REMINDER_ID FROM CONTACT_REMINDER WHERE CONTACT_ID = NEW.ID);

      DELETE
      FROM
        CONTACT_REMINDER
      WHERE
        CONTACT_ID = NEW.ID;

      FOR
        SELECT
          ID,
          USERS_ID,
          DAYSLEFT,
          PRIORITY,
          CONTACT_USERS_ID,
          TEXT_OPTION,
          REMINDER
        FROM
          CONTACT_RESULT_AUTO
        WHERE
          (CONTACT_RESULT_ID = NEW.CONTACT_RESULT_ID) AND
          ((CONTACT_USERS_ID IS NULL) OR (CONTACT_USERS_ID = NEW.USERS_ID))
        INTO
          :CONTACT_RESULT_AUTO_ID,
          :USERS_ID,
          :DAYSLEFT,
          :PRIORITY,
          :CONTACT_USERS_ID,
          :TEXT_OPTION,
          :REMINDER
      DO
        BEGIN
          DT_REMINDER = CURRENT_DATE + DAYSLEFT;
    
          IF (CONTACT_USERS_ID IS NULL)
          THEN
            CONTACT_USERS_ID = NEW.USERS_ID;

          IF (USERS_ID IS NULL)
          THEN
            USERS_ID = NEW.USERS_ID;
    
          REMINDER_COMMENT = SUBSTRING(REMINDER FROM 1 FOR 2048);

          IF (TEXT_OPTION = 1)
          THEN
            REMINDER_COMMENT = SUBSTRING(NEW.COMMENTS FROM 1 FOR 2048);

          IF (TEXT_OPTION = 2)
          THEN
            REMINDER_COMMENT = REMINDER_COMMENT || '
' || SUBSTRING(NEW.COMMENTS FROM 1 FOR 2048);

          REMINDER_ID = GEN_ID(GEN_REMINDER_ID, 1);

          INSERT INTO REMINDER
          (
              ID,
              PERSON_ID,
              FROM_USERS_ID,
              TO_USERS_ID,
              CONTACT_START_ID,
              CONTACT_FINAL_ID,
              DT_REMINDER,
              REMINDER,
              STATUS,
              PRIORITY
          )
          VALUES
          (
              :REMINDER_ID,
              NEW.PERSON_ID,
              :CONTACT_USERS_ID,
              :USERS_ID,
              NEW.ID,
              NULL, 
              :DT_REMINDER,
              :REMINDER_COMMENT,
              0,
              :PRIORITY
          );

          INSERT INTO CONTACT_REMINDER
          (
              CONTACT_ID,
              REMINDER_ID,
              CONTACT_RESULT_AUTO_ID
          )
          VALUES
          (
              NEW.ID,
              :REMINDER_ID,
              :CONTACT_RESULT_AUTO_ID
          );

        END

    END

  IF ((NEW.CONTACT_RESULT_ID = OLD.CONTACT_RESULT_ID) AND
      (NEW.COMMENTS <> OLD.COMMENTS))
  THEN
    BEGIN
      FOR
        SELECT
          CONTACT_REMINDER.REMINDER_ID,
          CONTACT_RESULT_AUTO.REMINDER,
          CONTACT_RESULT_AUTO.TEXT_OPTION
        FROM
          CONTACT_REMINDER
          LEFT JOIN CONTACT_RESULT_AUTO ON CONTACT_RESULT_AUTO.ID = CONTACT_REMINDER.CONTACT_RESULT_AUTO_ID
        WHERE
          (CONTACT_REMINDER.CONTACT_ID = NEW.ID) AND
          (CONTACT_RESULT_AUTO.TEXT_OPTION IN (1,2))
        INTO
          :REMINDER_ID,
          :REMINDER,
          :TEXT_OPTION
      DO
        BEGIN
          IF (TEXT_OPTION = 1)
          THEN
            REMINDER_COMMENT = SUBSTRING(NEW.COMMENTS FROM 1 FOR 2048);

          IF (TEXT_OPTION = 2)
          THEN
            REMINDER_COMMENT = SUBSTRING(REMINDER FROM 1 FOR 2048) || '
' || SUBSTRING(NEW.COMMENTS FROM 1 FOR 2048);

          UPDATE
            REMINDER
          SET
            REMINDER = :REMINDER_COMMENT
          WHERE
            ID = :REMINDER_ID;
        END
    END
END;

CREATE OR ALTER TRIGGER CONTACT_RESULT_AUTO_AU FOR CONTACT_RESULT_AUTO
ACTIVE AFTER UPDATE POSITION 0
AS
  DECLARE VARIABLE REMINDER_ID INTEGER;
  DECLARE VARIABLE CONTACT_COMMENT BLOB SUB_TYPE 1 SEGMENT SIZE 80;
  DECLARE VARIABLE REMINDER_COMMENT VARCHAR(2048);
BEGIN
  IF ((NEW.REMINDER <> OLD.REMINDER) OR
      (NEW.PRIORITY <> OLD.PRIORITY) OR
      (NEW.TEXT_OPTION <> OLD.TEXT_OPTION))
  THEN
    BEGIN
      FOR
        SELECT
          CONTACT_REMINDER.REMINDER_ID,
          CONTACT.COMMENTS
        FROM
          CONTACT_REMINDER
          LEFT JOIN CONTACT ON CONTACT.ID = CONTACT_REMINDER.CONTACT_ID
        WHERE
          CONTACT_RESULT_AUTO_ID = NEW.ID
        INTO
          :REMINDER_ID,
          :CONTACT_COMMENT
      DO
        BEGIN
          REMINDER_COMMENT = SUBSTRING(NEW.REMINDER FROM 1 FOR 2048);

          IF (NEW.TEXT_OPTION = 1)
          THEN
            REMINDER_COMMENT = SUBSTRING(CONTACT_COMMENT FROM 1 FOR 2048);

          IF (NEW.TEXT_OPTION = 2)
          THEN
            REMINDER_COMMENT = REMINDER_COMMENT || '
' || SUBSTRING(CONTACT_COMMENT FROM 1 FOR 2048);

          UPDATE
            REMINDER
          SET
            REMINDER = :REMINDER_COMMENT,
            PRIORITY = NEW.PRIORITY
          WHERE
            ID = :REMINDER_ID;
        END
    END
END;

EXECUTE PROCEDURE SET_VERSION('3.1.44');