CREATE OR ALTER procedure SELECT_SPAM_EMAILS
returns (
    ID integer,
    ORGANIZATION_ID integer,
    PERSON_ID integer,
    EMAIL_SEND integer,
    EMAIL varchar(64),
    ORGANIZATION_NAME varchar(128),
    BRIEF_NAME varchar(8),
    POST_INDEX varchar(16),
    ADDRESS blob sub_type 1 segment size 80,
    COUNTRY_NAME varchar(64),
    REGION_NAME varchar(64),
    CITY_NAME varchar(64),
    FIO varchar(128),
    JOB_STATUS_NAME varchar(64),
    JOBTITLE varchar(128),
    LASTNAME varchar(42),
    FIRSTNAME varchar(42),
    SECONDNAME varchar(42),
    TEL varchar(64),
    SEX integer,
    SPAM_ID integer,
    STATUS integer,
    ERROR_SMTP blob sub_type 1 segment size 80)
as
BEGIN
   PERSON_ID = 0;

    FOR
        SELECT
            SPAM_EMAILS.ID,
            SPAM_EMAILS.ORGANIZATION_ID,
            SPAM_EMAILS.PERSON_ID,
            SPAM_EMAILS.STATUS,
            SPAM_EMAILS.SPAM_ID,
            SPAM_EMAILS.ERROR_SMTP
        FROM
            SPAM_EMAILS
        INTO
            :ID,
            :ORGANIZATION_ID,
            :PERSON_ID,
            :STATUS,
            :SPAM_ID,
            :ERROR_SMTP
    DO
        BEGIN
          SELECT
            ORGANIZATION.NAME,
            ORGANIZATION.EMAIL,
            ORGANIZATION.EMAIL_SEND,
            ORGANIZATION.TEL,
            PROPERTY.BRIEF_NAME,
            POSTAL_ADDRESS.POST_INDEX,
            POSTAL_ADDRESS.ADDRESS,
            COUNTRY.NAME,
            REGION.NAME,
            CITY.NAME
          FROM
            ORGANIZATION
            LEFT JOIN PROPERTY ON PROPERTY.ID = ORGANIZATION.PROPERTY_ID
            LEFT JOIN ADDRESS POSTAL_ADDRESS ON POSTAL_ADDRESS.ID = ORGANIZATION.POSTAL_ADDRESS_ID
            LEFT JOIN COUNTRY ON COUNTRY.ID = POSTAL_ADDRESS.COUNTRY_ID
            LEFT JOIN REGION ON REGION.ID = POSTAL_ADDRESS.REGION_ID
            LEFT JOIN CITY ON CITY.ID = POSTAL_ADDRESS.CITY_ID
          WHERE
            ORGANIZATION.ID = :ORGANIZATION_ID
          INTO
            :ORGANIZATION_NAME,
            :EMAIL,
            :EMAIL_SEND,
            :TEL,
            :BRIEF_NAME,
            :POST_INDEX,
            :ADDRESS,
            :COUNTRY_NAME,
            :REGION_NAME,
            :CITY_NAME;

          IF ((PERSON_ID <> 0) AND (PERSON_ID IS NOT NULL))
          THEN
            BEGIN
              SELECT
                PERSON.JOBTITLE,
                PERSON.FIO,
                PERSON.LASTNAME,
                PERSON.FIRSTNAME,
                PERSON.SECONDNAME,
                PERSON.EMAIL,
                PERSON.EMAIL_SEND,
                PERSON.TEL_JOB,
                PERSON.SEX,
                JOB_STATUS.NAME
              FROM
                PERSON
                LEFT JOIN JOB_STATUS ON JOB_STATUS.ID = PERSON.JOB_STATUS_ID
              WHERE
                PERSON.ID = :PERSON_ID
              INTO
                :JOBTITLE,
                :FIO,
                :LASTNAME,
                :FIRSTNAME,
                :SECONDNAME,
                :EMAIL,
                :EMAIL_SEND,
                :TEL,
                :SEX,
                :JOB_STATUS_NAME;
            END
          ELSE
            BEGIN
              PERSON_ID = 0;
              JOBTITLE = NULL;
              FIO = NULL;
              LASTNAME = NULL;
              FIRSTNAME = NULL;
              SECONDNAME = NULL;
              SEX = NULL;
              JOB_STATUS_NAME = NULL;
            END

          SUSPEND;
        END
END;

EXECUTE PROCEDURE SET_VERSION('3.1.45');