CREATE GENERATOR GEN_EVENT_IPPHONE_ID;

CREATE TABLE EVENT_IPPHONE (
    ID                INTEGER NOT NULL,
    DT_EVENT          TIMESTAMP NOT NULL,
    DT_PROCESS        TIMESTAMP,
    ORGANIZATION_ID   INTEGER,
    PERSON_ID         INTEGER,
    USERS_ID_PROCESS  INTEGER,
    CALL_DIRECTION    INTEGER NOT NULL,
    CALL_STATUS       INTEGER NOT NULL,
    TEL_NUM           VARCHAR(16) NOT NULL
);

ALTER TABLE EVENT_IPPHONE ADD CONSTRAINT PK_EVENT_IPPHONE PRIMARY KEY (ID);
ALTER TABLE EVENT_IPPHONE ADD CONSTRAINT FK_EVENT_IPPHONE_1 FOREIGN KEY (ORGANIZATION_ID) REFERENCES ORGANIZATION (ID) ON DELETE SET NULL;
ALTER TABLE EVENT_IPPHONE ADD CONSTRAINT FK_EVENT_IPPHONE_2 FOREIGN KEY (PERSON_ID) REFERENCES PERSON (ID) ON DELETE SET NULL;
ALTER TABLE EVENT_IPPHONE ADD CONSTRAINT FK_EVENT_IPPHONE_3 FOREIGN KEY (USERS_ID_PROCESS) REFERENCES USERS (ID) ON DELETE SET NULL;

CREATE OR ALTER TRIGGER EVENT_IPPHONE_BI FOR EVENT_IPPHONE
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_EVENT_IPPHONE_ID, 1);
END;

create or alter procedure ADD_EVENT_IPPHONE (
    ORGANIZATION_ID integer,
    PERSON_ID integer,
    CALL_DIRECTION integer,
    USERS_PROCESS_ID integer,
    TEL_NUM varchar(16))
returns (
    ID integer,
    ORGANIZATION_INFO varchar(144),
    USER_OWNER_ID integer,
    USER_OWNER_INFO varchar(144),
    DT_EVENT timestamp)
as
declare variable CALL_STATUS integer;
declare variable DT_PROCESS timestamp;
BEGIN
  ID = -1;
  ORGANIZATION_INFO = '';
  USER_OWNER_ID = -1;
  USER_OWNER_INFO = '';

  IF (ORGANIZATION_ID > 0)
  THEN
    BEGIN
      ORGANIZATION_INFO = NULL;

      SELECT
        ORGANIZATION.ID || ' ( ' || ORGANIZATION.NAME || ' )',
        USERS.ID,
        USERS.LOGIN || ' ( ' || USERS.FIO || ' )'
      FROM
        ORGANIZATION
        LEFT JOIN USERS ON USERS.ID = ORGANIZATION.USERS_ID
      WHERE
        ORGANIZATION.ID = :ORGANIZATION_ID
      INTO
        :ORGANIZATION_INFO,
        :USER_OWNER_ID,
        :USER_OWNER_INFO;
    END

  IF (ORGANIZATION_INFO IS NOT NULL)
  THEN
    BEGIN
      ID = GEN_ID(GEN_EVENT_IPPHONE_ID, 1);

      CALL_STATUS = CALL_DIRECTION;
      DT_EVENT = 'NOW';

      IF (CALL_STATUS = 1)
      THEN
        DT_PROCESS = :DT_EVENT;
      ELSE
        DT_PROCESS = NULL;

      IF (ORGANIZATION_ID = -1)
      THEN
        ORGANIZATION_ID = NULL;

      IF (USERS_PROCESS_ID = -1)
      THEN
        USERS_PROCESS_ID = NULL;

      IF (PERSON_ID = -1)
      THEN
        PERSON_ID = NULL;

      INSERT INTO EVENT_IPPHONE
       (ID,
        DT_EVENT,
        DT_PROCESS,
        ORGANIZATION_ID,
        PERSON_ID,
        USERS_ID_PROCESS,
        CALL_DIRECTION,
        CALL_STATUS,
        TEL_NUM)
      VALUES
       (:ID,
        :DT_EVENT,
        :DT_PROCESS,
        :ORGANIZATION_ID,
        :PERSON_ID,
        :USERS_PROCESS_ID,
        :CALL_DIRECTION,
        :CALL_STATUS,
        :TEL_NUM);
    END

  SUSPEND;
END;

create or alter procedure UPDATE_EVENT_IPPHONE (
    EVENT_IPPHONE_ID integer,
    USERS_PROCESS_ID integer,
    CALL_STATUS integer)
returns (
    DT_PROCESS timestamp,
    USERS_PROCESS_INFO varchar(144))
as
declare variable OLD_CALL_STATUS integer;
begin
  OLD_CALL_STATUS = -1;
  USERS_PROCESS_INFO = '';
  DT_PROCESS = 'NOW';

  SELECT
    CALL_STATUS
  FROM
    EVENT_IPPHONE
  WHERE
    ID = :EVENT_IPPHONE_ID
  INTO
    :OLD_CALL_STATUS;

  IF (OLD_CALL_STATUS NOT IN (-1,1))
  THEN
    BEGIN

      UPDATE
        EVENT_IPPHONE
      SET
        USERS_ID_PROCESS = :USERS_PROCESS_ID,
        CALL_STATUS = :CALL_STATUS,
        DT_PROCESS = :DT_PROCESS
      WHERE
        ID = :EVENT_IPPHONE_ID;

      SELECT
        USERS.LOGIN || ' ( ' || USERS.FIO || ' )'
      FROM
        USERS
      WHERE
        USERS.ID = :USERS_PROCESS_ID
      INTO
        :USERS_PROCESS_INFO;

    END

  SUSPEND;
end;

create or alter procedure DEL_OLD_EVENT_IPPHONE (
    DT timestamp)
as
begin
  DELETE FROM
    EVENT_IPPHONE
  WHERE
    (DT_PROCESS IS NOT NULL) AND (DT_PROCESS < :DT);
end;

ALTER TABLE CONTACT ADD ORGANIZATION_ID INTEGER NOT NULL;

UPDATE
  CONTACT
SET
  CONTACT.ORGANIZATION_ID = (SELECT PERSON.ORGANIZATION_ID FROM PERSON WHERE PERSON.ID = CONTACT.PERSON_ID);

ALTER TABLE CONTACT ADD CONSTRAINT FK_CONTACT_8 FOREIGN KEY (ORGANIZATION_ID) REFERENCES ORGANIZATION(ID);

CREATE OR ALTER TRIGGER CONTACT_BI FOR CONTACT
ACTIVE BEFORE INSERT POSITION 0
AS
    DECLARE VARIABLE ORGANIZATION_ID INTEGER;
BEGIN
  IF (NEW.ID IS NULL)
  THEN
    NEW.ID = GEN_ID(GEN_CONTACT_ID, 1);

  IF (NEW.NOMENCLATURE_ID < 1)
  THEN
    NEW.NOMENCLATURE_ID = NULL;

  NEW.DOCUMENT_COUNT = 0;

  SELECT
    PERSON.ORGANIZATION_ID
  FROM
    PERSON
  WHERE
    PERSON.ID = NEW.PERSON_ID
  INTO
    :ORGANIZATION_ID;

  NEW.ORGANIZATION_ID = :ORGANIZATION_ID;
END;

CREATE OR ALTER TRIGGER PERSON_AU FOR PERSON
ACTIVE AFTER UPDATE POSITION 0
AS
begin
  if (new.ORGANIZATION_ID <> old.ORGANIZATION_ID)
  then
    begin
      UPDATE
        CONTACT
      SET
        CONTACT.ORGANIZATION_ID = NEW.ORGANIZATION_ID
      where
        CONTACT.PERSON_ID = NEW.ID;
    end
end;

CREATE OR ALTER TRIGGER CONTACT_AU FOR CONTACT
ACTIVE AFTER UPDATE POSITION 0
AS
    DECLARE VARIABLE CONTACT_RESULT_AUTO_ID INTEGER;
    DECLARE VARIABLE REMINDER_ID INTEGER;
    DECLARE VARIABLE DAYSLEFT INTEGER;
    DECLARE VARIABLE PRIORITY INTEGER;
    DECLARE VARIABLE USERS_ID INTEGER;
    DECLARE VARIABLE CONTACT_USERS_ID INTEGER;
    DECLARE VARIABLE TEXT_OPTION INTEGER;
    DECLARE VARIABLE REMINDER BLOB SUB_TYPE 1 SEGMENT SIZE 80;
    DECLARE VARIABLE REMINDER_COMMENT VARCHAR(2048);
    DECLARE VARIABLE DT_REMINDER TIMESTAMP;
BEGIN
  if (new.PERSON_ID <> old.PERSON_ID)
  then
    begin
      UPDATE
        CONTACT
      SET
        CONTACT.ORGANIZATION_ID = (SELECT PERSON.ORGANIZATION_ID FROM PERSON WHERE PERSON.ID = NEW.PERSON_ID)
      where
        CONTACT.ID = NEW.ID;
    end

  IF (NEW.CONTACT_RESULT_ID <> OLD.CONTACT_RESULT_ID)
  THEN
    BEGIN
      /* udalim napominania */
      DELETE
      FROM
        REMINDER
      WHERE
        ID IN (SELECT REMINDER_ID FROM CONTACT_REMINDER WHERE CONTACT_ID = NEW.ID);

      DELETE
      FROM
        CONTACT_REMINDER
      WHERE
        CONTACT_ID = NEW.ID;

      FOR
        SELECT
          ID,
          USERS_ID,
          DAYSLEFT,
          PRIORITY,
          CONTACT_USERS_ID,
          TEXT_OPTION,
          REMINDER
        FROM
          CONTACT_RESULT_AUTO
        WHERE
          (CONTACT_RESULT_ID = NEW.CONTACT_RESULT_ID) AND
          ((CONTACT_USERS_ID IS NULL) OR (CONTACT_USERS_ID = NEW.USERS_ID))
        INTO
          :CONTACT_RESULT_AUTO_ID,
          :USERS_ID,
          :DAYSLEFT,
          :PRIORITY,
          :CONTACT_USERS_ID,
          :TEXT_OPTION,
          :REMINDER
      DO
        BEGIN
          DT_REMINDER = CURRENT_DATE + DAYSLEFT;
    
          IF (CONTACT_USERS_ID IS NULL)
          THEN
            CONTACT_USERS_ID = NEW.USERS_ID;

          IF (USERS_ID IS NULL)
          THEN
            USERS_ID = NEW.USERS_ID;
    
          REMINDER_COMMENT = SUBSTRING(REMINDER FROM 1 FOR 2048);

          IF (TEXT_OPTION = 1)
          THEN
            REMINDER_COMMENT = SUBSTRING(NEW.COMMENTS FROM 1 FOR 2048);

          IF (TEXT_OPTION = 2)
          THEN
            REMINDER_COMMENT = REMINDER_COMMENT || '
' || SUBSTRING(NEW.COMMENTS FROM 1 FOR 2048);

          REMINDER_ID = GEN_ID(GEN_REMINDER_ID, 1);

          INSERT INTO REMINDER
          (
              ID,
              PERSON_ID,
              FROM_USERS_ID,
              TO_USERS_ID,
              CONTACT_START_ID,
              CONTACT_FINAL_ID,
              DT_REMINDER,
              REMINDER,
              STATUS,
              PRIORITY
          )
          VALUES
          (
              :REMINDER_ID,
              NEW.PERSON_ID,
              :CONTACT_USERS_ID,
              :USERS_ID,
              NEW.ID,
              NULL, 
              :DT_REMINDER,
              :REMINDER_COMMENT,
              0,
              :PRIORITY
          );

          INSERT INTO CONTACT_REMINDER
          (
              CONTACT_ID,
              REMINDER_ID,
              CONTACT_RESULT_AUTO_ID
          )
          VALUES
          (
              NEW.ID,
              :REMINDER_ID,
              :CONTACT_RESULT_AUTO_ID
          );

        END

    END

  IF ((NEW.CONTACT_RESULT_ID = OLD.CONTACT_RESULT_ID) AND
      (NEW.COMMENTS <> OLD.COMMENTS))
  THEN
    BEGIN
      FOR
        SELECT
          CONTACT_REMINDER.REMINDER_ID,
          CONTACT_RESULT_AUTO.REMINDER,
          CONTACT_RESULT_AUTO.TEXT_OPTION
        FROM
          CONTACT_REMINDER
          LEFT JOIN CONTACT_RESULT_AUTO ON CONTACT_RESULT_AUTO.ID = CONTACT_REMINDER.CONTACT_RESULT_AUTO_ID
        WHERE
          (CONTACT_REMINDER.CONTACT_ID = NEW.ID) AND
          (CONTACT_RESULT_AUTO.TEXT_OPTION IN (1,2))
        INTO
          :REMINDER_ID,
          :REMINDER,
          :TEXT_OPTION
      DO
        BEGIN
          IF (TEXT_OPTION = 1)
          THEN
            REMINDER_COMMENT = SUBSTRING(NEW.COMMENTS FROM 1 FOR 2048);

          IF (TEXT_OPTION = 2)
          THEN
            REMINDER_COMMENT = SUBSTRING(REMINDER FROM 1 FOR 2048) || '
' || SUBSTRING(NEW.COMMENTS FROM 1 FOR 2048);

          UPDATE
            REMINDER
          SET
            REMINDER = :REMINDER_COMMENT
          WHERE
            ID = :REMINDER_ID;
        END
    END
END;

EXECUTE PROCEDURE SET_VERSION('3.1.46');