ALTER TABLE ACTIVITY ADD ONLY_NEW_REMINDER INTEGER NOT NULL;

UPDATE ACTIVITY SET ONLY_NEW_REMINDER = 0;

CREATE OR ALTER procedure SEND_OLD_ORGANIZATION (
    USER_ID integer,
    SEND_USER_ID integer,
    LAST_DT_CONTACT timestamp,
    SEND_PERSON integer,
    SEND_CONTACT integer,
    ACTIVITY_ID integer,
    SEND_REMINDER integer,
    REMINDER_CREATE integer,
    REMINDER_TEXT blob sub_type 1 segment size 80,
    REMINDER_DT timestamp,
    REMINDER_PRIORITY integer,
    CONTACT_CREATE integer,
    CONTACT_COMMENTS blob sub_type 1 segment size 80,
    CONTACT_TYPE_ID integer,
    CONTACT_RESULT_ID integer)
returns (
    COUNT_SEND integer)
as
declare variable ORGANIZATION_ID integer;
declare variable PERSON_ID integer;
begin
  COUNT_SEND = 0;

  FOR
    SELECT DISTINCT
      ORGANIZATION.ID
    FROM
      ORGANIZATION
    WHERE
      (ORGANIZATION.USERS_ID = :USER_ID) AND
      ((:ACTIVITY_ID = -1) or (ORGANIZATION.ACTIVITY_ID <> :ACTIVITY_ID)) AND
      (ORGANIZATION.DT_CREATE <= :LAST_DT_CONTACT) AND
      (ORGANIZATION.ID NOT IN (SELECT
                                 PERSON.ORGANIZATION_ID
                               FROM
                                 PERSON
                                 LEFT JOIN CONTACT ON CONTACT.PERSON_ID = PERSON.ID
                               WHERE
                                (PERSON.ORGANIZATION_ID = ORGANIZATION.ID) AND
                                (CONTACT.USERS_ID = :USER_ID) AND
                                (CONTACT.DT_CONTACT > :LAST_DT_CONTACT)))
    INTO
      :ORGANIZATION_ID
  DO
    BEGIN
      COUNT_SEND = COUNT_SEND + 1;

      UPDATE
        ORGANIZATION    
      SET
        USERS_ID = :SEND_USER_ID
      WHERE
        ID = :ORGANIZATION_ID;

      IF (:SEND_PERSON = 1)
      THEN
        BEGIN
          UPDATE
            PERSON
          SET
            USERS_ID = :SEND_USER_ID
          WHERE
            (ORGANIZATION_ID = :ORGANIZATION_ID) AND
            (USERS_ID = :USER_ID);

          IF (:SEND_REMINDER = 1)
          THEN
            BEGIN
              DELETE
              FROM
                REMINDER
              WHERE
                ((REMINDER.TO_USERS_ID = :USER_ID) AND
                 (REMINDER.PERSON_ID IN (SELECT
                                           PERSON.ID
                                         FROM
                                           PERSON
                                         WHERE
                                           PERSON.ORGANIZATION_ID = :ORGANIZATION_ID)));
            END

          IF (:SEND_REMINDER = 2)
          THEN
            BEGIN
              UPDATE
                REMINDER
              SET
                REMINDER.FROM_USERS_ID = :SEND_USER_ID,
                REMINDER.TO_USERS_ID = :SEND_USER_ID
              WHERE
                ((REMINDER.FROM_USERS_ID = :USER_ID) AND
                 (REMINDER.TO_USERS_ID = :USER_ID) AND
                 (REMINDER.PERSON_ID IN (SELECT
                                           PERSON.ID
                                         FROM
                                           PERSON
                                         WHERE
                                           PERSON.ORGANIZATION_ID = :ORGANIZATION_ID)));

              UPDATE
                REMINDER
              SET
                REMINDER.TO_USERS_ID = :SEND_USER_ID
              WHERE
                ((REMINDER.TO_USERS_ID = :USER_ID) AND
                 (REMINDER.PERSON_ID IN (SELECT
                                           PERSON.ID
                                         FROM
                                           PERSON
                                         WHERE
                                           PERSON.ORGANIZATION_ID = :ORGANIZATION_ID)));
            END
        END

      IF (:SEND_CONTACT = 1)
      THEN
        BEGIN
          UPDATE
            CONTACT
          SET
            CONTACT.USERS_ID = :SEND_USER_ID
          WHERE
            (CONTACT.USERS_ID = :USER_ID) AND
            (CONTACT.PERSON_ID IN (SELECT
                                     PERSON.ID
                                   FROM
                                     PERSON
                                   WHERE
                                     PERSON.ORGANIZATION_ID = :ORGANIZATION_ID));
        END

      IF ((:CONTACT_CREATE = 1) OR (:REMINDER_CREATE = 1))
      THEN
        BEGIN
          PERSON_ID = 0;

          SELECT FIRST(1)
            ID
          FROM
            PERSON
          WHERE
            ORGANIZATION_ID = :ORGANIZATION_ID
          ORDER BY
            ID
          INTO
            :PERSON_ID;
        END

      IF ((:CONTACT_CREATE = 1) AND (:PERSON_ID > 0))
      THEN
        BEGIN
          INSERT INTO CONTACT(
            PERSON_ID,
            CONTACT_TYPE_ID,
            USERS_ID,
            EDIT_USERS_ID,
            CONTACT_RESULT_ID,
            DT_CONTACT,
            COMMENTS,
            DT_CONTACT_NEXT,
            DOCUMENT_COUNT,
            NOMENCLATURE_ID,
            USERS_LASTEDIT_ID,
            ORGANIZATION_ID)
          VALUES(
            :PERSON_ID,
            :CONTACT_TYPE_ID,
            :SEND_USER_ID,
            NULL,
            :CONTACT_RESULT_ID,
            'NOW',
            :CONTACT_COMMENTS,
            NULL,
            NULL,
            NULL,
            NULL,
           :ORGANIZATION_ID);
        END

      IF ((:REMINDER_CREATE = 1) AND (:PERSON_ID > 0))
      THEN
        BEGIN
          INSERT INTO REMINDER(
            PERSON_ID,
            FROM_USERS_ID,
            TO_USERS_ID,
            EDIT_USERS_ID,
            DT_REMINDER,
            REMINDER,
            STATUS,
            CONTACT_START_ID,
            CONTACT_FINAL_ID,
            PRIORITY)
          VALUES(
            :PERSON_ID,
            :SEND_USER_ID,
            :SEND_USER_ID,
            NULL,
            :REMINDER_DT,
            :REMINDER_TEXT,
            0,
            NULL,
            NULL,
           :REMINDER_PRIORITY);
        END
    END

  SUSPEND;
end;

CREATE OR ALTER procedure EXCHANGE_ORGANIZATION (
    ORGANIZATION_ID integer,
    OLD_OWNER_ID integer,
    NEW_OWNER_ID integer,
    PERSONS_ACTION integer,
    AGREEMENT_ACTION integer,
    DISCOUNT_ACTION integer,
    COTACTS_ACTION integer,
    REMINDER_ACTION integer,
    REMINDER_CREATE integer,
    REMINDER_TEXT blob sub_type 1 segment size 80,
    REMINDER_DT timestamp,
    REMINDER_PRIORITY integer,
    CONTACT_CREATE integer,
    CONTACT_COMMENTS blob sub_type 1 segment size 80,
    CONTACT_TYPE_ID integer,
    CONTACT_RESULT_ID integer)
as
declare variable PERSON_ID integer;
begin
  UPDATE ORGANIZATION
  SET
    USERS_ID = :NEW_OWNER_ID,
    USERS_BUSY_ID = NULL
  WHERE
    (ID = :ORGANIZATION_ID);

  IF (:PERSONS_ACTION > 0)
  THEN
    BEGIN
      UPDATE
        PERSON
      SET
        USERS_ID = :NEW_OWNER_ID
      WHERE
       (ORGANIZATION_ID = :ORGANIZATION_ID)
       AND
       ((:PERSONS_ACTION = 2) OR (USERS_ID = :OLD_OWNER_ID));


      IF (:REMINDER_ACTION = 1)
      THEN
        BEGIN
          DELETE
          FROM
            REMINDER
          WHERE
            ((REMINDER.TO_USERS_ID = :OLD_OWNER_ID) AND
            (REMINDER.PERSON_ID IN (SELECT
                                      PERSON.ID
                                    FROM
                                      PERSON
                                    WHERE
                                      PERSON.ORGANIZATION_ID = :ORGANIZATION_ID)));
        END

      IF (:REMINDER_ACTION = 2)
      THEN
        BEGIN
          UPDATE
            REMINDER
          SET
            REMINDER.FROM_USERS_ID = :NEW_OWNER_ID,
            REMINDER.TO_USERS_ID = :NEW_OWNER_ID
          WHERE
            ((REMINDER.FROM_USERS_ID = :OLD_OWNER_ID) AND
             (REMINDER.TO_USERS_ID = :OLD_OWNER_ID) AND
             (REMINDER.PERSON_ID IN (SELECT
                                       PERSON.ID
                                     FROM
                                       PERSON
                                     WHERE
                                       PERSON.ORGANIZATION_ID = :ORGANIZATION_ID)));

          UPDATE
            REMINDER
          SET
            REMINDER.TO_USERS_ID = :NEW_OWNER_ID
          WHERE
            ((REMINDER.TO_USERS_ID = :OLD_OWNER_ID) AND
             (REMINDER.PERSON_ID IN (SELECT
                                       PERSON.ID
                                     FROM
                                       PERSON
                                     WHERE
                                       PERSON.ORGANIZATION_ID = :ORGANIZATION_ID)));
        END
    END

  IF (:AGREEMENT_ACTION > 0)
  THEN
    BEGIN
      UPDATE
        AGREEMENT
      SET
        USERS_ID = :NEW_OWNER_ID
      WHERE
       (ORGANIZATION_ID = :ORGANIZATION_ID)
       AND
       ((:AGREEMENT_ACTION = 2) OR (USERS_ID = :OLD_OWNER_ID));
    END

  IF (:DISCOUNT_ACTION > 0)
  THEN
    BEGIN
      UPDATE
        ORGANIZATION_DISCOUNT
      SET
        USERS_ID = :NEW_OWNER_ID
      WHERE
       (ORGANIZATION_ID = :ORGANIZATION_ID)
       AND
       ((:DISCOUNT_ACTION = 2) OR (USERS_ID = :OLD_OWNER_ID));
    END

  IF (:COTACTS_ACTION > 0)
  THEN
    BEGIN
      UPDATE
        CONTACT
      SET
        USERS_ID = :NEW_OWNER_ID
      WHERE
       (PERSON_ID IN (SELECT ID FROM PERSON WHERE ORGANIZATION_ID = :ORGANIZATION_ID))
       AND
       ((:COTACTS_ACTION = 2) OR (USERS_ID = :OLD_OWNER_ID));
    END


  IF ((:CONTACT_CREATE = 1) OR (:REMINDER_CREATE = 1))
  THEN
    BEGIN
      PERSON_ID = 0;

      SELECT FIRST(1)
        ID
      FROM
        PERSON
      WHERE
        ORGANIZATION_ID = :ORGANIZATION_ID
      ORDER BY
        ID
      INTO
        :PERSON_ID;
    END

  IF ((:CONTACT_CREATE = 1) AND (:PERSON_ID > 0))
  THEN
    BEGIN
      INSERT INTO CONTACT(
        PERSON_ID,
        CONTACT_TYPE_ID,
        USERS_ID,
        EDIT_USERS_ID,
        CONTACT_RESULT_ID,
        DT_CONTACT,
        COMMENTS,
        DT_CONTACT_NEXT,
        DOCUMENT_COUNT,
        NOMENCLATURE_ID,
        USERS_LASTEDIT_ID,
        ORGANIZATION_ID)
      VALUES(
        :PERSON_ID,
        :CONTACT_TYPE_ID,
        :NEW_OWNER_ID,
        NULL,
        :CONTACT_RESULT_ID,
        'NOW',
        :CONTACT_COMMENTS,
        NULL,
        NULL,
        NULL,
        NULL,
       :ORGANIZATION_ID);
     END

  IF ((:REMINDER_CREATE = 1) AND (:PERSON_ID > 0))
  THEN
    BEGIN
      INSERT INTO REMINDER(
        PERSON_ID,
        FROM_USERS_ID,
        TO_USERS_ID,
        EDIT_USERS_ID,
        DT_REMINDER,
        REMINDER,
        STATUS,
        CONTACT_START_ID,
        CONTACT_FINAL_ID,
        PRIORITY)
      VALUES(
        :PERSON_ID,
        :NEW_OWNER_ID,
        :NEW_OWNER_ID,
        NULL,
        :REMINDER_DT,
        :REMINDER_TEXT,
        0,
        NULL,
        NULL,
        :REMINDER_PRIORITY);
    END
end;

create or alter procedure ADD_ADDRESS (
    POST_INDEX varchar(16),
    COUNTRY_ID integer,
    REGION_ID integer,
    CITY_ID integer,
    ADDRESS blob sub_type 1 segment size 80)
returns (
    ID integer)
as
begin
  if (:POST_INDEX = '')
  then
    POST_INDEX = NULL;

  if (:COUNTRY_ID < 1)
  then
    COUNTRY_ID = NULL;

  if (:REGION_ID < 1)
  then
    REGION_ID = NULL;

  if (:CITY_ID < 1)
  then
    CITY_ID = NULL;

  if (:ADDRESS = '')
  then
    ADDRESS = NULL;

  ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:ID,
     :POST_INDEX,
     :COUNTRY_ID,
     :REGION_ID,
     :CITY_ID,
     :ADDRESS);
  suspend;
end;

EXECUTE PROCEDURE SET_VERSION('3.1.47');