ALTER TABLE ORGANIZATION ADD BUH_TAX_SYSTEM_MAIN  INTEGER;
ALTER TABLE ORGANIZATION ADD BUH_TAX_SYSTEM_ADDIT INTEGER;
ALTER TABLE ORGANIZATION ADD BUH_ACTIVITY         VARCHAR(64);
ALTER TABLE ORGANIZATION ADD BUH_INET_BANK        VARCHAR(128);
ALTER TABLE ORGANIZATION ADD BUH_BASE             VARCHAR(64);
ALTER TABLE ORGANIZATION ADD BUH_ACCOUNT_TAX      VARCHAR(64);
ALTER TABLE ORGANIZATION ADD BUH_ACCOUNT_BUH      VARCHAR(64);
ALTER TABLE ORGANIZATION ADD BUH_ACCOUNT_OTHER    VARCHAR(64);
ALTER TABLE ORGANIZATION ADD BUH_KASSA            VARCHAR(64);
ALTER TABLE ORGANIZATION ADD BUH_BANK_PAYORDER    VARCHAR(64);
ALTER TABLE ORGANIZATION ADD BUH_BANK_REPORT      VARCHAR(64);
ALTER TABLE ORGANIZATION ADD BUH_ADVANCE          VARCHAR(64);
ALTER TABLE ORGANIZATION ADD BUH_REALIZATION      VARCHAR(64);
ALTER TABLE ORGANIZATION ADD BUH_MATERIAL         VARCHAR(64);
ALTER TABLE ORGANIZATION ADD BUH_GOODS            VARCHAR(64);
ALTER TABLE ORGANIZATION ADD BUH_OUTSOURCE        VARCHAR(64);
ALTER TABLE ORGANIZATION ADD BUH_ZARPLATA         VARCHAR(64);
ALTER TABLE ORGANIZATION ADD BUH_PERSONNEL        VARCHAR(64);
ALTER TABLE ORGANIZATION ADD  BUH_OTHER           VARCHAR(256);

CREATE GENERATOR GEN_BUH_HOLIDAY_ID;

CREATE TABLE BUH_HOLIDAY (
    ID             INTEGER NOT NULL,
    NAME           VARCHAR(64) NOT NULL,
    DAY_MONTH      INTEGER NOT NULL,
    MONTH_YEAR     INTEGER NOT NULL,
    HOLIDAY_WORK   INTEGER NOT NULL,
    EDIT_USERS_ID  INTEGER
);

ALTER TABLE BUH_HOLIDAY ADD CONSTRAINT UNQ1_BUH_HOLIDAY UNIQUE (DAY_MONTH, MONTH_YEAR);
ALTER TABLE BUH_HOLIDAY ADD CONSTRAINT UNQ2_BUH_HOLIDAY UNIQUE (NAME);
ALTER TABLE BUH_HOLIDAY ADD CONSTRAINT PK_BUH_HOLIDAY PRIMARY KEY (ID);
ALTER TABLE BUH_HOLIDAY ADD CONSTRAINT FK_BUH_HOLIDAY_1 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;

CREATE OR ALTER TRIGGER BUH_HOLIDAY_BI FOR BUH_HOLIDAY
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL) 
  THEN
    NEW.ID = GEN_ID(GEN_BUH_HOLIDAY_ID,1);
END;

CREATE GENERATOR GEN_BUH_ACCOUNT_ID;

CREATE TABLE BUH_ACCOUNT (
    ID             INTEGER NOT NULL,
    NAME           VARCHAR(256) NOT NULL,
    DAY_ACTION     INTEGER NOT NULL,
    CYCLING        INTEGER NOT NULL,
    DAY_WARNING    INTEGER NOT NULL,
    DEL            INTEGER NOT NULL,
    EDIT_USERS_ID  INTEGER
);

ALTER TABLE BUH_ACCOUNT ADD CONSTRAINT UNQ1_BUH_ACCOUNT UNIQUE (NAME);
ALTER TABLE BUH_ACCOUNT ADD CONSTRAINT PK_BUH_ACCOUNT PRIMARY KEY (ID);
ALTER TABLE BUH_ACCOUNT ADD CONSTRAINT FK_BUH_ACCOUNT_1 FOREIGN KEY (EDIT_USERS_ID) REFERENCES USERS (ID) ON DELETE SET NULL;

CREATE OR ALTER TRIGGER BUH_ACCOUNT_BI FOR BUH_ACCOUNT
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL) 
  THEN
    NEW.ID = GEN_ID(GEN_BUH_ACCOUNT_ID,1);
END;

CREATE GENERATOR GEN_BUH_ACCOUNT_ORGANIZATION_ID;

CREATE TABLE BUH_ACCOUNT_ORGANIZATION (
    ID               INTEGER NOT NULL,
    BUH_ACCOUNT_ID   INTEGER NOT NULL,
    ORGANIZATION_ID  INTEGER NOT NULL,
    ON_OFF           INTEGER NOT NULL
);

ALTER TABLE BUH_ACCOUNT_ORGANIZATION ADD CONSTRAINT UNQ1_BUH_ACCOUNT_ORGANIZATION UNIQUE (BUH_ACCOUNT_ID, ORGANIZATION_ID);
ALTER TABLE BUH_ACCOUNT_ORGANIZATION ADD CONSTRAINT PK_BUH_ACCOUNT_ORGANIZATION PRIMARY KEY (ID);
ALTER TABLE BUH_ACCOUNT_ORGANIZATION ADD CONSTRAINT FK_BUH_ACCOUNT_ORGANIZATION_1 FOREIGN KEY (BUH_ACCOUNT_ID) REFERENCES BUH_ACCOUNT (ID) ON DELETE CASCADE;
ALTER TABLE BUH_ACCOUNT_ORGANIZATION ADD CONSTRAINT FK_BUH_ACCOUNT_ORGANIZATION_2 FOREIGN KEY (ORGANIZATION_ID) REFERENCES ORGANIZATION (ID) ON DELETE CASCADE;

CREATE OR ALTER TRIGGER BUH_ACCOUNT_ORGANIZATION_BI FOR BUH_ACCOUNT_ORGANIZATION
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL) 
  THEN
    NEW.ID = GEN_ID(GEN_BUH_ACCOUNT_ORGANIZATION_ID,1);
END;

CREATE GENERATOR GEN_BUH_HISTORY_ID;

CREATE TABLE BUH_HISTORY (
    ID               INTEGER NOT NULL,
    BUH_ACCOUNT_ID   INTEGER NOT NULL,
    ORGANIZATION_ID  INTEGER NOT NULL,
    CYCLING          INTEGER NOT NULL,
    DT_END_ACCOUNT   TIMESTAMP NOT NULL,
    DT_ACTION        TIMESTAMP NOT NULL,
    DT_ACTUAL        TIMESTAMP,
    DT_ADD_RECORD    TIMESTAMP,
    COMMENTS         BLOB SUB_TYPE 1 SEGMENT SIZE 80,
    USERS_ID         INTEGER
);

ALTER TABLE BUH_HISTORY ADD CONSTRAINT PK_BUH_HISTORY PRIMARY KEY (ID);
ALTER TABLE BUH_HISTORY ADD CONSTRAINT FK_BUH_HISTORY_1 FOREIGN KEY (BUH_ACCOUNT_ID) REFERENCES BUH_ACCOUNT (ID) ON DELETE CASCADE;
ALTER TABLE BUH_HISTORY ADD CONSTRAINT FK_BUH_HISTORY_2 FOREIGN KEY (ORGANIZATION_ID) REFERENCES ORGANIZATION (ID) ON DELETE CASCADE;
ALTER TABLE BUH_HISTORY ADD CONSTRAINT UNQ1_BUH_HISTORY UNIQUE (BUH_ACCOUNT_ID,ORGANIZATION_ID,CYCLING,DT_END_ACCOUNT);

CREATE OR ALTER TRIGGER BUH_HISTORY_BI FOR BUH_HISTORY
ACTIVE BEFORE INSERT POSITION 0
AS
BEGIN
  IF (NEW.ID IS NULL) 
  THEN
    NEW.ID = GEN_ID(GEN_BUH_HISTORY_ID,1);
END;

CREATE OR ALTER procedure CLEAR_EDIT_USERS_ID (
    ID integer,
    IN_OUT integer,
    PROGRAM_ID integer)
as
BEGIN
  UPDATE SPAM
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE PROJECTS_STATUS_MAIN
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE REMINDER
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PROJECTS_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    EVENT_PROJECTS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    DOCUMENT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE_NUMBERS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    REGION
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    COUNTRY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TREND
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PERSON
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    JOB_STATUS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACTIVITY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    TASK
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CONTACT_RESULT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_TYPE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ORGANIZATION_SOURCE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACTIONS
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  IF (IN_OUT = 1)
  THEN
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = NULL
      WHERE
        USERS_ID = :ID;
    END
  ELSE
    BEGIN
      UPDATE
        PROGRAM
      SET
        USERS_ID = :ID
      WHERE
        ID = :PROGRAM_ID;
    END

  UPDATE
    ORGANIZATION_DISCOUNT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    NOMENCLATURE_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CURRENCY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ED
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    PAKAGE
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    CITY_DIRECT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    SMS_SPAM
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    EVENT_PROJECTS_SOURCE_PAY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACCOUNT_GROUP
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    ACCOUNT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    BUH_ACCOUNT
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;

  UPDATE
    BUH_HOLIDAY
  SET
    EDIT_USERS_ID = NULL
  WHERE
    EDIT_USERS_ID = :ID;
END;

CREATE OR ALTER procedure ADD_ORGANIZATION (
    USERS_ID integer,
    NAME varchar(128),
    TEL varchar(64),
    FAX varchar(64),
    EMAIL varchar(64),
    WEB varchar(64),
    HOURS integer,
    REQ_INN varchar(16),
    REQ_KPP varchar(16),
    REQ_RACCOUNT varchar(32),
    REQ_OKPO varchar(16),
    REQ_OKVED varchar(128),
    REQ_BIK varchar(16),
    REQ_BANK varchar(128),
    REQ_KACCOUNT varchar(32),
    ACTIVITY_ID integer,
    PROPERTY_ID integer,
    ORGANIZATION_TYPE integer,
    CHECKED integer,
    EMAIL_SEND integer,
    CONFIG blob sub_type 1 segment size 80,
    COMMENTS blob sub_type 1 segment size 80,
    ACTUAL_POST_INDEX varchar(16),
    LEGAL_POST_INDEX varchar(16),
    POSTAL_POST_INDEX varchar(16),
    ACTUAL_ADDRESS blob sub_type 1 segment size 80,
    LEGAL_ADDRESS blob sub_type 1 segment size 80,
    POSTAL_ADDRESS blob sub_type 1 segment size 80,
    ACTUAL_COUNTRY_ID integer,
    ACTUAL_REGION_ID integer,
    ACTUAL_CITY_ID integer,
    LEGAL_COUNTRY_ID integer,
    LEGAL_REGION_ID integer,
    LEGAL_CITY_ID integer,
    POSTAL_COUNTRY_ID integer,
    POSTAL_REGION_ID integer,
    POSTAL_CITY_ID integer,
    X integer,
    Y integer,
    SUMMA_XY integer,
    ORGANIZATION_GROUP_ID integer,
    ARM_COUNT integer,
    PERSON_COUNT integer,
    CONTRACT_SUMMA double precision,
    ORGANIZATION_SOURCE_ID integer,
    CONTRACT_CURRENCY_ID integer,
    CITY_DIRECT_ID integer,
    KM double precision,
    NAME_ALIAS varchar(256),
    BUH_TAX_SYSTEM_MAIN integer,
    BUH_TAX_SYSTEM_ADDIT integer,
    BUH_ACTIVITY varchar(64),
    BUH_INET_BANK varchar(128),
    BUH_BASE varchar(64),
    BUH_ACCOUNT_TAX varchar(64),
    BUH_ACCOUNT_BUH varchar(64),
    BUH_ACCOUNT_OTHER varchar(64),
    BUH_KASSA varchar(64),
    BUH_BANK_PAYORDER varchar(64),
    BUH_BANK_REPORT varchar(64),
    BUH_ADVANCE varchar(64),
    BUH_REALIZATION varchar(64),
    BUH_MATERIAL varchar(64),
    BUH_GOODS varchar(64),
    BUH_OUTSOURCE varchar(64),
    BUH_ZARPLATA varchar(64),
    BUH_PERSONNEL varchar(64),
    BUH_OTHER varchar(256))
returns (
    ERROR integer,
    ID integer)
as
declare variable ACTUAL_ADDRESS_ID integer;
declare variable LEGAL_ADDRESS_ID integer;
declare variable POSTAL_ADDRESS_ID integer;
BEGIN
  ERROR = -1;
  ID = -1;
  /****************************************************************************/
  IF ((:ACTUAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :ACTUAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 1; 
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_COUNTRY_ID = 0)
  THEN
    ACTUAL_COUNTRY_ID = NULL;

  IF ((:LEGAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :LEGAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 2; 
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_COUNTRY_ID = 0)
  THEN
    LEGAL_COUNTRY_ID = NULL;

  IF ((:POSTAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :POSTAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 3; 
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_COUNTRY_ID = 0)
  THEN
    POSTAL_COUNTRY_ID = NULL;

  IF ((:ACTUAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :ACTUAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 4; 
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_REGION_ID = 0)
  THEN
    ACTUAL_REGION_ID = NULL;

  IF ((:LEGAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :LEGAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 5; 
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_REGION_ID = 0)
  THEN
    LEGAL_REGION_ID = NULL;

  IF ((:POSTAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :POSTAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 6; 
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_REGION_ID = 0)
  THEN
    POSTAL_REGION_ID = NULL;

  IF ((:ACTUAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :ACTUAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 7; 
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_CITY_ID = 0)
  THEN
    ACTUAL_CITY_ID = NULL;

  IF ((:LEGAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :LEGAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 8; 
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_CITY_ID = 0)
  THEN
    LEGAL_CITY_ID = NULL;

  IF ((:POSTAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :POSTAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 9; 
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_CITY_ID = 0)
  THEN
    POSTAL_CITY_ID = NULL;


 /* IF (EXISTS (SELECT ID FROM ORGANIZATION WHERE NAME = :NAME))
  THEN
    BEGIN
      ERROR = 10;  
      SUSPEND;
      EXIT;
    END
  */
  IF ((:PROPERTY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM PROPERTY WHERE ID = :PROPERTY_ID)))
  THEN
    BEGIN
        ERROR = 11;
        SUSPEND;
        EXIT;
    END

  IF (:PROPERTY_ID = 0)
  THEN
    PROPERTY_ID = NULL;

  IF ((:ACTIVITY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM ACTIVITY WHERE ID = :ACTIVITY_ID)))
  THEN
    BEGIN
        ERROR = 12;
        SUSPEND;
        EXIT;
    END

  IF (:ACTIVITY_ID = 0)
  THEN
    ACTIVITY_ID = NULL;

  IF (:ORGANIZATION_SOURCE_ID = 0)
  THEN
    ORGANIZATION_SOURCE_ID = NULL;

  IF (:CITY_DIRECT_ID = 0)
  THEN
    CITY_DIRECT_ID = NULL;

  /*******************************************************************/
  ACTUAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:ACTUAL_ADDRESS_ID,
     :ACTUAL_POST_INDEX,
     :ACTUAL_COUNTRY_ID,
     :ACTUAL_REGION_ID,
     :ACTUAL_CITY_ID,
     :ACTUAL_ADDRESS);
  /*******************************************************************/
  LEGAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:LEGAL_ADDRESS_ID,
     :LEGAL_POST_INDEX,
     :LEGAL_COUNTRY_ID,
     :LEGAL_REGION_ID,
     :LEGAL_CITY_ID,
     :LEGAL_ADDRESS);
  /*******************************************************************/
  POSTAL_ADDRESS_ID = GEN_ID(GEN_ADDRESS_ID, 1);
  INSERT INTO ADDRESS
    (ID,
     POST_INDEX,
     COUNTRY_ID,
     REGION_ID,
     CITY_ID,
     ADDRESS)
  VALUES
    (:POSTAL_ADDRESS_ID,
     :POSTAL_POST_INDEX,
     :POSTAL_COUNTRY_ID,
     :POSTAL_REGION_ID,
     :POSTAL_CITY_ID,
     :POSTAL_ADDRESS);
  /*********************************************************************/
  ID = GEN_ID(GEN_ORGANIZATION_ID, 1);
  INSERT INTO ORGANIZATION
    (ID, 
    USERS_ID,
    ACTUAL_ADDRESS_ID,
    LEGAL_ADDRESS_ID,
    POSTAL_ADDRESS_ID,
    NAME,
    TEL,
    FAX,
    EMAIL,
    WEB,
    HOURS,
    REQ_INN,
    REQ_KPP,
    REQ_RACCOUNT,
    REQ_OKPO,
    REQ_OKVED,
    REQ_BIK,
    REQ_BANK,
    REQ_KACCOUNT,
    ACTIVITY_ID,
    REMINDER_COUNT,
    DT_CREATE,
    DT_LASTMODIFY,
    ORGANIZATION_TYPE,
    CHECKED,
    EMAIL_SEND,
    CONFIG,
    COMMENTS,
    PROPERTY_ID,
    X,
    Y,
    SUMMA_XY,
    ORGANIZATION_GROUP_ID,
    ARM_COUNT,
    PERSON_COUNT,
    CONTRACT_SUMMA,
    ORGANIZATION_SOURCE_ID,
    CONTRACT_CURRENCY_ID,
    USERS_BUSY_ID,
    CITY_DIRECT_ID,
    KM,
    NAME_ALIAS,
    BUH_TAX_SYSTEM_MAIN,
    BUH_TAX_SYSTEM_ADDIT,
    BUH_ACTIVITY,
    BUH_INET_BANK,
    BUH_BASE,
    BUH_ACCOUNT_TAX,
    BUH_ACCOUNT_BUH,
    BUH_ACCOUNT_OTHER,
    BUH_KASSA,
    BUH_BANK_PAYORDER,
    BUH_BANK_REPORT,
    BUH_ADVANCE,
    BUH_REALIZATION,
    BUH_MATERIAL,
    BUH_GOODS,
    BUH_OUTSOURCE,
    BUH_ZARPLATA,
    BUH_PERSONNEL,
    BUH_OTHER
    )
  VALUES
    (:ID,
    :USERS_ID,
    :ACTUAL_ADDRESS_ID,
    :LEGAL_ADDRESS_ID,
    :POSTAL_ADDRESS_ID,
    :NAME,
    :TEL,
    :FAX,
    :EMAIL,
    :WEB,
    :HOURS,
    :REQ_INN,
    :REQ_KPP,
    :REQ_RACCOUNT,
    :REQ_OKPO,
    :REQ_OKVED,
    :REQ_BIK,
    :REQ_BANK,
    :REQ_KACCOUNT,
    :ACTIVITY_ID,
    0,
    'NOW',
    'NOW',
    :ORGANIZATION_TYPE,
    :CHECKED,
    :EMAIL_SEND,
    :CONFIG,
    :COMMENTS,
    :PROPERTY_ID,
    :X,
    :Y,
    :SUMMA_XY,
    :ORGANIZATION_GROUP_ID,
    :ARM_COUNT,
    :PERSON_COUNT,
    :CONTRACT_SUMMA,
    :ORGANIZATION_SOURCE_ID,
    :CONTRACT_CURRENCY_ID,
    NULL,
    :CITY_DIRECT_ID,
    :KM,
    :NAME_ALIAS,
    :BUH_TAX_SYSTEM_MAIN,
    :BUH_TAX_SYSTEM_ADDIT,
    :BUH_ACTIVITY,
    :BUH_INET_BANK,
    :BUH_BASE,
    :BUH_ACCOUNT_TAX,
    :BUH_ACCOUNT_BUH,
    :BUH_ACCOUNT_OTHER,
    :BUH_KASSA,
    :BUH_BANK_PAYORDER,
    :BUH_BANK_REPORT,
    :BUH_ADVANCE,
    :BUH_REALIZATION,
    :BUH_MATERIAL,
    :BUH_GOODS,
    :BUH_OUTSOURCE,
    :BUH_ZARPLATA,
    :BUH_PERSONNEL,
    :BUH_OTHER
    );
  /****************************************************************************/
  ERROR = 0;
  SUSPEND;
END;

CREATE OR ALTER procedure EDIT_ORGANIZATION (
    ID integer,
    NAME varchar(128),
    TEL varchar(64),
    FAX varchar(64),
    EMAIL varchar(64),
    WEB varchar(64),
    HOURS integer,
    REQ_INN varchar(16),
    REQ_KPP varchar(16),
    REQ_RACCOUNT varchar(32),
    REQ_OKPO varchar(16),
    REQ_OKVED varchar(128),
    REQ_BIK varchar(16),
    REQ_BANK varchar(128),
    REQ_KACCOUNT varchar(32),
    ACTIVITY_ID integer,
    ORGANIZATION_TYPE integer,
    CHECKED integer,
    EMAIL_SEND integer,
    CONFIG blob sub_type 1 segment size 80,
    COMMENTS blob sub_type 1 segment size 80,
    ACTUAL_POST_INDEX varchar(16),
    LEGAL_POST_INDEX varchar(16),
    POSTAL_POST_INDEX varchar(16),
    ACTUAL_ADDRESS blob sub_type 1 segment size 80,
    LEGAL_ADDRESS blob sub_type 1 segment size 80,
    POSTAL_ADDRESS blob sub_type 1 segment size 80,
    ACTUAL_COUNTRY_ID integer,
    ACTUAL_REGION_ID integer,
    ACTUAL_CITY_ID integer,
    LEGAL_COUNTRY_ID integer,
    LEGAL_REGION_ID integer,
    LEGAL_CITY_ID integer,
    POSTAL_COUNTRY_ID integer,
    POSTAL_REGION_ID integer,
    POSTAL_CITY_ID integer,
    A_ADDRESS_ID integer,
    L_ADDRESS_ID integer,
    P_ADDRESS_ID integer,
    PROPERTY_ID integer,
    X integer,
    Y integer,
    SUMMA_XY integer,
    ORGANIZATION_GROUP_ID integer,
    ARM_COUNT integer,
    PERSON_COUNT integer,
    CONTRACT_SUMMA double precision,
    ORGANIZATION_SOURCE_ID integer,
    CONTRACT_CURRENCY_ID integer,
    CITY_DIRECT_ID integer,
    KM double precision,
    NAME_ALIAS varchar(256),
    BUH_TAX_SYSTEM_MAIN integer,
    BUH_TAX_SYSTEM_ADDIT integer,
    BUH_ACTIVITY varchar(64),
    BUH_INET_BANK varchar(128),
    BUH_BASE varchar(64),
    BUH_ACCOUNT_TAX varchar(64),
    BUH_ACCOUNT_BUH varchar(64),
    BUH_ACCOUNT_OTHER varchar(64),
    BUH_KASSA varchar(64),
    BUH_BANK_PAYORDER varchar(64),
    BUH_BANK_REPORT varchar(64),
    BUH_ADVANCE varchar(64),
    BUH_REALIZATION varchar(64),
    BUH_MATERIAL varchar(64),
    BUH_GOODS varchar(64),
    BUH_OUTSOURCE varchar(64),
    BUH_ZARPLATA varchar(64),
    BUH_PERSONNEL varchar(64),
    BUH_OTHER varchar(256))
returns (
    ERROR integer)
as
BEGIN
  ERROR = -1;
  /****************************************************************************/
  IF ((:ACTUAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :ACTUAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 1;
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_COUNTRY_ID = 0)
  THEN
    ACTUAL_COUNTRY_ID = NULL;

  IF ((:LEGAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :LEGAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 2;
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_COUNTRY_ID = 0)
  THEN
    LEGAL_COUNTRY_ID = NULL;

  IF ((:POSTAL_COUNTRY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM COUNTRY WHERE ID = :POSTAL_COUNTRY_ID)))
  THEN
    BEGIN
      ERROR = 3;
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_COUNTRY_ID = 0)
  THEN
    POSTAL_COUNTRY_ID = NULL;

  IF ((:ACTUAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :ACTUAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 4;
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_REGION_ID = 0)
  THEN
    ACTUAL_REGION_ID = NULL;

  IF ((:LEGAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :LEGAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 5;
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_REGION_ID = 0)
  THEN
    LEGAL_REGION_ID = NULL;

  IF ((:POSTAL_REGION_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM REGION WHERE ID = :POSTAL_REGION_ID)))
  THEN
    BEGIN
      ERROR = 6;
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_REGION_ID = 0)
  THEN
    POSTAL_REGION_ID = NULL;

  IF ((:ACTUAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :ACTUAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 7;
      SUSPEND;
      EXIT;
    END

  IF (:ACTUAL_CITY_ID = 0)
  THEN
    ACTUAL_CITY_ID = NULL;

  IF ((:LEGAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :LEGAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 8;
      SUSPEND;
      EXIT;
    END

  IF (:LEGAL_CITY_ID = 0)
  THEN
    LEGAL_CITY_ID = NULL;

  IF ((:POSTAL_CITY_ID <> 0) AND
     (NOT EXISTS (SELECT ID FROM CITY WHERE ID = :POSTAL_CITY_ID)))
  THEN
    BEGIN
      ERROR = 9;
      SUSPEND;
      EXIT;
    END

  IF (:POSTAL_CITY_ID = 0)
  THEN
    POSTAL_CITY_ID = NULL;


  IF ((:PROPERTY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM PROPERTY WHERE ID = :PROPERTY_ID)))
  THEN
    BEGIN
        ERROR = 10;
        SUSPEND;
        EXIT;
    END

  IF (:PROPERTY_ID = 0)
  THEN
    PROPERTY_ID = NULL;

  IF ((:ACTIVITY_ID <> 0) AND
      (NOT EXISTS (SELECT ID FROM ACTIVITY WHERE ID = :ACTIVITY_ID)))
  THEN
    BEGIN
        ERROR = 11; 
        SUSPEND;
        EXIT;
    END

  IF (:ACTIVITY_ID = 0)
  THEN
    ACTIVITY_ID = NULL;

  IF (:ORGANIZATION_SOURCE_ID = 0)
  THEN
    ORGANIZATION_SOURCE_ID = NULL;

  IF (:CITY_DIRECT_ID = 0)
  THEN
    CITY_DIRECT_ID = NULL;

  /*******************************************************************/

  /*******************************************************************/
  UPDATE ADDRESS
    SET
        POST_INDEX = :ACTUAL_POST_INDEX,
        COUNTRY_ID = :ACTUAL_COUNTRY_ID,
        REGION_ID = :ACTUAL_REGION_ID,
        CITY_ID =:ACTUAL_CITY_ID,
        ADDRESS = :ACTUAL_ADDRESS
     WHERE
        ID = :A_ADDRESS_ID;

  /*******************************************************************/
  UPDATE ADDRESS
    SET
        POST_INDEX = :LEGAL_POST_INDEX,
        COUNTRY_ID = :LEGAL_COUNTRY_ID,
        REGION_ID = :LEGAL_REGION_ID,
        CITY_ID =:LEGAL_CITY_ID,
        ADDRESS = :LEGAL_ADDRESS
     WHERE
        ID = :L_ADDRESS_ID;
  /*********************************************************************/
  UPDATE ADDRESS
    SET
        POST_INDEX = :POSTAL_POST_INDEX,
        COUNTRY_ID = :POSTAL_COUNTRY_ID,
        REGION_ID = :POSTAL_REGION_ID,
        CITY_ID =:POSTAL_CITY_ID,
        ADDRESS = :POSTAL_ADDRESS
     WHERE
        ID = :P_ADDRESS_ID;
  /*********************************************************************/
  UPDATE ORGANIZATION
    SET
        NAME =: NAME,
        TEL =: TEL,
        FAX =: FAX,
        EMAIL =: EMAIL,
        WEB =: WEB,
        HOURS =: HOURS,
        REQ_INN =: REQ_INN,
        REQ_KPP =: REQ_KPP,
        REQ_RACCOUNT =: REQ_RACCOUNT,
        REQ_OKPO =: REQ_OKPO,
        REQ_OKVED =: REQ_OKVED,
        REQ_BIK =: REQ_BIK,
        REQ_BANK =: REQ_BANK,
        REQ_KACCOUNT =: REQ_KACCOUNT,
        ACTIVITY_ID =: ACTIVITY_ID,
        PROPERTY_ID =: PROPERTY_ID,
        DT_LASTMODIFY = 'NOW',
        ORGANIZATION_TYPE =:ORGANIZATION_TYPE,
        CHECKED =:CHECKED,
        EMAIL_SEND =: EMAIL_SEND,
        CONFIG =: CONFIG,
        COMMENTS =:COMMENTS,
        X = :X,
        Y = :Y,
        SUMMA_XY = :SUMMA_XY,
        organization_group_id = :organization_group_id,
        ARM_COUNT = :ARM_COUNT,
        PERSON_COUNT = :PERSON_COUNT,
        CONTRACT_SUMMA = :CONTRACT_SUMMA,
        ORGANIZATION_SOURCE_ID = :ORGANIZATION_SOURCE_ID,
        CONTRACT_CURRENCY_ID = :CONTRACT_CURRENCY_ID,
        CITY_DIRECT_ID = :CITY_DIRECT_ID,
        KM = :KM,
        NAME_ALIAS = :NAME_ALIAS,

        BUH_TAX_SYSTEM_MAIN = :BUH_TAX_SYSTEM_MAIN,
        BUH_TAX_SYSTEM_ADDIT = :BUH_TAX_SYSTEM_ADDIT,
        BUH_ACTIVITY = :BUH_ACTIVITY,
        BUH_INET_BANK = :BUH_INET_BANK,
        BUH_BASE = :BUH_BASE,
        BUH_ACCOUNT_TAX = :BUH_ACCOUNT_TAX,
        BUH_ACCOUNT_BUH = :BUH_ACCOUNT_BUH,
        BUH_ACCOUNT_OTHER = :BUH_ACCOUNT_OTHER,
        BUH_KASSA = :BUH_KASSA,
        BUH_BANK_PAYORDER = :BUH_BANK_PAYORDER,
        BUH_BANK_REPORT = :BUH_BANK_REPORT,
        BUH_ADVANCE = :BUH_ADVANCE,
        BUH_REALIZATION = :BUH_REALIZATION,
        BUH_MATERIAL = :BUH_MATERIAL,
        BUH_GOODS = :BUH_GOODS,
        BUH_OUTSOURCE = :BUH_OUTSOURCE,
        BUH_ZARPLATA = :BUH_ZARPLATA,
        BUH_PERSONNEL = :BUH_PERSONNEL,
        BUH_OTHER = :BUH_OTHER
     WHERE
        ID =:ID ;

  /****************************************************************************/
  ERROR = 0;
  SUSPEND;
END;

CREATE OR ALTER TRIGGER BUH_ACCOUNT_AU FOR BUH_ACCOUNT
ACTIVE AFTER UPDATE POSITION 0
AS
BEGIN
  IF ((OLD.DEL = 0) AND (NEW.DEL = 1))
  THEN
    BEGIN
      DELETE
      FROM
        BUH_ACCOUNT_ORGANIZATION
      WHERE
        BUH_ACCOUNT_ID = NEW.ID;

      DELETE
      FROM
        BUH_HISTORY
      WHERE
        ((BUH_HISTORY.BUH_ACCOUNT_ID = NEW.ID) AND
         (BUH_HISTORY.DT_ACTUAL IS NULL) AND
         (BUH_HISTORY.DT_ACTION > 'NOW'));
    END
END;

CREATE OR ALTER TRIGGER BUH_ACCOUNT_ORGANIZATION_AU FOR BUH_ACCOUNT_ORGANIZATION
ACTIVE AFTER UPDATE POSITION 0
AS
BEGIN
  IF ((OLD.ON_OFF = 1) AND (NEW.ON_OFF = 0))
  THEN
    BEGIN
      DELETE
      FROM
        BUH_HISTORY
      WHERE
        ((BUH_HISTORY.BUH_ACCOUNT_ID = NEW.BUH_ACCOUNT_ID) AND
         (BUH_HISTORY.ORGANIZATION_ID = NEW.ORGANIZATION_ID) AND
         (BUH_HISTORY.DT_ACTUAL IS NULL) AND
         (BUH_HISTORY.DT_ACTION > 'NOW'));
    END
END;

CREATE OR ALTER procedure ADD_BUH_HISTORY (
    BUH_ACCOUNT_ID integer,
    ORGANIZATION_ID integer,
    CYCLING integer,
    DT_END_ACCOUNT timestamp,
    DT_ACTION timestamp)
as
declare variable BUH_HISTORY_ID integer;
declare variable DT_ACTUAL timestamp;
begin
  BUH_HISTORY_ID = NULL;
  DT_ACTUAL = NULL;

  SELECT
    ID,
    DT_ACTUAL
  FROM
    BUH_HISTORY
  WHERE
    (BUH_ACCOUNT_ID = :BUH_ACCOUNT_ID) AND
    (ORGANIZATION_ID = :ORGANIZATION_ID) AND
    (CYCLING = :CYCLING) AND
    (DT_END_ACCOUNT = :DT_END_ACCOUNT)
  INTO
    :BUH_HISTORY_ID,
    :DT_ACTUAL;

  IF (:BUH_HISTORY_ID IS NULL)
  THEN
    BEGIN
      BUH_HISTORY_ID = GEN_ID(GEN_BUH_HISTORY_ID, 1);

      INSERT INTO
      BUH_HISTORY(
        ID,
        BUH_ACCOUNT_ID,
        ORGANIZATION_ID,
        CYCLING,
        DT_END_ACCOUNT,
        DT_ACTION,
        DT_ACTUAL,
        DT_ADD_RECORD,
        COMMENTS,
        USERS_ID)
      VALUES(
        :BUH_HISTORY_ID,
        :BUH_ACCOUNT_ID,
        :ORGANIZATION_ID,
        :CYCLING,
        :DT_END_ACCOUNT,
        :DT_ACTION,
        NULL,
        NULL,
        NULL,
        NULL);
    END
  ELSE
    BEGIN
      IF (:DT_ACTUAL IS NULL)
      THEN
        BEGIN
          UPDATE
            BUH_HISTORY
          SET
            DT_ACTION = :DT_ACTION
          WHERE
            ID = :BUH_HISTORY_ID;
        END
    END
end;

CREATE OR ALTER TRIGGER BUH_ACCOUNT_ORGANIZATION_AD FOR BUH_ACCOUNT_ORGANIZATION
ACTIVE AFTER DELETE POSITION 0
AS
begin
  DELETE
  FROM
    BUH_HISTORY
  WHERE
    ((BUH_HISTORY.BUH_ACCOUNT_ID = OLD.BUH_ACCOUNT_ID) AND
     (BUH_HISTORY.ORGANIZATION_ID = OLD.ORGANIZATION_ID) AND
     (BUH_HISTORY.DT_ACTUAL IS NULL) AND
     (BUH_HISTORY.DT_ACTION > 'NOW'));
end;

CREATE TRIGGER BUH_HISTORY_BU FOR BUH_HISTORY
ACTIVE BEFORE UPDATE POSITION 0
AS
begin
  if (NEW.DT_ACTUAL IS NULL)
  then
    NEW.DT_ADD_RECORD = NULL;
  else
    if (OLD.DT_ADD_RECORD IS NULL)
    then
      NEW.DT_ADD_RECORD = 'NOW';
end;

EXECUTE PROCEDURE SET_VERSION('3.1.48');